#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

import os
import sys
import logging
import logging.config
import traceback
import commands
import re
import subprocess
import threading
import time as timeonly
from datetime import datetime, date, time, timedelta


## Working Dir for suppoprting FMAR Integration, it can be run from any folder.
strStoreBinDir = ""
PLATFORM_LINUX = 'linux'
PLATFORM_WINDOWS = 'windows'


debugLog = 0
'''
loggerFlat object is set here, __name__ is the module name appended to log file.
'''
loggerflat = logging.getLogger(__name__)

'''
Initializing the logger here. This will write data to log file. The filename attribute is set with logger filename. The file will be stored in the 'log' foldedr in the current working dir. When run as standalone tool, logging is stored in the file debug_log_filepathname, but in case of integration with FMAR it is stored in FMAR logs.
'format' attribute is set to have timestamp in the log file. This logger has the details when run FLAT tool independently. When run with FMAR the debug logs are updated in FMAR log.
'''
def init_logger(moduleName):
    #logging.basicConfig(filename='logs/flatlogger.log', filemode='w', level=logging.DEBUG, format='%(asctime)s [%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s')
    debug_log_filepathname = getCurrentWorkingDir() + "/logs/flatlogger.log"
    logging.basicConfig(filename=debug_log_filepathname, filemode='w', level=logging.DEBUG, format='%(asctime)s %(message)s')


'''
print_on_debug_and_log : This method is used to print the message on console and write to log file. If 'debugLog' variable is set than message gets printed on the console else it only writes to log file.
msg - The input message which is printed on console and/or written to the file.
logType - It is set with the type of message to be logged. Default it is set with info, it can be set with info, critical, warning, error and debug types. In the log file the message is appended with the type set here.
'''
def print_on_debug_and_log(msg, logType = loggerflat.info):
    try:
        if 'Loggerflat.info' in str(logType):
            if debugLog == 1:
                print msg
            loggerflat.info(bcolors.bcolorsRemove(msg))
        elif 'Loggerflat.warn' in str(logType):
            if debugLog == 1:
                print colortext(msg,bcolors.WARNING)
            loggerflat.warn(bcolors.bcolorsRemove(msg))
        elif 'Loggerflat.critical' in str(logType) or 'Loggerflat.error' in str(logType):
            if debugLog == 1:
                print colortext(msg,bcolors.FAIL)
            loggerflat.critical(bcolors.bcolorsRemove(msg))
        else:
            if debugLog == 1:
                print msg
            loggerflat.debug(bcolors.bcolorsRemove(msg))
            return True
    except Exception,e:
        return False


'''
print_on_console_and_log_summary : This method is used to print the message on console and write to summary file.
mode - This is required to open a file in write mode or append mode.
msg - The input message which is printed on console and/or written to the file.
'''
def print_on_console_and_log_summary(file_path_name, mode, msg):
    try:
        #print msg
        file_handle = fileOpen(file_path_name, mode)
        if file_handle is not None:
            fileWriteLine(file_handle, bcolors.bcolorsRemove(msg))
            fileClose(file_handle)
        return True
    except:
        return False


def isLinux():
    if getPlatform() == PLATFORM_LINUX:
        return True
    else:
        return False

def getPlatform():
    _platform = sys.platform
    if _platform == "linux" or _platform == "linux2":
        return PLATFORM_LINUX
    elif _platform == "win32":
        return PLATFORM_WINDOWS


'''
colortext : It returns the message in the colored format. The message input font color is changed to input color.
'''
def colortext(printText, color):
    if isLinux():
        return ("%s%s%s" %(color, printText, bcolors.ENDC))
    else:
        return printText

'''
colortext2 : It returns the message in the colored format. The message input font color is changed to input color. Here the input color is more than one for one line
'''
def colortext2(printTextColor1, color1, printTextColor2, color2):
    if isLinux():
        return ("%s%s%s%s%s" %(color1, printTextColor1, color2, printTextColor2, bcolors.ENDC))
    else:
        return printTextColor1 + printTextColor2

'''
executeCLICommand : This method is used to execute the input command. The command should be input to the function. It return a tuple, which first value at 0th location is set with 0 or non zero values. 0 indicates the command run was success else not. The second value is set with the output that command returned.If the output is redirected to a file than location 1 is set with empty string.
'''
def executeCLICommand(stringCommandFormat):
    try:
        return commands.getstatusoutput(stringCommandFormat)
    except:
        return None

##### Progress Bar code ###########
'''
tobj: Used globally to store the ID for starting and stopping the progress.
'''
tobj = None
'''
stopTimer: True/False, Used globally so that the start method can use it to stop while loop, when it is set.
'''
stopTimer = False

'''
showProgressBarInternal : It displays the pattern on the console, so that it appears rotating at one place. The numbers used here are random and only for the specific need.
'''
def showProgressBarInternal(it, prefix = "", size = 60):
    try:
        global stopTimer
        count = len(it)
        def _show(_i):
            _pattern = ['-', '\\', '|', '/']
            period = 1
            _delay = float(float(period) / (2*len(_pattern)))
            x = int(size*_i/count)
            #sys.stdout.write("%s[%s%s] %i/%i\r" % (prefix, "#"*x, "."*(size-x), _i, count))
            for symbol in _pattern:
                sys.stdout.write('%s ' % (symbol))
                sys.stdout.flush()
                timeonly.sleep(_delay)
                sys.stdout.write('\b\b')

        _show(0)
        for i, item in enumerate(it):
            yield item
            _show(i+1)
        sys.stdout.write(' \b')
        sys.stdout.write("\n")
        sys.stdout.flush()
    except:
        stopTimer = True
        return

'''
startProgressInternalNoBoundary : This calls the showProgressBarInternal method in a endless loop, until stop is issued by the caller. If stopTimer is set to false this loop should break. Here the limit is not known i.e. when the progress bar will stop and thus while loop is used.
msg - Input string to be displayed as prefix with the progress bar.
'''
def startProgressInternalNoBoundary(msg):
    try:
        global stopTimer
        while stopTimer != True:
            try:
                for i in showProgressBarInternal(range(40), msg, 40):
                    timeonly.sleep(1) # long computation
                    if stopTimer == True:
                        break
            except:
                stopTimer = True
                return
    except:
        stopTimer = True
        return

'''
startProgressBar : The caller can call this method to start the progress bar. Time delay second input parameter is the time before strating the progress bar, by default it is set with 2 sec dealy.
msg - Input string to be displayed as prefix with the progress bar.
'''
def startProgressBar(msg, timedelayBeforeStartingTheProgress = 2.0):
    try:
        global tobj
        global stopTimer
        stopTimer = False
        tobj = threading.Timer(timedelayBeforeStartingTheProgress, startProgressInternalNoBoundary, [msg])
        tobj.start()
    except:
        tobj.cancel()
        stopTimer = True
        return
'''
stopProgressBar : The caller can call this method to stop the running progress bar.
'''
def stopProgressBar():
    try:
        global tobj
        global stopTimer
        stopTimer = True
        tobj.cancel()
        sys.stdout.write("\r\n")
    except:
        return
########## Progress bar code ends ###########


'''
getLastNDaysDates : This method is used to get previous two days from the input date. It return three dates in the list.
latest_day_for_analysis - Input date from which next 3 dates would be returned. 3 days is default input.
nDays - Input in Days to check the log N days back, i.e. from latest to N days events in the logs will be analyzed. 
'''
def getLastNDaysDates(latest_day_for_analysis, nDays):
    try:
        listofDays = []
        if latest_day_for_analysis is '' or nDays <= 0:
            return None
        stringFormat = latest_day_for_analysis #+ " " + str(datetime.now().year)
        # Get date from input string, the date input is should be in the format as specified below.
        d = datetime.strptime(stringFormat, "%Y/%m/%d")
        if d is not None:
            # Insert first date
            listofDays.insert(0, d.strftime("%Y/%m/%d"))
            # insert second date after deducting one day.
            for i in xrange(1, nDays):
                d = d - timedelta(days=+1)
                # insert third date after deducting one more day.
                listofDays.append(d.strftime("%Y/%m/%d"))
        return listofDays
    except:
        return None


'''
getLast72HrsDatesWithoutYear : This method is used to get previous two days from the input date. It return three dates in the list.
latest_day_for_analysis - Input date from which next 3 dates would be returned. 
'''
def getLast72HrsDatesWithoutYear(latest_day_for_analysis):
    try:
        listofDays = []
        if latest_day_for_analysis is '':
            return None
        stringFormat = latest_day_for_analysis #+ " " + str(datetime.now().year)
        # Get date from input string, the date input is should be in the format as specified below.
        d = datetime.strptime(stringFormat, "%Y/%m/%d")
        if d is not None:
            # Insert first date
            listofDays.insert(0, d.strftime("%b %d").replace(' 0', '  '))
            # insert second date after deducting one day.
            d = d - timedelta(days=+1)
            listofDays.append(d.strftime("%b %d").replace(' 0', '  '))
            # insert third date after deducting one more day.
            d = d - timedelta(days=+1)
            listofDays.append(d.strftime("%b %d").replace(' 0', '  '));
        return listofDays
    except:
        return None


'''
getLast24HrsDates : This method is used to return string as date in fixed format 
latest_day_for_analysis - Input date in string format 
'''
def getLast24HrsDates(latest_day_for_analysis):
    try:

        stringFormat = latest_day_for_analysis
        # Get date from input string
        d = datetime.strptime(stringFormat, "%Y/%m/%d")
        if d is not None:
            return d.strftime("%Y/%m/%d")
        return None
    except:
        return None
'''
setCurrentBinDir : Set the directory of binary for accessing the XML, temp logs in config, etc when run from any other dir.
'''
def setCurrentBinDir(strBinaryDir):
    global strStoreBinDir
    strStoreBinDir = strBinaryDir
    return

'''
getCurrentWorkingDir : Return the current working dir of the tool
'''
def getCurrentWorkingDir():
    global strStoreBinDir
    if not strStoreBinDir:
        return os.getcwd()
    return strStoreBinDir

'''
changeDir : Change the working dir to the input dir_path_name.
dir_path_name - Holds the dir which has to be changed to. 
'''
def changeDir(dir_path_name):
    os.chdir( dir_path_name )
    return


'''
fileOpen : Open a file and returns the file handler. If any exception returns None
filePathName - Set with the name of the file to be opened.
filemode - Set with the mode withe read "r" or write "w" mode.
'''
def fileOpen(filePathName, fileMode):
    try:
        fileHandle = open(filePathName, fileMode)
        if fileHandle is None:
            return None
        return fileHandle
    except:
        print_on_debug_and_log("Failed to open file(%s) with mode(%s).", filePathName, fileMode)
        return None
    


'''
fileClose : Close the file.
fileHandle - Handle to the file to be closed.
'''
def fileClose(fileHandle):
    try:
        if fileHandle is not None:
            fileHandle.close()
    except:
        return 

'''
fileReadLines : Reads the entire file content and returns in the list. Return None in case of exception.
fileHandle - Handle to the file which has to be read.
'''
def fileReadLines(fileHandle):
    try:
        if fileHandle is not None:
            return fileHandle.readlines()
    except:
        return None
    return None

'''
fileReadLine(fileHandle) : Read only single line and returns.
fileHandle -  Handle to the file
'''
def fileReadLine(fileHandle):
    try:
         if fileHandle is not None:
             return  fileHandle.readline()
    except:
        return None
    return None

'''
fileWriteLine : Write in the file hold by file handle. The input message is written to the file.
fileHandle - Handle to the file which has to be written.
message - Data that is to be written in file.
'''
def fileWriteLine(fileHandle, message):
    try:
        fileHandle.write(message)
        return True
    except:
        return False

'''
fileRemove : Remove the file passed as input
FilePathName - The file path name to be removed
'''
def fileRemove(FilePathName):
    try:
        if FilePathName is not '':
            os.remove(FilePathName)
    except:
        return

def createLogDir(folderName):
    try:
        if not os.path.exists(folderName):
            os.mkdir( folderName, 0777 );
    except:
        return

'''
checkIfFileExists : This method checks if a file exists or not.
filePathName - File path name to be checked for existence
'''
def checkIfFileExists(filePathName):
    return os.path.exists(filePathName)

'''
validateIPAddressString - Validates if the Input is IPV4 address range.
strIPAddress - Input IP string 
'''
def validateIPAddressString(strIPAddress):
    if strIPAddress is '' or strIPAddress is None:
        return False
    try:

        match = re.match("^(\d{0,3})\.(\d{0,3})\.(\d{0,3})\.(\d{0,3})$", strIPAddress)
        if not match:
            return False
        if int(match.group(1)) == 0:
            return False
        for i in xrange(1, 5):
            if int(match.group(i)) < 0 or  int(match.group(i)) > 255:
                return False
        return True
    except:
        common_calls.print_on_debug_and_log("Failed in common module: In validateIPAddressString(). Error (%s)" % sys.exc_info()[0])
        return False

'''
checkIfDirectorIsReachable : If the log files has to be analyzed from director, the nsfw files has to be imported locally. For this tool need to connect to remote director(s) to fetch the file. Before this fetching of file, check if the IP is valid and reachable. This is required to avaoid SSH timeouts for a unreachable IP during file fethcing process. 
'''
def checkIfDirectorIsReachable(strIPAddress):
    try:
        psub = subprocess.Popen(['ping', '-c 2', strIPAddress], stdout=subprocess.PIPE)
        for message in psub.stdout:
            result = message.strip().decode('UTF-8')
            if "Host Unreachable" in result or "Ping request could not find host" in result or "Request timed out" in result or "100% packet loss" in result:
                return False
        return True
    except:
        common_calls.print_on_debug_and_log("Failed in common module: In checkIfDirectorIsReachable(). Error (%s)" % sys.exc_info()[0])
        common_calls.print_on_debug_and_log(traceback.format_exc())
        return False



'''
class bcolors : Set with the different colors. This is used throughout the program. 
'''
class bcolors:
    # For headers this color is set.
    HEADER = '\033[95m'
    # Blue color 
    OKBLUE = '\033[94m'
    # Green color
    OKGREEN = '\033[92m'
    # Wanrning color
    WARNING = '\033[93m'
    # Fail color
    FAIL = '\033[91m'
    # End color
    ENDC = '\033[0m'
    
    @staticmethod
    def bcolorsRemove(colorText):
        try:
            removeedColorText = colorText.strip(bcolors.HEADER).strip(bcolors.OKBLUE).strip(bcolors.OKGREEN).strip(bcolors.WARNING).strip(bcolors.FAIL).strip(bcolors.ENDC)
            return removeedColorText
        except:
            return None


