#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

#Imports libs
import os
import sys
import traceback
import warnings

if not sys.warnoptions:
    warnings.simplefilter("ignore")


#import local modules
from common import common_calls as common_calls
from common.common_calls import bcolors
from modules import xmlParserModule as XMLP
from modules import logFileParser as logFileP
from modules import handleDirectorFirmwareLogs as handleDirectorFiles
from modules import summaryReport as summaryReport



'''
class flatBaseModule : The main class which binds all the modules. This module start method has to be called to run the Tool analysis.
'''
class flatBaseModule :
    
    m_filePathNameDict = {}
    m_isRunAsStandAlone = True
    m_logSummaryReportPath = ''
    m_productVersion = ''
    def __init__(self, firmwareLogFilePathList = None, isRunAsStandAlone = True, logSummarryReportFilePathName = None, director_ip = None, product_version = None):
	
	ncount = 0
        self.m_productVersion = ''
        if product_version:
            self.m_productVersion = product_version

	if firmwareLogFilePathList:
	    # This code can be simpler, but to match the XML flow and same code should run from Upgrade checker tool and standalone.
	    for eachfilepath in firmwareLogFilePathList:
		temp_dict = {}
		temp_dict["FilePath"] = eachfilepath
       	 	self.m_filePathNameDict[ncount] = temp_dict
		ncount = ncount + 1
	else:
	    self.m_filePathNameDict = {}	
	if logSummarryReportFilePathName:
	    self.m_logSummaryReportPath = logSummarryReportFilePathName
	else:
	    self.m_logSummaryReportPath = ''
	self.m_isRunAsStandAlone = isRunAsStandAlone
	if director_ip is not None:
	    for eachdirector_ip in director_ip:
		    temp_dict = {}
		    #<FilePath use_from_director="True" director-ip="128.221.252.35">/var/log/nsfw.log</FilePath>
		    temp_dict["use_from_director"] = True
		    temp_dict["director-ip"] = eachdirector_ip
		    temp_dict["FilePath"] = '/var/log/nsfw.log'
		    self.m_filePathNameDict[ncount] = temp_dict
		    ncount = ncount + 1


    '''
        start : main call this method, which calls the XML parsing module, enumerate through each event and display the data on the console. 
    '''
    def start(self):
        try:

            # Change the work dir to tool path, store the oldpath and restore it back at the end.
            script_exec_dir = common_calls.getCurrentWorkingDir()
            script_path_dir = os.path.split(os.path.realpath(__file__))[0]
            common_calls.changeDir(script_path_dir)
            #initialize the logger:
            common_calls.init_logger("FLAT Tool")
            # Initailize XMLParser module class object.
            moduleParserHandle = XMLP.xmlParserModule()
            # Initialize log file parser object
            logfileobj = logFileP.firmwareLogFileParser()
            # Initialize Display summary class object.
            moduleDisplaySummaryHandle = None
	    displayNameOnConsole = 'Firmware'
            if self.m_logSummaryReportPath is not '':
		moduleDisplaySummaryHandle = summaryReport.summaryReport(self.m_logSummaryReportPath)
	    else:
	    	moduleDisplaySummaryHandle = summaryReport.summaryReport()
            filePathName = {}
	    # Get path name of the log file to be analyzeid
	    if self.m_filePathNameDict or not self.m_isRunAsStandAlone:
                if len(self.m_filePathNameDict) > 1:
                    filePathName = handleDirectorFiles.toMergeEventLogFileForNDays(self.m_filePathNameDict)
                else:
                    filePathName = self.m_filePathNameDict
	    else:    
		filePathName = moduleParserHandle.getFirwmareTagFilePath()

	    if not filePathName:
		print("Error! Firmware/NSFW log file path is empty. Unable to analyze Firmware/NSFW log file.")
		common_calls.print_on_debug_and_log("Error! Firmware/NSFW log file path is empty. Unable to analyze Firmware/NSFW log file.")
		return
	    # Get the current product version from the XML file
            if not self.m_productVersion or self.m_productVersion == '':
                productbaseVersion = moduleParserHandle.getCurrentproductBaseVersion()
            else:
                productbaseVersion = self.m_productVersion
            toolVersion = moduleParserHandle.getToolVersion()
            lastNhrsTimeValue = moduleParserHandle.getLastNhrsValue()
            if not lastNhrsTimeValue or int(lastNhrsTimeValue) <= 0 or int(lastNhrsTimeValue) > 720:
                lastNhrsTimeValue = 72 #Pick from config file else use 3 days by default
            common_calls.print_on_debug_and_log("Analyze the logs for last (%s) days." % lastNhrsTimeValue)
            
            #If log file to be analyzed is not set or empty, do not continue further.
            if filePathName is None or len(filePathName) <= 0:
                ## To give proper error/exit message
                moduleDisplaySummaryHandle.displayXMLParseError()
                return -1
            flagFileParseSuccess = False
            moduleDisplaySummaryHandle.startLoggingSummaryReport(toolVersion)
            bool_analyze_firmware_logs = True # False for directors nsfw logs. Requried to verify if firmware log or nsfw log is not going back 3 days.
            bool_display_file_name_once = True
 
            # Enumerate through log file list.
            for logfilepathnamedictkey in filePathName.keys():
                logfilepathnamedict = filePathName.get(logfilepathnamedictkey, None)
                if logfilepathnamedict is None:
                    ## To give proper error/exit message
                    moduleDisplaySummaryHandle.displayXMLParseError()
                    #return -1
                    continue

                logfilepathname = logfilepathnamedict.get("FilePath", '')
                # If log file to be analyzed is not set or empty, do not continue further.
                if logfilepathname is None or logfilepathname is '':
                    ## To give proper error/exit message
                    moduleDisplaySummaryHandle.displayXMLParseError()
                    #return -1
                    continue
		if bool_display_file_name_once:
			if logfilepathnamedict.has_key("director-ip") is True:
			    director_ip = logfilepathnamedict.get("director-ip", '')
			    displayNameOnConsole = 'NSFW'
                            moduleDisplaySummaryHandle.dispalyFileNameOnConsole(displayNameOnConsole)
			else:
                            if self.m_isRunAsStandAlone:
			        moduleDisplaySummaryHandle.dispalyFileNameOnConsole(logfilepathname)
                            else:
                                moduleDisplaySummaryHandle.dispalyFileNameOnConsole(displayNameOnConsole)
			bool_display_file_name_once = False
                    
                if common_calls.checkIfFileExists(logfilepathname) == False and logfilepathnamedict.has_key("director-ip") is not True:
                    #print '1' * 100
                    ## To give proper error/exit message
		    if self.m_isRunAsStandAlone:
                    	moduleDisplaySummaryHandle.displayLogFileNotFoundError(logfilepathname)
                    continue
                    #return -1
                #print '2' * 100
                
                
                common_calls.startProgressBar(common_calls.colortext(" Analyzing firmware log : ", bcolors.OKGREEN))
                # Create temporary files once in the tool runtime
                # Check if the firmware files need to be from directors, the files from director needs to be formated as per firmware log.
                director_ip = logfilepathnamedict.get("director-ip", '')
		if director_ip is not '':
		    bool_analyze_firmware_logs = False
                    if len(director_ip) == 0:
                        #print "IPAddress is empty in XML file. Check XML file for a fix."
                        moduleDisplaySummaryHandle.displayDirectorRelatedErrors(1, '', '')
                        continue
                    if common_calls.validateIPAddressString(director_ip) is False:
                        #print "IPAddress(%s) for director has invalid IPV4 format." % director_ip
                        moduleDisplaySummaryHandle.displayDirectorRelatedErrors(2, logfilepathname, director_ip)
                        continue
                    # Check if IP is pingable, before getting log file from director. directly doing SSH on unreachable IP will take lot of time in timeout.
                    if common_calls.checkIfDirectorIsReachable(director_ip) is False:
                        #print "Director at IPAddress(%s) is unreachable." % director_ip
                        moduleDisplaySummaryHandle.displayDirectorRelatedErrors(3, logfilepathname, director_ip)
                        continue
                    if handleDirectorFiles.tocreateEventLogFileForNDays(director_ip, logfilepathname) is False:
                        moduleDisplaySummaryHandle.displayDirectorRelatedErrors(4, logfilepathname, director_ip)
                        common_calls.print_on_debug_and_log("Failed! Getting file(%s) from director (%s)" % (logfilepathname, director_ip))
                        # If entire firmware log file has no warning events, display the below message
                        #moduleDisplaySummaryHandle.displayTextifNoEventAlarming(script_path_dir)
                        #moduleDisplaySummaryHandle.displayLogFileParserError(director_ip)
                        continue
                    logfileobj.setLatestEventDate(handleDirectorFiles.strLatestEventDate)
                    #print logfilepathname 
                    temp_director_filename = common_calls.getCurrentWorkingDir() + format("/logs/FLAT_TEMP_LOG_FILE_72HRS_%s.TXT" % director_ip) 
                    if logfileobj.createEventLogFileForNDays(temp_director_filename, lastNhrsTimeValue, 2) is False:#filepath and name
                        #common_calls.print_on_debug_and_log("Failed! Parsing file for (%s)" % director_ip)
                        moduleDisplaySummaryHandle.displayLogFileParserError(director_ip)
                        #moduleDisplaySummaryHandle.displayTextifNoEventAlarming(script_path_dir)
                        common_calls.fileRemove(temp_director_filename)
                        continue
                    common_calls.fileRemove(temp_director_filename)
                else:
                    #print logfilepathname 
                    if logfileobj.createEventLogFileForNDays(logfilepathname, lastNhrsTimeValue, 2) is False:#filepath and name
                        #common_calls.print_on_debug_and_log("Failed! Parsing file(%s)" % logfilepathname)
                        #moduleDisplaySummaryHandle.displayLogFileParserError(logfilepathname)
                        #moduleDisplaySummaryHandle.displayTextifNoEventAlarming(script_path_dir)
                        if logfileobj.getIfLogFileGoesBackToNdays() is False and bool_analyze_firmware_logs is True:
                            moduleDisplaySummaryHandle.displaySMSEventRelatedWarning()
                            bool_analyze_firmware_logs = False
                            # Clean up the temporary files created for an event.
                            logfileobj.cleanTempLogFiles('')
                            # Clean up temporary files created for all events
                            logfileobj.cleanTempLogFilesFinal()
			    bool_display_file_name_once = True
                        continue
                if logfileobj.getIfLogFileGoesBackToNdays() is False and bool_analyze_firmware_logs is True:
                    if not self.m_isRunAsStandAlone:
                        moduleDisplaySummaryHandle.displaySMSEventRelatedWarning()
                        bool_analyze_firmware_logs = False
                        # Clean up the temporary files created for an event.
                        logfileobj.cleanTempLogFiles('')
                        # Clean up temporary files created for all events
                        logfileobj.cleanTempLogFilesFinal()
                        bool_display_file_name_once = True
                        continue
                    else:
                        moduleDisplaySummaryHandle.displayWarningNotGoingBackNDays(lastNhrsTimeValue)
                flagFileParseSuccess = True
           	if bool_analyze_firmware_logs and not self.m_isRunAsStandAlone:
			break
 
            common_calls.stopProgressBar()
            
            if flagFileParseSuccess is False:
                moduleDisplaySummaryHandle.displayLogFileParserError("firmware/nsfw")
                # Clean up temporary files created for all events
                logfileobj.cleanTempLogFilesFinal()
                #Revert the dir to the script calling directory.
                common_calls.changeDir(script_exec_dir)
                return

            #Check if one or all of the nsfw logs do not go back 3 days than prompt a warning
            if logfileobj.getIfLogFileGoesBackToNdays() is False and bool_analyze_firmware_logs is False:
                moduleDisplaySummaryHandle.displayNFSWRelatedWarning()
            
            moduleDisplaySummaryHandle.setprintInfo(True)
            # Get all the events in the XML file
            allDictFromXML = moduleParserHandle.getDictEventWithAllData()

            # Get all the events for the supported VPLEX version
            allDictFortheCurrentVersion = moduleParserHandle.getDictEventListForCurrentVersion(productbaseVersion)
            
            #empty dict should be ignored 
            if allDictFortheCurrentVersion is None or len(allDictFortheCurrentVersion) == 0:
                moduleDisplaySummaryHandle.displayXMLParseError()
                #return -1
            flagNoEventFound = 0

            #Enumerate through all the events in the dictionary.
            for keys in allDictFortheCurrentVersion.keys():
                common_calls.print_on_debug_and_log(keys)
                # Get dictinary details of the event in key.
                dicts = allDictFortheCurrentVersion.get(keys, None)
                logfilePathName = logfilepathname
                
                common_calls.print_on_debug_and_log("logfilePathName %s" % logfilePathName)
                common_calls.print_on_debug_and_log ("Event Name " + dicts.get("Name", ""))
                # Get name of event from the dictionary
                eventName = dicts.get("Name", "")

                # If event name is not found, continue to look for another event.
                if eventName is '':
                    continue

                common_calls.print_on_debug_and_log("Event Status " + dicts.get("Status", ""))
                # Get status of an event to be checked in the log
                eventStatus = dicts.get("Status", "")
                common_calls.print_on_debug_and_log("Event Includes" + dicts.get("EventIncludes", ""))
                # Get includes for an event
                eventIncludes = dicts.get("EventIncludes", "")
                common_calls.print_on_debug_and_log("Event Excludes " + dicts.get("ExcludeIfFound", ""))
                # Get exclude list for an event
                eventExcludes = dicts.get("ExcludeIfFound", "")
                common_calls.print_on_debug_and_log("Secondary Event Name " + dicts.get("secondaryEventName", ""))
                #Get secondary event name
                secondaryEventName = dicts.get("secondaryEventName", "")
                common_calls.print_on_debug_and_log("Secondary event details " + dicts.get("relationship_primary_event_position", ""))
                #Get secondary and primary event position
                primaryEventPosition =  dicts.get("relationship_primary_event_position", "")
                secondaryEventPosition =  dicts.get("relationship_secondary_event_position", "")
                common_calls.print_on_debug_and_log("data summary desc : %s" %  dicts.get("displayFormat", None))
                # Checks
                checkEventCountThreshold =  dicts.get("CheckEventCountMin", 0)
                if checkEventCountThreshold == 0:
                    common_calls.print_on_debug_and_log("Skipping the event, input count is %d" % checkEventCountThreshold)
                    continue
                checkInLast24Hrs = dicts.get("CheckLast24Hrs", False)
                # Get 'displayFormat' in a dict.
                dictDataSummary = dicts.get("displayFormat", None)
                common_calls.print_on_debug_and_log("dict data summary : ")
                common_calls.print_on_debug_and_log(str(dictDataSummary))
                
                event_checks_to_include_dict = {}
                ## Special case if secondary event is present call this method else skip
                # Check order, primary appears before or secondary.
                orderofevent = dicts.get("order", 0)
                logfileobj.prepareDataToHandlePrimaryAndSecondaryEventsRelation(eventName, secondaryEventName, primaryEventPosition,secondaryEventPosition, orderofevent);
                # Get data from log
                #logfileobj.prepareEventDataForNdays(eventName, event_checks_to_include_dict, strTemp)
                # Prepare string for parsing the event
                logfileobj.prepareEventParserFormatString(eventName, eventStatus, eventIncludes, eventExcludes)
                #common_calls.print_on_debug_and_log("String Format = %s" % strTemp)

                
                ## Get required Data summary from final file
                ## Types of checks and data from XML
                listSummaryData = logfileobj.getDataSummaryFromResultFile(eventName, dictDataSummary, checkEventCountThreshold, checkInLast24Hrs)
                # Get count of event for a period
                intTotalEventCount = logfileobj.getTotalCountForAnEvent()
                # Get Event check if any event present in last 24 hrs.
                flagEventInLast24Hrs = logfileobj.getIfAnyEventInLast24Hrs()
                # Event name + secondary event name.
                combinedName = eventName
                if secondaryEventName is not None and secondaryEventName is not '':
                    combinedName += " (" + secondaryEventName + ")"
                strDateTimeRange = logfileobj.getDateRangeForAnEvent()
                # Display the data on console
                moduleDisplaySummaryHandle.displayResultSummary(combinedName, intTotalEventCount, flagEventInLast24Hrs, displayNameOnConsole, dictDataSummary, listSummaryData, strDateTimeRange)
                # Clean up the temporary files created for an event.
                logfileobj.cleanTempLogFiles(eventName)
                temp_director_filename = common_calls.getCurrentWorkingDir() + format("/logs/FLAT_TEMP_LOG_FILE_72HRS_FWMERGED.TXT")
                common_calls.fileRemove(temp_director_filename)
                #common_calls.print_on_debug_and_log listSummaryData
            # If entire firmware log file has no warning events, display the below message
            moduleDisplaySummaryHandle.displayTextifNoEventAlarming(script_path_dir)
            # Clean up temporary files created for all events
            logfileobj.cleanTempLogFilesFinal()
            #Revert the dir to the script calling directory.    
            common_calls.changeDir(script_exec_dir)
        except Exception, e:
            print traceback.format_exc()
            print e
            common_calls.stopProgressBar()
            print "Error! Failed to run the FLAT tool. \n"


'''
main : Main method called when program starts, initializes the flatBaseModule object and calls the start method.
'''
def main(argv = None):
    moduleHandler = flatBaseModule(None, True, None)	
    moduleHandler.start()


if __name__ == "__main__":
    main()

