#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

import glob
import fnmatch
import shutil
import re
import time
import datetime
import os
import sys
import traceback

#importing from FMAR dir
try:
    from FLAT.common import common_calls as common_calls
    from FLAT.common.common_calls import bcolors
except:
    # Importing from FLAT dir for standalone run
    from common import common_calls as common_calls
    from common.common_calls import bcolors

strLatestEventDate = ''

class TclParser:
    '''!
    A class to parse tcl formatted strings
    '''
    @classmethod
    def parse_convert_tcl(cls, tk ):
        token = tk.token()
        convert,text = '',''
        while(token != ''):
            if token == 'txt':
                text +=  cls.parse_txt( tk ).replace('+',' ')
            else:
                convert += token
                convert += ' '
            tk.advance()
            token = tk.token()
        convert += text
        convert += '\n'
        return convert

    @classmethod
    def parse_txt(cls, tk ):
        tk.advance()
        key = cls.replace_format_specifiers(tk.token())
        tk.advance()
        if tk.token() == 'pars':
            tk.advance()
        if tk.token() == '[':
            try:
                return key % tuple(cls.parse_list(tk))
            except Exception,v:
                pass
        return key

    @classmethod
    def replace_format_specifiers(cls,key):
        str = re.sub('%[-+]?25[0-9]*(d|u|i|o|x|X|c|s|f|e|p|g|z[d|u|x|X]|l[d|u|x|X])','%s',key)
        return re.sub('%[-+]?[0-9]*[^s]', '',str)

    #-------------------------------------------------------------
    @classmethod
    def parse_list(cls, tk ):
        plist = []
        tk.advance()                        # consume opening '['
        while 1:
            if tk.token() == ']':
                break
            elif tk.token() == '':
                raise Exception( 'Unclosed list.Bailing out' )
            else:
                plist.append( tk.token() )
                tk.advance()
        return plist

    # ------------------------------------------------------------



class Tokenizer:
    '''!
    Tokenize the TCL string input.
    '''
    def __init__( self, text ):
        self.text = text
        self.tokens = text.split()
        self.pos = 0

    def token( self ):
        if self.pos < len( self.tokens ):
            return self.tokens[self.pos]
        else:
            return ''


    def advance( self ):
        self.pos += 1


    def getContext( self ):
        before = ' '.join( self.tokens[max( 0, self.pos):self.pos] )
        after = ' '.join( self.tokens[self.pos + 1: self.pos + 1] )
        return '%s >> %s << %s' % (before, self.tokens[self.pos], after)


def getDirectorIPAddress(director):
    ids = director.split('-')
    if len(ids) != 4:
        return 'director-unknown'
    side = None
    if ids[3].upper() == 'A':
        side = 1
    elif ids[3].upper() == 'B':
        side = 2
    else:
        return 'director-unknown'
    address = int(ids[1])*32 + int(ids[2])*2 + side
    ip = '128.221.252.%s' % str(address)
    return ip



#
def convertFile(inputfile,outputfile,director):
    input_fd = open(inputfile,'r')
    output_fd = open(outputfile,'w')

    for line in input_fd:
        try:
            tk = Tokenizer(line)
            postTcl = TclParser.parse_convert_tcl(tk)

            if re.match(r'^\s+', postTcl):
            # String is empty, don't bother copying.
                continue

            if re.match(r'^[=]+', postTcl) is not None:
                output_fd.write(postTcl)
                continue
            m = re.match(r'\w{3} \d{1,2} (\d{2}:\d{2}:\d{2}) [\w|-]+ nsfw: { ds (\d{8}) ts \d{8} ser 0x\w{8} uuid 0x\w{6} name [\?|\w]+ lvl (\d{1}) cmp ([\w|-]+) id (\d+) STATUS \d+ } (.*)', postTcl)
            if m is not None:
                # m.group(1) is time (03:04:52)
                # m.group(2) is date (20140714)
                # m.group(3) is level (lvl)
                # m.group(4) is component (utl)
                # m.group(5) is component event number (11)
                # m.group(6) is the rest of the log message
                director_ip = getDirectorIPAddress(director)
                date = '%s/%s/%s' % (m.group(2)[:4], m.group(2)[4:6], m.group(2)[6:8])
                output_fd.write(director_ip + "/cpu0/log:5988:W/\"1234567891234567-2\":12345:" + "<" + m.group(3) + ">" + date + " " + m.group(1) + ": ")
                component = m.group(4) + "/" + m.group(5)
                output_fd.write("%s" % component)
                # Replace 0x0x to 0x. During TCL conversion, hex digits get additional 0x0 added making it 0x0x.
                # So, restore it back.
                text = m.group(6).replace('0x0x', '0x')
                output_fd.write(" " + text + "\n")
            else:
                # Crap, didn't match the string? Just write tcl string to output file and continue
                output_fd.write(postTcl)
                print ("Cannnot convert line: %s" % postTcl)
        except Exception,v:
            common_calls.print_on_debug_and_log("Failed in handle director logs  module: In convertFile(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(v)
            common_calls.print_on_debug_and_log(traceback.format_exc()) 
            continue

def toMergeEventLogFileForNDays(file_path_name_list, nDays = 3):
   try:
     if not file_path_name_list:
	return None
     temp_dict = {} 
     # Check the latest log file for the dates, it will check for the last 3 days from there.
     # Ensure file_path_name_list [0] index has the latest file
     resultString = []
     for each_file in file_path_name_list.keys():
       try:
           file_path_name = file_path_name_list.get(each_file, None).get('FilePath', '')
           common_calls.print_on_debug_and_log( file_path_name)
           filehandle = common_calls.fileOpen(file_path_name, 'r')
           resultData = common_calls.fileReadLines(filehandle)
           common_calls.fileClose(filehandle)
           linecount = len(resultData)
           if linecount <= 0:
             continue
           for i in xrange(linecount-4, linecount):
              try:
                  if resultData[i].find("==") > 0:
                    continue
                  if not resultData or not resultData[i] or resultData[i] == '\n' or len(resultData[i]) == 0:
                    continue
                  resultString.append(resultData[i].split()[0])
              except:
                continue
           break
       except:
          continue
     strDatesArray = [] 
     if resultString:
        for each_entry in resultString:
            try:
	      latestDateInLogFile = each_entry.split('\n')[0].split('>')[1]
	      common_calls.print_on_debug_and_log( latestDateInLogFile)
              # Get date fromat array, last 3 days
              strDatesArray = common_calls.getLastNDaysDates(latestDateInLogFile, nDays)
	      if not strDatesArray:
                continue
            except:
                continue
        if not strDatesArray:
            return file_path_name_list
        # Convert the dates list into grep understandable format
	strDates = '';
        for items in reversed(strDatesArray):
            #strDates += items + "|"
            #strDates = strDates.strip("|");#Strip from start and end. Do not need pipe at the start or end.

            # grep -i -E \"%s\" %s' > %s/logs/FLAT_TEMP_LOG_FILE_72HRS_NSFW.TXT
            formatString = "grep -i -E "
            formatString += '\"'+items+'\"'
            formatString += ' '
            for eachpath in file_path_name_list.keys():
              dict_temp = file_path_name_list.get(eachpath, None)
              if dict_temp and dict_temp.get("use_from_director", False) == False:
                formatString += dict_temp.get('FilePath', '')
                formatString += ' '
            formatString += ' >> '
            formatString +=  common_calls.getCurrentWorkingDir() + '/logs/FLAT_TEMP_LOG_FILE_72HRS_FWMERGED.TXT'
            common_calls.print_on_debug_and_log( formatString)
            # Execute the command
            resultString1 = common_calls.executeCLICommand(formatString)
            if resultString1[0] == 0:
                common_calls.print_on_debug_and_log( "File prepared and merged for Last 72 HRS monitoring")
	  
        temp_dict['FilePath'] = common_calls.getCurrentWorkingDir() + '/logs/FLAT_TEMP_LOG_FILE_72HRS_FWMERGED.TXT'
        file_path_name_list[0] = temp_dict
        temp_dic2 = {}
        for keys in file_path_name_list.keys():
            if keys == 0:
              continue
            temp_dic2 = file_path_name_list.get(keys, None)
            if temp_dic2.get("use_from_director", False) == True:
                continue
            del file_path_name_list[keys]
        return file_path_name_list
   except Exception,v:
	print (v)
        print (traceback.format_exc())
	return file_path_name_list

'''
tocreateEventLogFileForNDays : This file gets the content of directors nsfw log for past 72 hrs, converts into the format of firmware.log and stores in a temporary file.
director_ip - IP of the director to connect to for firmware logs. Here 'root' user is used which does not asks for password. In case the system will be password protected in future this code needs to be modified.
file_path_name - Name and path of file on the director.
'''
def tocreateEventLogFileForNDays(director_ip, file_path_name):
    try:
        if file_path_name is '' or file_path_name is None or director_ip is '' or director_ip is None:
            common_calls.print_on_debug_and_log("Failed! Return from tocreateEventLogFileForNDays")
            return False
        common_calls.print_on_debug_and_log( file_path_name)
        #if os.path.isfile(file_path_name):
        #formatString = "ssh root@%s ls -ltr --time-style=+%Y/%m/%d %s | tail -1 | awk '{ print( $6, $7}'" % (file_path_name)
        # Prepare format string to get the last line of the file to get the date.
        formatString = "ssh root@%s " % director_ip
        formatString += "ls -ltr --time-style=+%Y/%m/%d "
        formatString += file_path_name
        formatString += " | tail -1 | awk '{ print( $6 )}'"
        #formatString = "ssh root@%s tail -1 %s | awk '{print $1}'" % (director_ip, file_path_name)
        common_calls.print_on_debug_and_log( "Format String = %s" % formatString)
        #resultString = commands.getstatusoutput(formatString)
        # Execute the string format
        resultString = common_calls.executeCLICommand(formatString)
        common_calls.print_on_debug_and_log( str(resultString) )
        # resultString[0] return '0' if the above command success
        if resultString[0] == 0:
            latestDateInLogFile = str(resultString[1])
            # Get date fromat array, last 3 days
            strDatesArray = common_calls.getLastNDaysDates(latestDateInLogFile, 3)      #getLast72HrsDates(latestDateInLogFile)
            strLatestEventDate = strDatesArray[0]
            strDates = '';
            for items in strDatesArray:
                strDates += items + "|"
            
            strDates = strDates.strip("|");#Strip from start and end. Do not need pipe at the start or end.
            common_calls.print_on_debug_and_log( strDates)
            # Prepare events file with last 72 hrs logs.
            formatString = "ssh root@%s 'grep -i -E \"%s\" %s' > %s/logs/FLAT_TEMP_LOG_FILE_72HRS_NSFW.TXT" % (director_ip, ''.join(strDates.split('/')), file_path_name, common_calls.getCurrentWorkingDir())
            common_calls.print_on_debug_and_log( formatString)
            #resultString1 = commands.getstatusoutput(formatString)
            # Execute the command
            resultString1 = common_calls.executeCLICommand(formatString)
            if resultString1[0] == 0:
                common_calls.print_on_debug_and_log( "File prepared for Last 72 HRS monitoring")
                # Do something extra heere for directors NSFW
                # Directors NSFW are not formatted as per the LAT output or firmware.log file on magment server
                # For common XML parser we need to modify the file as per LAT output.
                inputfile = common_calls.getCurrentWorkingDir() + "/logs/FLAT_TEMP_LOG_FILE_72HRS_NSFW.TXT"
                common_calls.print_on_debug_and_log( "Input file" + inputfile)
                outputfile =  common_calls.getCurrentWorkingDir() + format("/logs/FLAT_TEMP_LOG_FILE_72HRS_%s.TXT"%director_ip)
                common_calls.print_on_debug_and_log( "Output file" + outputfile)
                director =  getDirectorNameFromIPAddress(director_ip)#Get director name from IP here
                convertFile(inputfile,outputfile,director)
                clearTempFiles()
                return True
            return False
    except:
        common_calls.print_on_debug_and_log("Failed in handle director logs  module: In tocreateEventLogFileForNDays(). Error (%s)" % sys.exc_info()[0])
        common_calls.print_on_debug_and_log(traceback.format_exc())
        return False

'''
getDirectorNameFromIPAddress : Return the name of the director from the IP Address.
strIPAddress - Input string, hold IPAddress of director
'''
def getDirectorNameFromIPAddress(strIPAddress):
    try:

        director_name = ['director-1', 'director-2', 'director-unknown']
        common_calls.print_on_debug_and_log( "getDirectorNameFromIPAddress (%s)", strIPAddress)
        for k in xrange(1, 3): # Cluster 1 and 2
            for j in xrange(1, 17): # Directors 1 to 16 on both clusters
                # Side A
                address = (k)*32 + (j)*2 + 1 #'A'
                ip = str(address) #'128.221.252.%s' % str(address) #'128.221.252.%s'
                if ip == strIPAddress.split('.')[3]:
                    return director_name[k-1] + "-" + str(j) + "-A"
                # Side B
                address = (k)*32 + (j)*2 + 2 #'B'
                ip = str(address) #'128.221.252.%s' % str(address)
                if ip == strIPAddress.split('.')[3]:
                     return director_name[k-1] + "-" + str(j) + "-B"

        return director_name[2]
    except:
        common_calls.print_on_debug_and_log("Failed in handle director logs  module: In getDirectorNameFromIPAddress(). Error (%s)" % sys.exc_info()[0])
        common_calls.print_on_debug_and_log(traceback.format_exc())
        return director_name[2]


def clearTempFiles():
    try:
        # Get path of last N hrs files
        filepathname = common_calls.getCurrentWorkingDir() + "/logs/FLAT_TEMP_LOG_FILE_72HRS_NSFW.TXT"
        common_calls.fileRemove(filepathname)
    except:
        return


#end of file
