#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

import os
import sys
import re
import commands
import traceback
from datetime import datetime, date, time, timedelta

#From FMAR dir 
try:
    from FLAT.common import common_calls
    from FLAT.modules import handleDirectorFirmwareLogs
except:
    # Running as standalone from FLAT dir
    from common import common_calls
    from modules import handleDirectorFirmwareLogs
    pass

'''
class firmwareLogFileParser : This class is used to parse the log file based on the input XML events. All the methods related to parsing the log file should be handled here.
'''
class firmwareLogFileParser:
    
    m_intTotalEventCount = 0
    m_dateOfLatestEvent = ''
    m_eventInLast24Hrs = 0
    m_eventDateTimeRange = ''
    m_listWithAllFirmwareLogDataForPastNDays = []
    m_listEachEventSpecialCase = []
    m_listEachEventDataForNDays = []
    m_flagEventGoesBackNDays = True

    '''
    prepareEventParserFormatString : This method prepares the string format for the primary event.
    '''
    def prepareEventParserFormatString(self, event_name, event_status, event_includes, event_excludes):
        ''' Start activity'''
        try:

            if event_name is '':
                return ''
            list_all_file_data = self.m_listEachEventSpecialCase
            flag_do_logical_operation = False
            flag_insert_element = True
            # If status and inclusion is not set, include all lines of events.
            if event_status is not '' or event_includes is not '' or event_excludes is not '':
                flag_do_logical_operation = True # Set to True if  multiple words has to be matched in the line
                flag_insert_element = False # Set to false if multiple words has to be matched in the line. If true just add the line.
            #print flagfound
            #int_process_1k_event_count = 0
            for eachline in list_all_file_data:
                if event_status is not '':
                    for item in event_status.split('|'):
                        if eachline.find(item) > 0:
                            flag_do_logical_operation = True
                    
                # Add includes to the event format
                if event_includes is not '':
                    for item in event_includes.split('|'):
                        if eachline.find(item) > 0:
                            flag_do_logical_operation = True
                            break
                    
                # Add exclusion to the event format
                if event_excludes is not '':
                    for item in event_excludes.split(', '):
                        if eachline.find(item) > 0:
                            flag_do_logical_operation = False
                            break
                if flag_do_logical_operation or flag_insert_element:
                    self.m_listEachEventDataForNDays.append(eachline)
                    flag_do_logical_operation = False
                    #int_process_1k_event_count += 1

                #if int_process_1k_event_count >= 1000:
                #    break
            return True
        except:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In prepareEventParserFormatString(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False

    '''
    prepareEventDataForNdays : This method execute the command to create a temporary file. It is based on the format string.
    event_name - Name of the event
    event_checks_to_include_dict - unsused
    event_format_string - Format of string used for redirecting an event.
    '''
    def prepareEventDataForNdaysUnused(self, event_name, event_checks_to_include_dict, event_format_string):
        try:

            if event_name is '' or event_format_string is '':
                return False 
            output_file_name = event_name.split("/")[0] + event_name.split("/")[1]
            stringFormat = event_format_string + " | tail -1000 " + " > %s/logs/%s.txt" % (common_calls.getCurrentWorkingDir(), output_file_name)               
            common_calls.print_on_debug_and_log( stringFormat )
            # Execute the command
            commandOutput = common_calls.executeCLICommand(stringFormat)
            #commandOutput = commands.getstatusoutput(stringFormat);
            common_calls.print_on_debug_and_log( str(commandOutput) )
            if commandOutput[0] == 0:
                return True
            return False
        except:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In prepareEventDataForNdays(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False
    
    '''
    prepareDataToHandlePrimaryAndSecondaryEventsRelation : This method is used to parse log file based on the relation between primary and secondary event.
    event_name - Name of the event
    file_name - Name of the file where data has to be parsed from.
    secondary_event_name - Name of the secondary event to be parsed.
    position_primary_event - Position of the tag common in both the events, here it is set with postion of the primary event in the log entry
    position_secondary_event - Position of the tag common in both the events, here it is set with postion of the secondary event in the log entry
    '''
    def prepareDataToHandlePrimaryAndSecondaryEventsRelation(self, event_name, secondary_event_name, position_primary_event, position_secondary_event, order_event):
        try:
            if event_name is '' :
                return
            stringEventParserFormat = ""
            output_file_name = event_name.split("/")[0] + event_name.split("/")[1]
            if output_file_name is None:
                return
            getcwd =  common_calls.getCurrentWorkingDir()
            list_all_file_data = self.m_listWithAllFirmwareLogDataForPastNDays
            self.m_listEachEventSpecialCase = []
            flagfound = False
            storeLine = ''
            flag_no_event_in_file = True
            for eachline in list_all_file_data:
                #Look for relation between primary and secondary event
                if secondary_event_name is not '' and position_primary_event is not '' and position_secondary_event is not '':
                    if order_event == "1":
                        if eachline.find(secondary_event_name) > 0 and flagfound is False:
                            flagfound = True
                        elif eachline.find(event_name) > 0 and flagfound is True:
                            self.m_listEachEventSpecialCase.append(eachline)
                            flagfound = False
                            flag_no_event_in_file = False
                        else:
                            flagfound = False
                    else:
                        if eachline.find(event_name) > 0 and flagfound is False:
                            storeLine = eachline
                            flagfound = True
                        elif eachline.find(secondary_event_name) > 0 and flagfound is True:
                            self.m_listEachEventSpecialCase.append(eachline)
                            flagfound = False
                            storeLine = ''
                            flag_no_event_in_file = False
                        else:
                            flagfound = False
                            storeLine = ''
                else:
                    if eachline.find(event_name) > 0:
                        #print event_name
                        self.m_listEachEventSpecialCase.append(eachline)
                        flag_no_event_in_file = False
            if flag_no_event_in_file:
                return False
            return True
        except:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In prepareDataToHandlePrimaryAndSecondaryEventsRelation(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False


    '''
    createEventLogFileForNDays : This method creates two temp log files, the files are for last 24 hrs and 72 hrs events. From these temp files the log file parsing starts. By creating these files once saves time for parsing all the events in this periods.
    file_path_name - Log file path name
    '''
    def createEventLogFileForNDays(self, file_path_name, iHrsToCheckBackLogFile = 0, iAppendLogFileToAnalysisFile = 0):
        try:
            if file_path_name is '' or file_path_name is None:
                return False
            
            common_calls.print_on_debug_and_log( file_path_name)
            filehandle = common_calls.fileOpen(file_path_name, 'r')
            resultData = common_calls.fileReadLines(filehandle)
            common_calls.fileClose(filehandle)
            linecount = len(resultData)
            resultString = []
            for i in xrange(linecount-4, linecount):
                try:
                    if resultData[i].find("==") > 0:
                        continue
                    if not resultData or not resultData[i] or resultData[i] == '\n' or len(resultData[i]) == 0:
                        continue
                    resultString.append(resultData[i].split()[0])
                except:
                    continue
            strDatesArray = []
            if resultString:
                for each_entry in resultString:
                    try:
                        latestDateInLogFile = each_entry.split('\n')[0].split('>')[1]
                        common_calls.print_on_debug_and_log( latestDateInLogFile)
                        # Get 24 Hrs date fromat
                        strDates = common_calls.getLast24HrsDates(latestDateInLogFile)# Last 24 hrs 
                        if not strDates:
                            continue
                        #Store last 24 hrs date for check, if event occurred in last 24 hrs.
                        self.setLatestEventDate(strDates)
                        # Get date fromat array, last 3 days
                        strDatesArray = common_calls.getLastNDaysDates(latestDateInLogFile, int(iHrsToCheckBackLogFile)/24)
                        if not strDatesArray:
                            continue
                        break
                    except:
                        continue
                common_calls.print_on_debug_and_log( "Analyzing log file for following dates (%s)" % str(strDatesArray))
                # If the strDatesArray is None, just do analysis of the entire log file, instead of the N hrs as spwecified in XML config file.
                if not strDatesArray:
                    for eachline in resultData:
                        self.m_listWithAllFirmwareLogDataForPastNDays.append(eachline)
                    return True
                icount = len(strDatesArray)-1
                strDates = strDatesArray[icount]
                flagFoundbackDateInLogFile = False
                for eachline in resultData:
                    if eachline.find(strDates) > 0:
                        flagFoundbackDateInLogFile = True
                       
                if iAppendLogFileToAnalysisFile is 1 and flagFoundbackDateInLogFile is False:
                    self.m_flagEventGoesBackNDays = False # Skip firmware log if it does not go back 3 days, look for director wise nsfw.logs. Return false to check nsfw logs
                    return False
                elif flagFoundbackDateInLogFile is False:
                    self.m_flagEventGoesBackNDays = False # Print warning for nsfw.logs once if any of the director log does not go back 3 days. Continue analysis
                else:
                    self.m_flagEventGoesBackNDays = True  # Logs go back 3 days continue analysis
                strDates = '';
                #formatString = "grep -ri -E \"%s\" %s" % (strDates, file_path_name)
                temp_log_file_path = ''
                for eachline in resultData:
                    for item in strDatesArray:
                        if eachline.find(item) > 0:
                            self.m_listWithAllFirmwareLogDataForPastNDays.append(eachline)
                common_calls.print_on_debug_and_log( "File prepared for Last 72 HRS monitoring")
                return True
            return False
        except Exception,e:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In createEventLogFileForNDays(). Error (%s)" % e)
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False

    # This method is only to print warning if NSFW logs for each director does not go back 3 days. 
    def getIfLogFileGoesBackToNdays(self):
        return self.m_flagEventGoesBackNDays
    
    def checkIfLogFileGoesBackToEndDate(self, strDate, file_path_name):
        try:

            #if os.path.isfile(file_path_name):
            #formatString = "ls -ltr %s | tail -1 | awk '{ print( $6, $7}'" % (file_path_name)
            # Prepare format string to get the last line of the file to get the date.
            formatString = "grep %s %s | tail -3" % (strDate, file_path_name)
            common_calls.print_on_debug_and_log( "Format String = %s" % formatString)
            #resultString = commands.getstatusoutput(formatString)
            # Execute the string format
            resultString = common_calls.executeCLICommand(formatString)
            common_calls.print_on_debug_and_log( str(resultString) )
            if resultString[0] == 0 and len(resultString[1]) > 0:
                return True
            return False
        except:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In checkIfLogFileGoesBackToEndDate(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False


    '''
    getDataSummaryFromResultFile : Opens the temp files, parse the data to be displayed and returns final list for each event.
    event_name - Name of the event
    data_summary_dict - Details of the event set in the dictionary
    check_min_event_count - Count if greater than this should be captured.
    '''
    def getDataSummaryFromResultFile(self, event_name, data_summary_dict, check_min_event_count, checkAnyInLast24Hrs):
        try:
            listDataSummary = []
            if event_name is '' :
                return listDataSummary
            # Total event count is set to 0 at start of summary for an event.
            self.m_intTotalEventCount = 0
            common_calls.print_on_debug_and_log( "data_summary_dict = ")
            if data_summary_dict is not None:
                common_calls.print_on_debug_and_log( str(data_summary_dict) )
                #for item in data_summary_dict:
                # Get name of data to be captured from log entry
                stringData = data_summary_dict.get("Name", None)
                if stringData is '' or stringData is None:
                    return None
                common_calls.print_on_debug_and_log( "Name = " + stringData)
                stringPos = ''
                checkPerData = 0
                listFileData = []
                file_handle = None
                if stringData is not None:
                    # Get postion of the data to be captured from the log entry
                    stringPos = data_summary_dict.get("Position", None)
                    if stringPos is '':
                        return listDataSummary
                    common_calls.print_on_debug_and_log( stringPos)
                    # Primary data to be used for grouping the duplicat entries.
                    if data_summary_dict.get("checkpervalue") is "Yes":
                        checkPerData = 1
                    try:
                        # Get the path of temporary file.
                        output_file_name = common_calls.getCurrentWorkingDir() + "/logs/" + event_name.split("/")[0] + event_name.split("/")[1] + ".txt"
                        common_calls.print_on_debug_and_log( output_file_name)
                        # Read all the lines in a list
                        listFileData = self.m_listEachEventDataForNDays#common_calls.fileReadLines(file_handle)
                        common_calls.print_on_debug_and_log( "listFileData = %s" % listFileData)
                        # If list has contents process it, else return as this event has no entry
                        if listFileData is not None and len(listFileData) != 0:
                            # Total event count stored here for an event.
                            self.m_intTotalEventCount = len(listFileData)
                            iTimeCount = self.m_intTotalEventCount
                            # Total event is more than threhold for this event, this input count (check_min_event_count) is defined in XML file
                            if int(self.m_intTotalEventCount) < int(check_min_event_count):
                                # Return empty list
                                common_calls.print_on_debug_and_log("Eevnt (%s) threshold expected (%d), found it (%d). Skipping." % (event_name, int(check_min_event_count), self.m_intTotalEventCount))
                                return listDataSummary
                            self.m_eventInLast24Hrs = 0 
                            # Enumerate through each line, here each line contains the current events only
                            for eachline in listFileData:
                                dictEventData = {}#collections.OrderedDict()
                                # Check if this event is present in last 24 Hrs
                                if checkAnyInLast24Hrs == "Yes" and self.m_eventInLast24Hrs == 0:
                                    if self.m_dateOfLatestEvent in eachline:
                                        self.m_eventInLast24Hrs = 1
                                common_calls.print_on_debug_and_log( "Value Pos = %d" % int(stringPos))
                                ## Special Case to get Director name from IP ##
                                if int(stringPos) == 0:
                                    zero_position = eachline.split(' ')[int(stringPos)]
                                    # re.match has been replaced by re.search for fixing VPLEX-20126.
                                    m = re.search("\d+\.\d+\.\d+\.\d+", zero_position)#zero_position.split('/')[0]
                                    strIPAddress = ''
                                    if m is not None:
                                        strIPAddress = m.group(0)
                                    director_name = handleDirectorFirmwareLogs.getDirectorNameFromIPAddress(strIPAddress)
                                    dictEventData[stringData] = director_name
                                ####
                                else:
                                    # Store the name and its data from log entry in a dictionary. Here name in the XML file is the key. 
                                    dictEventData[stringData] = eachline.split(' ')[int(stringPos)].strip(',')
                                #Set the count of the occurance, here it is set with 1. This is updated later while removing duplicates.
                                dictEventData["Count"] = 1
                                if iTimeCount == self.m_intTotalEventCount:
                                    datevalue = eachline.split(' ')[0].split('>')[1].strip() + ' ' + eachline.split(' ')[1].strip(':').split('.')[0]
                                    self.m_eventDateTimeRange = datevalue
                                common_calls.print_on_debug_and_log( eachline.split()[int(stringPos)])
                                # Get other data that has to be displayed from the event log entry
                                other_data_summary_dict = data_summary_dict.get("OtherDataSummary", None)
                                #common_calls.print_on_debug_and_log( "other_data_summary_dict = "
                                #common_calls.print_on_debug_and_log( other_data_summary_dicti
                                dictOtherDataSummary2 = {}
                                ncount = 0
                                if other_data_summary_dict is not None:
                                    # Enumerate through each entry
                                    for odsd in other_data_summary_dict.keys():
                                        dictOtherDataSummary = {}
                                        dicts = other_data_summary_dict.get(odsd, None)
                                        #common_calls.print_on_debug_and_log( "other_data_summary_dict.get(odsd, None) => "
                                        #common_calls.print_on_debug_and_log( dicts
                                        # Get name of the data to be parsed from log entry
                                        string_name = dicts.get("OtherDataSummaryName", None)
                                        # Get the position of the above data in the log entry
                                        string_pos =  dicts.get("OtherDataSummary_position", None)
                                        # Get the data at postion from the line and add to dict.
                                        dictOtherDataSummary[string_name] = eachline.split()[int(string_pos)].strip(',')
                                        dictOtherDataSummary2.update({ncount:dictOtherDataSummary})
                                        ncount = ncount + 1
                                        del dictOtherDataSummary
                                    dictEventData["OtherDataSummary"] = dictOtherDataSummary2
                                if iTimeCount == 1:
                                    datevalue = eachline.split(' ')[0].split('>')[1].strip() + ' ' + eachline.split(' ')[1].strip(':').split('.')[0]
                                    self.m_eventDateTimeRange = self.m_eventDateTimeRange + ' - ' + datevalue
                                # The data of an event is stored in the list as dict.
                                listDataSummary.append(dictEventData)
                                iTimeCount = iTimeCount - 1
                    except Exception,e:
                        common_calls.print_on_debug_and_log( "Unexpected error: %s" % sys.exc_info()[0])
                        common_calls.print_on_debug_and_log(traceback.format_exc())
                            #raise
                        return listDataSummary
                ## Remove duplicate entries from the list and increment the count.
                ncount = 0
                k = 1
                # Enumerate through list
                for items in listDataSummary:
                    #common_calls.print_on_debug_and_log(  len(listDataSummary)
                    iflag = len(listDataSummary)
                    common_calls.print_on_debug_and_log( "To match = %s" % items)
                    i = 0
                    if items is None:
                        k+=1
                        continue
                    # Enumerate through K+1 entry to find an item in the list, if found increment the count and remove it from the list.
                    for i in xrange(k, len(listDataSummary)):
                        #common_calls.print_on_debug_and_log( "Found %s ==> [i=%d, k=%d]" % (listDataSummary[i],i, k))
                        if  listDataSummary[i] is not None and listDataSummary[i] is not '' and (cmp(items, listDataSummary[i]) == 0):
                            #common_calls.print_on_debug_and_log( "Duplicate : %s[%d] ==> %d" % (listDataSummary[i], i, iflag))
                            tempDel = listDataSummary[i]
                            listDataSummary[i] = None
                            ncount += 1
                            del tempDel
                    dict_test = items
                    dict_test["Count"] += ncount #Entry per value is stored in count variable
                    ncount = 0
                    k+=1
                #Log the entry
                iflag = 0
                for item in listDataSummary:
                    iflag += 1
                    common_calls.print_on_debug_and_log( "%s[%d]" % (item, iflag))
                # If XML has entry for check events in last 24 hrs as Yes, but still none is seen in last 24 hrs. Then set this flag as 2, 2 will allow checkAnyInLast24Hrs to be displayed to user.
                if checkAnyInLast24Hrs == "Yes" and self.m_eventInLast24Hrs == 0:
                    self.m_eventInLast24Hrs = 2
                #return the list with data for an event. If it is empty there is no entry for current event in the log file.
                return listDataSummary
        except:
            common_calls.print_on_debug_and_log("Failed in Log Parser module: In getDataSummaryFromResultFile(). Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return None

    '''
    getTotalCountForAnEvent : This method returns the total count for an event for a specific period. 
    '''
    def getTotalCountForAnEvent(self):
        return self.m_intTotalEventCount

    
    def getIfAnyEventInLast24Hrs(self):
        return self.m_eventInLast24Hrs
    
    def getDateRangeForAnEvent(self):
        return self.m_eventDateTimeRange

    '''
    setLatestEventDate : Sets the member variable with the input date, if the input is blank sets the member with blank.
    strDate - Input date in string, could be blank for reset.
    '''
    def setLatestEventDate(self, strDate):
        self.m_dateOfLatestEvent = strDate

    '''
    cleanTempLogFiles : This method is used to clean the temporary log files created for an event.
    event_name - Name of the event
    '''
    def cleanTempLogFiles(self, event_name):
        try:
            self.m_listEachEventDataForNDays = []
            self.m_listEachEventSpecialCase = []
            return True
        except:
            return False
    
    
    '''
    cleanTempLogFilesFinal : Remove the temporary log files for last 24 hrs and 72 hrs file which has all the events during this period. It is written in separate method as all the events uses these files and has to be deleted at last.
    '''
    def cleanTempLogFilesFinal(self):
        try:
            self.m_listWithAllFirmwareLogDataForPastNDays = [] 
            return True
        except:
            return False


## End Of File ##
