#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

import traceback
# import from FMAR dir
try:
    from FLAT.common import common_calls as common_calls
    from FLAT.common.common_calls import bcolors
except:
    # import from FLAT dir as standalone tool
    from common import common_calls as common_calls
    from common.common_calls import bcolors
    pass


'''
class summaryReport : This class is used for displaying data on the console. 
'''
class summaryReport:
    
    # If this is set it means no event is in the warning list.
    m_flagNoEventFound = 0
    # Print the warning message when program starts and not again for other events.
    m_printInfo = True 
    
    m_reportFilepathName = "FirmwareReportSummary.txt"
    m_summaryFilePathName = ''

    def __init__(self, summary_report_dir=None):
        if summary_report_dir is not None:
            self.m_summaryFilePathName = summary_report_dir + "/" + self.m_reportFilepathName
        else:
            self.m_summaryFilePathName = common_calls.getCurrentWorkingDir() + "/logs/" + self.m_reportFilepathName 


    '''
    displayResultSummary : It displays event with its details, which is more than warning level.
    '''
    def displayResultSummary(self, eventName, totalEventCount, flagEventInLast24Hrs, filePathName, dictDataSummary, listSummaryData, strDateTimeRange):
        try:
            if listSummaryData is not None and len(listSummaryData) > 0:
                if self.m_printInfo is True :
                    #Warning message when program starts
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Warning             : The following events were detected in the (%s) within the last 3 days. Further investigation will be needed to confirm if the events are ongoing. Follow the event(s) instruction summaries to further isolate the problem and determine the root cause.\n\n" % filePathName, bcolors.HEADER))
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n\n" )
                    
                    print common_calls.colortext(" --------------------", bcolors.HEADER)
                    print common_calls.colortext(" Warning             : The following events were detected in the (%s) within the last 3 days. Further investigation will be needed to confirm if the events are ongoing. Follow the event(s) instruction summaries to further isolate the problem and determine the root cause.\n" % filePathName, bcolors.HEADER)
                    print common_calls.colortext(" --------------------", bcolors.HEADER)
                    self.m_printInfo = False
                #if totalEventCount >= 1000:
                #    print common_calls.colortext(" %sEvent Name (Count)  :%s %s (%d+). Over 1000 entries found in last 3 days, reporting data for 1000. Please manually investigate firmware logs for details on the rest of the events." % (bcolors.HEADER, bcolors.OKBLUE, eventName, totalEventCount), bcolors.OKGREEN)
                #    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Event Name (Count)  : %s (%d+) . Over 1000 entries found in last 3 days, reporting data for 1000. Please manually investigate firmware logs for details on the rest of the events.\n" % (eventName, totalEventCount), bcolors.OKGREEN))
                #else:
                print common_calls.colortext2(" Event Name (Count)  : ",bcolors.HEADER,"%s (%d)" % (eventName, totalEventCount), bcolors.OKBLUE)
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Event Name (Count)  : %s (%d)\n" % (eventName, totalEventCount), bcolors.OKGREEN))
                if flagEventInLast24Hrs == 1:
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" SeenInLast24Hrs     : Yes\n", bcolors.HEADER))
                    print common_calls.colortext2(" SeenInLast24Hrs     : ", bcolors.HEADER, "Yes", bcolors.OKBLUE)

                if flagEventInLast24Hrs == 2:
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" SeenInLast24Hrs     : No\n", bcolors.HEADER))
                    print common_calls.colortext2(" SeenInLast24Hrs     : ", bcolors.HEADER, "No", bcolors.OKBLUE)
                
                if  dictDataSummary.get("Description", "").strip() is not "":
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" Description         : %s\n" % (dictDataSummary.get("Description", "").strip()), bcolors.HEADER))
                    print common_calls.colortext2(" Description         : ", bcolors.HEADER,"%s" % (dictDataSummary.get("Description", "").strip()), bcolors.OKBLUE)
                if  dictDataSummary.get("InstructionSummary", "").strip() is not "":
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Instruction Summary : %s\n" %(dictDataSummary.get("InstructionSummary", "").strip()), bcolors.OKBLUE))
                    print common_calls.colortext2(" Instruction Summary : ",bcolors.HEADER,"%s" % (dictDataSummary.get("InstructionSummary", "").strip()), bcolors.OKBLUE)
                if strDateTimeRange:
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Time Range Seen     : %s\n" %(strDateTimeRange), bcolors.OKBLUE))
                    print common_calls.colortext2(" Time Range Seen     : ",bcolors.HEADER,"%s" %(strDateTimeRange), bcolors.OKBLUE)
                
                
                stringData = common_calls.colortext(" Data Summary        : ", bcolors.HEADER)
                stringLog = " Data Summary        :  "
                for i in xrange(0, len(listSummaryData)):
                    if listSummaryData[i] is None:
                        continue
                    dictEventSummary = listSummaryData[i]#collections.OrderedDict(listSummaryData[i])
                    stringData += format('%d %s events for ' % ( dictEventSummary.get("Count", 0),eventName))
                    stringLog  += format('%d %s events for ' % (dictEventSummary.get("Count", 0),eventName))
                    for dict_key in dictEventSummary.keys():
                        if dict_key == "Count" or dict_key == "OtherDataSummary" or dict_key == "Time Range Seen":
                            continue
                        stringData += format( "%s : %s " % (dict_key, str(dictEventSummary.get(dict_key, ''))))
                        stringLog  += format( "%s : %s " % (dict_key, str(dictEventSummary.get(dict_key, ''))))
                    dictenumeratekeys = dictEventSummary.get("OtherDataSummary", None)
                    for dict_key in dictenumeratekeys.keys():
                        dict_next = dictenumeratekeys.get(dict_key, '')
                        for dictnext in dict_next.keys():
                            stringData += format( "%s : %s " % (dictnext, str(dict_next.get(dictnext, ''))))
                            stringLog  += format( "%s : %s " % (dictnext, str(dict_next.get(dictnext, ''))))
                    print common_calls.colortext(stringData, bcolors.ENDC)
                    stringLog += "\n"
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", stringLog)
                    stringData = '\t\t\t'
                    stringLog = '\t\t\t'
                self.m_flagNoEventFound = 1
                print "=====\n"
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", "\n=====\n\n")
                return True
        except Exception,f:
            common_calls.print_on_debug_and_log(f)
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False

    '''
    saveResultSummary : It saves the event summary in a file. It does not display on console.
    '''
    def saveResultSummary(self, eventName, totalEventCount, flagEventInLast24Hrs, filePathName, dictDataSummary, listSummaryData):
        try:
            if listSummaryData is not None and len(listSummaryData) > 0:
                if self.m_printInfo is True :
                    #Warning message when program starts
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Warning             : The following events were detected in the (%s) within the last 3 days. Further investigation will be needed to confirm if the events are ongoing. Follow the event(s) instruction summaries to further isolate the problem and determine the root cause.\n\n" % filePathName, bcolors.HEADER))
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n\n" )
                    
                    self.m_printInfo = False
                #if totalEventCount >= 1000:
                #    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Event Name (Count)  : %s (%d+) . Over 1000 entries found in last 3 days, reporting data for 1000. Please manually investigate firmware logs for details on the rest of the events.\n" % (eventName, totalEventCount), bcolors.OKGREEN))
                #else:
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Event Name (Count)  : %s (%d)\n" % (eventName, totalEventCount), bcolors.OKGREEN))
                if flagEventInLast24Hrs == 1:
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" SeenInLast24Hrs     : Yes\n", bcolors.HEADER))

                if flagEventInLast24Hrs == 2:
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" SeenInLast24Hrs     : No\n", bcolors.HEADER))
                
                if  dictDataSummary.get("Description", "").strip() is not "":
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",common_calls.colortext(" Description         : %s\n" % (dictDataSummary.get("Description", "").strip()), bcolors.HEADER))
                if  dictDataSummary.get("InstructionSummary", "").strip() is not "":
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", common_calls.colortext(" Instruction Summary : %s\n" %(dictDataSummary.get("InstructionSummary", "").strip()), bcolors.OKBLUE))

                #print common_calls.colortext(" Data Summary        : ", bcolors.HEADER)
                stringData = common_calls.colortext(" Data Summary        :  ", bcolors.HEADER)
                stringLog = " Data Summary        :  "
                for i in xrange(0, len(listSummaryData)):
                    if listSummaryData[i] is None:
                        continue
                    dictEventSummary = listSummaryData[i]
                    stringData += format('%s%d %s events for ' % (bcolors.OKBLUE, dictEventSummary.get("Count", 0),eventName))
                    stringLog  += format('%d %s events for ' % (dictEventSummary.get("Count", 0),eventName))
                    for dict_key in dictEventSummary.keys():
                        if dict_key == "Count" :
                            continue
                        stringData += format( "%s : %s " % (dict_key, str(dictEventSummary.get(dict_key, ''))))
                        stringLog  += format( "%s : %s " % (dict_key, str(dictEventSummary.get(dict_key, ''))))
                    stringLog += "\n"
                    common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", stringLog)
                    stringData = '\t\t\t'
                    stringLog = '\t\t\t'
                self.m_flagNoEventFound = 1
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", "\n=====\n\n")
                return True
        except Exception,f:
            common_calls.print_on_debug_and_log(f)
            common_calls.print_on_debug_and_log(traceback.format_exc())  
            return False
    
    
    '''
    displayTextifNoEventAlarming : This is displayed if none of the event is in the warning zone.
    '''
    def displayTextifNoEventAlarming(self, script_dir):
        try:
            if self.m_flagNoEventFound == 0:
                print common_calls.colortext(" --------------------", bcolors.HEADER)
                print common_calls.colortext(" Report Summary:    No alarming firmware events were found in the past 3 days of the firmware/NSFW log(s).  No further action needed.\n", bcolors.HEADER)
                print common_calls.colortext("                    Report Summary is appended to file (%s)." %  self.m_summaryFilePathName, bcolors.HEADER)
                print common_calls.colortext(" --------------------", bcolors.HEADER)
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",  common_calls.colortext(" Report Summary:    No alarming firmware events were found in the past 3 days of the firmware/NSFW log file(s).  No further action needed.\n", bcolors.HEADER))
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n\n" )
                #print common_calls.colortext(" Done. ", bcolors.OKGREEN)
            else:
                print common_calls.colortext(" --------------------", bcolors.HEADER)
                print common_calls.colortext(" Report Summary:    Analysis report of firmware events found in the firmware/NSFW log(s) is appended to file (%s)." % self.m_summaryFilePathName, bcolors.HEADER)
                print common_calls.colortext(" --------------------\n", bcolors.HEADER)
            #Reset after this to zero.
            self.setflagNoEventFound(0)
            return True
        except:
            return False

    '''
    dispalyFileNameOnConsole : This method displays the name of file that is currently analyzed. 
    '''
    def dispalyFileNameOnConsole(self, logFileName):
        try:
            if logFileName is not None and logFileName is not '':
                print common_calls.colortext(" --------------------", bcolors.HEADER)
                print common_calls.colortext(" Info:    Analyzing log file (%s) for any alarming firmware events." % logFileName, bcolors.HEADER)
                print common_calls.colortext(" --------------------", bcolors.HEADER)
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a",  common_calls.colortext(" Info:    Analyzing log file (%s) for any alarming firmware events.\n" % logFileName, bcolors.HEADER))
                common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
        except:
            return

    '''
    displayXMLParseError : Display error on console in case of XML is not parsed properly.
    '''
    def displayXMLParseError(self):
        print common_calls.colortext(" Error:    Firmware/NSFW log file path is empty in the XML or there is some issue with the XML parsing. Please verify the XML and re-run again.",  bcolors.FAIL)
        #common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error:    Firmware/NSFW log file path is empty in the XML or there is some issue with the XML parsing. Please verify the XML and re-run again.\n")

    '''
    displayLogFileNotFoundError : Display error when file path and name does not exist.
    '''
    def displayLogFileNotFoundError(self, filePathName):
        print common_calls.colortext(" Error:    Log file at path (%s) does not exist." % filePathName,  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error:    Log file at path (%s) does not exist.\n" % filePathName)

    '''
    displayLogFileNotFoundError : Display error when XML config file does not exist.
    '''
    def displayXMLConfigFileNotFound(self, filePathName):
        print common_calls.colortext(" Error:    XML config file at path (%s) does not exist." % filePathName,  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error:    XML config file at path (%s) does not exist.\n" % filePathName)
    
    # If this is set it means no event is in the warning list.
    # m_flagNoEventFound = 0, default value
    def setflagNoEventFound(self, newValue):
        self.m_flagNoEventFound = newValue

    # Print the warning message when program starts and not again for other events.
    # m_printInfo = True, default value
    def setprintInfo(self, newValue):
        self.m_printInfo = newValue

    def startLoggingSummaryReport(self, version_info):
        try:

            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "w", " --------------------------------\n" )
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " Firmware Log Analysis Tool (FLAT) " + version_info + "\n")
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n\n" )
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n" )
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " Summary of analyzed event(s): \n" )
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a", " --------------------------------\n\n" )
            print  common_calls.colortext(" --------------------------------", bcolors.HEADER)
            print  common_calls.colortext(" Firmware Log Analysis Tool (FLAT) " + version_info, bcolors.HEADER)
            print  common_calls.colortext(" --------------------------------\n", bcolors.HEADER)
        except:
            return

    def displayDirectorRelatedErrors(self, errorCode, logfilepathname, director_ip):
        if errorCode == 1:
            print common_calls.colortext(" Error: IPAddress is empty in XML file.",  bcolors.FAIL)
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error: IPAddress is empty in XML file.\n")
        elif errorCode == 2:
            print common_calls.colortext(" Error: IPAddress(%s) for director has invalid IPV4 format." % director_ip,  bcolors.FAIL)
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error: IPAddress(%s) for director has invalid IPV4 format.\n" % director_ip)
        elif errorCode == 3:
            print common_calls.colortext(" Error: Director at IPAddress(%s) is unreachable." % director_ip,  bcolors.FAIL)
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error: Director at IPAddress(%s) is unreachable.\n" % director_ip)
        elif errorCode == 4:
            print common_calls.colortext(" Error: Failed! Getting file(%s) from director (%s)" % (logfilepathname, director_ip),  bcolors.FAIL)
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error: Failed! Getting file(%s) from director (%s)\n" % (logfilepathname, director_ip))
        else:
            common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error: Failed! Unknown error in displayDirectorRelatedErrors().\n")
        return

    def displayLogFileParserError(self, logfilepathname):
        print common_calls.colortext(" Error:    Failed! Parsing file (%s)"%logfilepathname,  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," Error:    Failed! Parsing file(%s).\n"%logfilepathname)
        print common_calls.colortext(" Info:    The firmware/nsfw log file needs to be parsed manualy for analysis. ",bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a"," nfo:    The firmware/nsfw log file needs to be parsed manualy for analysis.\n")
        return

    def displaySMSEventRelatedWarning(self):
        print common_calls.colortext(" WARNING:  Either the firmware.log(s) from the management-server wasn't found, or it doesn't go back 3 days, so the nsfw.logs from the directors will be used to analyze the firmware events. Due to this the SMS related events can't be checked. Please use VPlexcli command 'director uptime' to check if any directors firmware has restarted recently.  If this is the case, and the cause is unknown, engage the VPLEX Remote Support team to investigate as soon as possible.",  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a","  WARNING:    Either the firmware.log(s) from the management-server wasn't found, or it doesn't go back 3 days, so the nsfw.logs from the directors will be used to analyze the firmware events. Due to this the SMS related events can't be checked. Please use VPlexcli command 'director uptime' to check if any directors firmware has restarted recently.  If this is the case, and the cause is unknown, engage the VPLEX Remote Support team to investigate as soon as possible.\n")

    def displayWarningNotGoingBackNDays(self, timeInHrs):
        nValue = int(timeInHrs)/24 
        print common_calls.colortext(" WARNING:  Either the firmware.log(s) from the management-server wasn't found, or it doesn't go back %d days." % nValue,  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a","  WARNING:    Either the firmware log(s) from the management-server wasn't found, or it doesn't go back %d days.\n" % nValue)

    def displayNFSWRelatedWarning(self):
        print common_calls.colortext(" WARNING:  The nsfw.logs from the directors don't go back 3 days, analyzing <3 days of event history.",  bcolors.FAIL)
        common_calls.print_on_console_and_log_summary(self.m_summaryFilePathName, "a","  WARNING:    The nsfw.logs from the directors don't go back 3 days, analyzing <3 days of event history.\n")
#End of file
