#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.

import sys, os
import getopt
import shutil
import xml.dom.minidom
from xml.dom.minidom import parse
import traceback

# Importing from FMAR dir
try:
    from FLAT.common import common_calls
except:
    #Importing as standalone from FLAT dir
    from common import common_calls

'''
class xmlParserModule : XML related methods are handled in this class.
'''
class xmlParserModule:

    dictEventListForCurrentVersion = {}
    m_dictEventListWithData = {}
    m_listOfEventsForAnalysis = []
    m_dictFirmwareTagFilePathName = {}
    m_strCurrentProductsBaseVersion = ''
    m_strToolVersion = '' 
    m_strLastNhrsTimeValue = '' 
    '''
    XMLData : This method is used for parsing the XML file. This file is common parser for all events. XML file should be in config folder.
    It parses all the tags and update the dictionary for each event. The set dictionary is returned when called by caller.
    '''
    def XMLData(self): 
        
        try:

            # Open XML document using the minidom parser
            xmlFileName = common_calls.getCurrentWorkingDir() + "/config/FLAT.xml"
            domTree = xml.dom.minidom.parse(xmlFileName)
            # Get the elemnts
            flatElement = domTree.documentElement
            
            common_calls.print_on_debug_and_log( "FLAT data = %s" % flatElement)
            
            # Add version of tool in XML file.
            if flatElement.hasAttribute("version") :
                self.m_strToolVersion = flatElement.getAttribute("version")
                common_calls.print_on_debug_and_log( "XML file version is : %s" % self.m_strToolVersion)
            #Get the firmware tag from XML. Firmware tag contains all the values which is used throughout the tool.
            firmwareTags = flatElement.getElementsByTagName("Firmware")
            if firmwareTags is not None:
                for tags in firmwareTags:
                    # File path and name of log file to be analyzed.
                    filePathName = tags.getElementsByTagName("FilePath")
                    ncount = 0
                    # All path is anlayzed that is present in FilePath tag.
                    for names in filePathName:
                        localdictforeachfilepath = {}
                        # In the XML file check if the log file has to be fetched from the director or local-system. "use_from_director" is true firmware/nsfw log files will be fetched from director.
                        if names.hasAttribute("use_from_director") and names.getAttribute("use_from_director") == "True" :
                            localdictforeachfilepath["director-ip"] = names.getAttribute("director-ip")

                        if names is not None and len(names.childNodes[0]) > 0:
                            #self.m_dictFirmwareTagFilePathName.append(names.childNodes[0].data)
                            localdictforeachfilepath["FilePath"] = names.childNodes[0].data
                            common_calls.print_on_debug_and_log("Global file name = %s"  % self.m_dictFirmwareTagFilePathName)
                        self.m_dictFirmwareTagFilePathName[ncount] = localdictforeachfilepath 
                        ncount += 1 
                    # Product version is used to check if the event has to be analyzed for the current VPLEX version or not.
                    productVersion = tags.getElementsByTagName("CurrentProductBaseVersion")[0]
                    if productVersion is not None and len(productVersion.firstChild) > 0:
                        self.m_strCurrentProductsBaseVersion = productVersion.firstChild.data
                    # Last 'N' hrs value for analysis of log file
                    try:
                        timeValueInHrs = tags.getElementsByTagName("ToAnalyzeFileForLastNHrs")[0]
                        if timeValueInHrs and len(timeValueInHrs.firstChild) > 0:
                            self.m_strLastNhrsTimeValue = timeValueInHrs.firstChild.data
                        else:
                            self.m_strLastNhrsTimeValue = None
                    except:
                        pass
                        self.m_strLastNhrsTimeValue = None
            # Get all the events from XML
            #common_calls.print_on_debug_and_log( "XML file event is : %s" % flatElement.getElementsByTagName("EventTags")[0]
            eventTagsElements = flatElement.getElementsByTagName("EventTags");
            i = 0;
            eventElements = flatElement.getElementsByTagName("Event");
            nameKey = None
            # Enumerate through each element in the Event.
            for event in eventElements :
                dictEachEventData = {}
                # If event has to be analyzed or skipped, "do_check" if set it has to be analyzed. 
                if event.hasAttribute("do_check") and event.getAttribute("do_check") == "True" :
                    # Event analyzed upto 'X' version is set in "version_value_max"
                    if event.hasAttribute("version_value_max") :
                        common_calls.print_on_debug_and_log( "Event is supported in version : %s" % event.getAttribute("version_value_max"))
                        nameKey = event.getAttribute("version_value_max")
                        dictEachEventData["version_value_max"] = event.getAttribute("version_value_max")
                    # Event analyzed from 'X' version is set in "version_value_min"
                    elif event.hasAttribute("version_value_min") : 
                        common_calls.print_on_debug_and_log( "Event is supported in version : %s" % event.getAttribute("version_value_min"))
                        nameKey = event.getAttribute("version_value_min")
                        dictEachEventData["version_value_min"] = event.getAttribute("version_value_min")
                    # The event which has to be analyzed is set in "PrimaryEvent"
                    primaryEvents = event.getElementsByTagName("PrimaryEvent")
                    for primaryEvent in primaryEvents :
                        #Name of the event
                        nameValue = primaryEvent.getElementsByTagName("Name")[0]
                        if nameValue is not None and len(nameValue.firstChild)>0 : 
                            common_calls.print_on_debug_and_log( "Event Name : %s" % nameValue.firstChild.data)
                            dictEachEventData["Name"] = nameValue.firstChild.data
                            nameKey += '_' + nameValue.firstChild.data
                            #"unique_filed" tag is required if a single event is analyzed for mutiple things. As event name is common, the unique filed is used to distinguish between different entries of an event.
                            if nameValue.hasAttribute("unique_filed"):
                                nameKey += nameValue.getAttribute("unique_filed")
                        # Status if any to be checked with the event is set here.
                        statusValue =  primaryEvent.getElementsByTagName("Status")[0]
                        if statusValue is not None and len(statusValue.childNodes)>0 : 
                            common_calls.print_on_debug_and_log( "Event Status : %s" % statusValue.childNodes[0].data)
                            dictEachEventData["Status"] = statusValue.firstChild.data
                        # Any includes to be analyzed with event should be set here
                        eventIncludesValues = primaryEvent.getElementsByTagName("EventIncludes")[0] 
                        if eventIncludesValues is not None and len(eventIncludesValues.childNodes)>0 : 
                            common_calls.print_on_debug_and_log( "Event Includes : %s" % eventIncludesValues.childNodes[0].data)
                            dictEachEventData["EventIncludes"] = eventIncludesValues.firstChild.data
                        # If an event has to be excluded for any value has to be set here
                        excludeIfFoundValues = primaryEvent.getElementsByTagName("ExcludeIfFound")[0]
                        if excludeIfFoundValues is not None and len(excludeIfFoundValues.childNodes)>0 : 
                            common_calls.print_on_debug_and_log( "Event Excludes : %s" % excludeIfFoundValues.childNodes[0].data)
                            dictEachEventData["ExcludeIfFound"] =  excludeIfFoundValues.firstChild.data
                    # If an event has to be analyzed on basis of another event, it should be set in "SecondaryEvent". 
                    #Secondary event and its details
                    secondaryEvent = event.getElementsByTagName("SecondaryEvent")[0]
                    if secondaryEvent is not None and len(secondaryEvent.childNodes) > 0:
                        common_calls.print_on_debug_and_log( "secondaryEvent : %s" % secondaryEvent.firstChild.data)
                        dictEachEventData["secondaryEventName"] =   secondaryEvent.firstChild.data
                        nameKey += '_' + secondaryEvent.firstChild.data
                        # There should be some relationship between two events, here position is set for the tags in primary event which is common in both.
                        if secondaryEvent.hasAttribute("relationship_primary_event_position"):
                            common_calls.print_on_debug_and_log( "secondaryEvent details : %s" % secondaryEvent.getAttribute("relationship_primary_event_position"))
                            dictEachEventData["relationship_primary_event_position"] = secondaryEvent.getAttribute("relationship_primary_event_position")
                        # Here position is set for the tag in secondary event, which is common in both.
                        if secondaryEvent.hasAttribute("relationship_secondary_event_position"):
                            dictEachEventData["relationship_secondary_event_position"] = secondaryEvent.getAttribute("relationship_secondary_event_position")
                        # Order is set which event should be checked before
                        if secondaryEvent.hasAttribute("order"):
                            dictEachEventData["order"] = secondaryEvent.getAttribute("order")
                    # This filename is only used for display the event details to the console.
                    logFileName = event.getElementsByTagName("FileName")[0]
                    if logFileName is not None and len(logFileName.childNodes)>0 : 
                        common_calls.print_on_debug_and_log( "Event to be parsed from file : %s" % logFileName.childNodes[0].data)
                        dictEachEventData["FileName"] = logFileName.childNodes[0].data
                    
                    # Set with checks which has to be used to filter logs from the file.
                    checksToValidate = event.getElementsByTagName("ChecksToValidate")
                    if checksToValidate is not None:
                        for checks in checksToValidate:
                            #If this is set than event is analyzed in last 24 hrs.
                            checkLast24HrsEvents = checks.getElementsByTagName("CheckLast24Hrs")[0]
                            if checkLast24HrsEvents is not None and len(checkLast24HrsEvents.firstChild) > 0:
                                dictEachEventData["CheckLast24Hrs"] = checkLast24HrsEvents.firstChild.data
                            #If set it checks for last 3 days for an event
                            checkTimeHrsMax = checks.getElementsByTagName("CheckTimeHrsMax")[0]
                            if checkTimeHrsMax is not None and len(checkLast24HrsEvents.firstChild) > 0:
                                dictEachEventData["CheckTimeHrsMax"] = checkTimeHrsMax.firstChild.data
                            checkEventCountMin = checks.getElementsByTagName("CheckEventCountMin")[0]
                            #It checks the event has count greater than specified limit, should be shown in console.
                            if checkEventCountMin is not None and len(checkEventCountMin.firstChild) > 0:
                                dictEachEventData["CheckEventCountMin"] = checkEventCountMin.firstChild.data
                    #"DisplayFormat" is set with the values displayed in the console.
                    dictdisplayFormat = {}
                    displayFormat = event.getElementsByTagName("DisplayFormat")
                    common_calls.print_on_debug_and_log( " displayFormat = ")
                    common_calls.print_on_debug_and_log( str(displayFormat) )
                    if displayFormat is not None:
                        for display in displayFormat:
                            common_calls.print_on_debug_and_log( str(display))
                            # Set with the events "Description"
                            description = display.getElementsByTagName("Description")[0]
                            if description is not None and  description.firstChild is not None and len(description.firstChild) > 0:
                                common_calls.print_on_debug_and_log( description.firstChild.data)
                                dictdisplayFormat["Description"] = description.firstChild.data
                            # Set with the data to be displayed on the console for an event
                            dataSummary = display.getElementsByTagName("DataSummary")[0]
                            # name of the data to be displayed
                            if dataSummary.hasAttribute("name"):
                                dictdisplayFormat["Name"] = dataSummary.getAttribute("name")
                            #position of the data in the log entry.
                            if dataSummary.hasAttribute("position"):
                                dictdisplayFormat["Position"] = dataSummary.getAttribute("position")
                            #Parsed on this value.
                            if dataSummary.hasAttribute("checkpervalue"):
                                dictdisplayFormat["checkpervalue"] = dataSummary.getAttribute("checkpervalue")
                            #Instruction to be shown for this event.
                            instructionSummary =  display.getElementsByTagName("InstructionSummary")[0]
                            if instructionSummary is not None and len(instructionSummary.firstChild) > 0:
                                dictdisplayFormat["InstructionSummary"] = instructionSummary.firstChild.data 
                            # Other data to be displayed in the console, there could be multiple data 
                            otherDataSummary = dataSummary.getElementsByTagName("OtherDataSummary")
                            dictOtherDataSummary2 = {}#collections.OrderedDict()
                            ncount = 0
                            # List of data anlyzed is in XML tag "OtherDataSummary"
                            # Here the other data summary is stored in a dictionary becuase for every event the data could be different. 
                            # Dictionary helps here to maintain any names in the key-value pair for any event.
                            for ods in otherDataSummary:
                                dictOtherDataSummary = {}
                                common_calls.print_on_debug_and_log( "ods = %s" % ods)
                                #Name of the data to be shown in console.
                                if ods.hasAttribute("name"):
                                    dictOtherDataSummary["OtherDataSummaryName"] = ods.getAttribute("name")
                                    # Position of the other data in the log entry
                                    if ods.hasAttribute("position"):
                                        dictOtherDataSummary["OtherDataSummary_position"] = ods.getAttribute("position")
                                    dictOtherDataSummary2.update({ncount:dictOtherDataSummary})
                                    #count is used here so that the order of insert is maintained in the dictionary.
                                    ncount = ncount + 1
                                    del dictOtherDataSummary
                            dictdisplayFormat.update({"OtherDataSummary":dictOtherDataSummary2})
                        common_calls.print_on_debug_and_log(  "dictdisplayFormat = %s" % str(dictdisplayFormat))
                        #Add the "displayFormat" to a dictionary
                        dictEachEventData["displayFormat"] = dictdisplayFormat
                    #Add the event to a dictionary
                    self.m_dictEventListWithData[nameKey] = dictEachEventData
                    del dictEachEventData
            return True
        except:
            common_calls.print_on_debug_and_log("Failed: In parsing XML File. Error (%s)" % str(sys.exc_info()[0]))
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return False

    '''
    __init__ : This method is called when object of this class is created. Here the XML data is parsed and all the variables are set.
    '''
    def __init__(self):
        dictEventListWithData = {}   
        
        self.XMLData()
    
    '''
    getDictEventWithAllData : Returns the dictionary with all the event set.
    '''
    def getDictEventWithAllData(self):
        return self.m_dictEventListWithData

    
    '''
    getFirwmareTagFilePath : Returns the log file path and name which has to be analyzed.
    '''
    def getFirwmareTagFilePath(self):
        return self.m_dictFirmwareTagFilePathName

    
    '''
    getCurrentproductBaseVersion : Returns the product version of the current log file.
    '''
    def getCurrentproductBaseVersion(self):
        return self.m_strCurrentProductsBaseVersion

    '''
    getToolVersion : Returns the tool version present in the XML file.
    '''
    def getToolVersion(self):
        return self.m_strToolVersion
    
    '''
    getDictEventListForCurrentVersion : Enumerates through all the event in the dictionary and skips all the event which is not to be analyzed for this version.
    strCurrentVPLEXVersion - Input string is set with current VPLEX version. Based on this an event is skipped.
    '''
    def getDictEventListForCurrentVersion(self, strCurrentVPLEXVersion):
        try:
            if strCurrentVPLEXVersion is None or strCurrentVPLEXVersion is '':
                if self.m_dictEventListWithData is not None and len(self.m_dictEventListWithData) > 0:
                    return self.m_dictEventListWithData
                else:
                    return None
            strCurrentVPLEXVersion = strCurrentVPLEXVersion[0:3]
            # Enumerate through event dictionary
            if self.m_dictEventListWithData is not None and len(self.m_dictEventListWithData) > 0:
                for item in self.m_dictEventListWithData.keys():
                    dictEventData = self.m_dictEventListWithData.get(item, None)
                    common_calls.print_on_debug_and_log( "dictEventData.items = %s" % dictEventData.items())
                    # Based on the version_value_max and version_value_min, the event is skipped.
                    version_value = dictEventData.get('version_value_max', None)
                    if version_value is None:
                        version_value = dictEventData.get('version_value_min', None)
                        if version_value is not None:
                            common_calls.print_on_debug_and_log( "version_value min = %s" % version_value)
                            if float(version_value) > float(strCurrentVPLEXVersion):
                                common_calls.print_on_debug_and_log( "item to be removed : %s" % self.m_dictEventListWithData[item])
                                del self.m_dictEventListWithData[item]
                    else:
                        common_calls.print_on_debug_and_log( "version_value max = %s" % version_value)
                        if float(version_value) < float(strCurrentVPLEXVersion):
                            common_calls.print_on_debug_and_log( "item to be removed : %s" % self.m_dictEventListWithData[item])
                            del self.m_dictEventListWithData[item]
                return self.m_dictEventListWithData
            else:
                return None
        except:
            common_calls.print_on_debug_and_log("Failed in XML Parser module: In parsing version info. Error (%s)" % sys.exc_info()[0])
            common_calls.print_on_debug_and_log(traceback.format_exc())
            return None

    def getLastNhrsValue(self):
        return self.m_strLastNhrsTimeValue

#End Of file
