#!/usr/bin/python


import glob
import os, fnmatch
import shutil
import re
import time
import datetime


inputfile = "nsfw.log"
outputfile = "nsfw.log.merged"
director = "director-1-1-b"


class TclParser:
    '''!
    A class to parse tcl formatted strings
    '''
    @classmethod
    def parse_convert_tcl(cls, tk ):
        token = tk.token()
        convert,text = '',''
        while(token != ''):
            if token == 'txt':
                text +=  cls.parse_txt( tk ).replace('+',' ')
            else:
                convert += token
                convert += ' '
            tk.advance()
            token = tk.token()
        convert += text
        convert += '\n'
        return convert

    @classmethod
    def parse_txt(cls, tk ):
        tk.advance()
        key = cls.replace_format_specifiers(tk.token())
        tk.advance()
        if tk.token() == 'pars':
            tk.advance()
        if tk.token() == '[':
            try:
                return key % tuple(cls.parse_list(tk))
            except Exception,v:
                pass
        return key

    @classmethod
    def replace_format_specifiers(cls,key):
        str = re.sub('%[-+]?25[0-9]*(d|u|i|o|x|X|c|s|f|e|p|g|z[d|u|x|X]|l[d|u|x|X])','%s',key)
        return re.sub('%[-+]?[0-9]*[^s]', '',str)

    #-------------------------------------------------------------
    @classmethod
    def parse_list(cls, tk ):
        plist = []
        tk.advance()                        # consume opening '['
        while 1:
            if tk.token() == ']':
                break
            elif tk.token() == '':
                raise Exception( 'Unclosed list.Bailing out' )
            else:
                plist.append( tk.token() )
                tk.advance()
        return plist

    # ------------------------------------------------------------



class Tokenizer:
    '''!
    Tokenize the TCL string input.
    '''
    def __init__( self, text ):
        self.text = text
        self.tokens = text.split()
        self.pos = 0

    def token( self ):
        if self.pos < len( self.tokens ):
            return self.tokens[self.pos]
        else:
            return ''


    def advance( self ):
        self.pos += 1


    def getContext( self ):
        before = ' '.join( self.tokens[max( 0, self.pos):self.pos] )
        after = ' '.join( self.tokens[self.pos + 1: self.pos + 1] )
        return '%s >> %s << %s' % (before, self.tokens[self.pos], after)


def getDirectorIPAddress(director):
    ids = director.split('-')
    if len(ids) != 4:
        return 'director-unknown'
    side = None
    if ids[3].upper() == 'A':
        side = 1
    elif ids[3].upper() == 'B':
        side = 2
    else:
        return 'director-unknown'
    address = int(ids[1])*32 + int(ids[2])*2 + side
    ip = '128.221.252.%s' % str(address)
    return ip



#
#def convertFile(self,inputfile,outputfile,director):
input_fd = open(inputfile,'r')
output_fd = open(outputfile,'w')

for line in input_fd:
    try:
        tk = Tokenizer(line)
        postTcl = TclParser.parse_convert_tcl(tk)

        if re.match(r'^\s+', postTcl):
        # String is empty, don't bother copying.
            continue

        if re.match(r'^[=]+', postTcl) is not None:
            output_fd.write(postTcl)
            continue
        m = re.match(r'\w{3} \d{1,2} (\d{2}:\d{2}:\d{2}) [\w|-]+ nsfw: { ds (\d{8}) ts \d{8} ser 0x\w{8} uuid 0x\w{6} name [\?|\w]+ lvl (\d{1}) cmp ([\w|-]+) id (\d+) STATUS \d+ } (.*)', postTcl)
        if m is not None:
            # m.group(1) is time (03:04:52)
            # m.group(2) is date (20140714)
            # m.group(3) is level (lvl)
            # m.group(4) is component (utl)
            # m.group(5) is component event number (11)
            # m.group(6) is the rest of the log message
            print director
            director_ip = getDirectorIPAddress(director)
            print director_ip
            date = '%s/%s/%s' % (m.group(2)[:4], m.group(2)[4:6], m.group(2)[6:8])
            output_fd.write(director_ip + "/cpu0/log:5988:W/\"1234567891234567-2\":12345:" + "<" + m.group(3) + ">" + date + " " + m.group(1) + ": ")
            component = m.group(4) + "/" + m.group(5)
            output_fd.write("%s" % component)
            # Replace 0x0x to 0x. During TCL conversion, hex digits get additional 0x0 added making it 0x0x.
            # So, restore it back.
            text = m.group(6).replace('0x0x', '0x')
            output_fd.write(" " + text + "\n")
        else:
            # Crap, didn't match the string? Just write tcl string to output file and continue
            output_fd.write(postTcl)
            print ("Cannnot convert line: %s" % postTcl)
    except Exception,v:
        print("failed to convert %s"%line)
        print(v)
        continue


