#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
#
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.
#

__author__ = "Dell Inc."
__date__ = "2021"
__version__ = "2.8"

import time
import os
import sys
import json
import threading

LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_Upgrade_Checker_.log'
C_LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/Cn.log'
SUMMARY_DICT_TEMP_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/UCT_temp_dict_file.txt'
DIR_VERSION_TEMP_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/UCT_temp_dir_version_file.txt'



class ToolResources():

  def __init__(self):
    #self.notSusceptibleIssue = {}
    self.SummaryAnalysis = {}
    self.dirVersionList = []
    self.logname = LOG_FILE
    self.clog = C_LOG_FILE
    self.reportbrk = '==========================================================\n'
    self.progress_bar_stop = True
    self.progress_bar_obj = None
    self.progress_bar_event_handle = threading.Event()

  def addSummaryIssuesToReport(self, check_name, summaryReport):
    self.logit("%s called" %(self.addSummaryIssuesToReport.__name__),verbose=True)
    self.logit("[CheckName: %s] [IssueSummaryReport: %s]" %(check_name, summaryReport))
    try:
        if type(summaryReport) is list and check_name is not None:
          self.SummaryAnalysis[check_name] = summaryReport
        else:
          self.logit("Could not add summary to IssuesSummary report.")
          self.logit("Check name (%s) is None or summary (%s) is not a list."
                               %(check_name, summaryReport),verbose=True)
    except (Exception), e:
      self.logit("Failed to add to SummaryAnalysis due to error: %s" % e ,verbose=True)


  def logit(self, msg, verbose=False, verbosenotime=False, altFile=False):
    now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
    logfile = open(self.logname,'a')
    if verbosenotime:
      logfile.write('%s\n' % (msg))
      logfile.close()
    else:
      logfile.write('[%s] %s\n' % (now, msg))
      logfile.close()
    if altFile:
      now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
      logfilealt = open(self.clog,'a')
      if verbosenotime:
        logfilealt.write('%s\n' % (msg))
        logfilealt.close()
      else:
        logfilealt.write('[%s] %s\n' % (now, msg))
        logfilealt.close()

  def printIssuesSummary(self, platform, version):
    self.logit(self.reportbrk,verbose=True,verbosenotime=True,altFile=True)
    self.logit(self.reportbrk,verbose=True,verbosenotime=True,altFile=True)
    self.logit("\t\t Issues Summary Report\n",verbose=True,verbosenotime=True,altFile=True)
    self.logit(self.reportbrk,verbose=True,verbosenotime=True,altFile=True)
    self.logit(self.reportbrk,verbose=True,verbosenotime=True,altFile=True)
    try:
      if len(self.SummaryAnalysis) == 0:
        self.logit("No issues were found while performing the prechecks required for a  " \
                   "%s at GeoSynchrony %s.\n" % (platform, version),verbose=True,verbosenotime=True,altFile=True)
        self.logit("NOTES:\n",verbose=True,verbosenotime=True,altFile=True)
        self.logit("%s\n" %(('='*6)),verbose=True,verbosenotime=True,altFile=True)
        self.logit("1] If preparing for a NDU (using cca option SolveCCAReview), or for a GenU\n" \
                   "on a VS2 please review the FirmwareReportSummary.txt file, which includes\n" \
                   "analysis of concerning firmware events that have been logged over the last 3 days.\n" \
                   "Please use the description and instruction summaries noted for each event found\n" \
                   "to further investigate the events, and confirm if they pose an intermittent or\n" \
                   "persistent problem.  In particular, please thoroughly review any events that\n" \
                   "indicate potential issues with FrontEnd, BackEnd, or WAN COM connectivity, and\n" \
                   "further investigate and resolve those issues as necessary to ensure stability\n" \
                   "prior to the upgrade.\n", verbose=True,verbosenotime=True,altFile=True)
        self.logit("2] If preparing for a GenU, please ensure the VS2 & VS6 clusters are running the\n" \
                   "same GeoSynchrony version.\n", verbose=True,verbosenotime=True,altFile=True)
      else:
        self.logit("The following issue(s) were found while performing the prechecks " \
                   "required for a %s at GeoSynchrony %s:"
                                           % (platform, version),verbose=True,verbosenotime=True,altFile=True)
        i = 1
        for key in self.SummaryAnalysis:
          self.logit("\n[%s] %s:\n\t" %(i, key),verbose=True,verbosenotime=True,altFile=True)
          self.logit("\n\t".join(map(lambda x: str(x), self.SummaryAnalysis[key])),verbose=True,verbosenotime=True,altFile=True)
          i+=1

        self.logit("NOTES:\n",verbose=True,verbosenotime=True,altFile=True)
        self.logit("%s\n" %(('='*6)),verbose=True,verbosenotime=True,altFile=True)
        self.logit("1] Please reference the Master Document for more information on the " \
                   "issues reported.\n", verbose=True,verbosenotime=True,altFile=True)
        self.logit("2] If preparing for a NDU (on any platform), or for a GenU on a VS2\n" \
                   "please review the FirmwareReportSummary.txt file, which includes analysis of\n" \
                   "concerning firmware events that have been logged over the last 3 days.\n" \
                   "Please use the description and instruction summaries noted for each event found\n" \
                   "to further investigate the events, and confirm if they pose an intermittent or\n" \
                   "persistent problem.  In particular, please thoroughly review any events that\n" \
                   "indicate potential issues with FrontEnd, BackEnd, or WAN COM connectivity, and\n" \
                   "further investigate and resolve those issues as necessary to ensure stability\n" \
                   "prior to the upgrade.\n", verbose=True,verbosenotime=True,altFile=True)
        self.logit("3] If preparing for a GenU, please ensure the VS2 & VS6 clusters are running the\n" \
                   "same GeoSynchrony version.\n", verbose=True,verbosenotime=True,altFile=True)
    except (Exception), e:
      self.logit("Failed to print the Issues Summary due to error: %s" % e ,verbose=True)


  def storeVplexUpgradeCheckerDict(self):
    try:
      if len(self.SummaryAnalysis) == 0:
        self.logit("No contents in SummaryAnalysis dict from VPlexUpgradeChecker script.")
      else:
        #first clear out any old temp files so it doesn't cause a failure
        if os.path.isfile(SUMMARY_DICT_TEMP_FILE):
          os.remove(SUMMARY_DICT_TEMP_FILE)
        self.logit("Writing contents of SummaryAnalysis dict from VPlexUpgradeChecker script to temp file.")
        logfile = open(SUMMARY_DICT_TEMP_FILE, 'a')
        logfile.write(json.dumps(self.SummaryAnalysis))
        logfile.close()
    except (Exception), e:
      self.logit("Failed to write SummaryAnalysis dict to temp file due to error: %s" % e ,verbose=True)


  def importVplexUpgradeCheckerDict(self):
    try:
      if os.path.isfile(SUMMARY_DICT_TEMP_FILE):
        self.logit("Re-adding the contents of the SummaryAnalysis dict from VPlexUpgradeChecker script from temp file.")
        logfile = open(SUMMARY_DICT_TEMP_FILE, 'r')
        self.SummaryAnalysis = json.loads(logfile.read())
        logfile.close()
        os.remove(SUMMARY_DICT_TEMP_FILE)
      else:
        self.logit("The SummaryAnalysis dict temp file was not found.")
    except (Exception), e:
      self.logit("Failed to import SummaryAnalysis dict from temp file due to error: %s" % e ,verbose=True)

  def addDirectorVersionToTempFile (self, dirVersionList):
    self.logit("%s called" %(self.addDirectorVersionToTempFile.__name__),verbose=True)
    try:
      if not dirVersionList:
        self.logit("No contents in director version list from VPlexUpgradeChecker script.")
      else:
        #first clear out any old temp file to ensure it doesn't cause a failure
        if os.path.isfile(DIR_VERSION_TEMP_FILE):
          os.remove(DIR_VERSION_TEMP_FILE)
        self.logit("Writing contents of director version list from VPlexUpgradeChecker script to temp file.")
        logfile = open(DIR_VERSION_TEMP_FILE, 'a')
        logfile.write(json.dumps(dirVersionList))
        logfile.close()
    except (Exception), e:
      self.logit("Failed to write director version list to temp file due to error: %s" % e ,verbose=True)

  def getDirectorVersion (self):
    self.logit("%s called" %(self.getDirectorVersion.__name__),verbose=True)
    try:
      if os.path.isfile(DIR_VERSION_TEMP_FILE):
        self.logit("Re-adding the contents of the director version list from VPlexUpgradeChecker script from temp file.")
        logfile = open(DIR_VERSION_TEMP_FILE, 'r')
        dirVersionList = json.loads(logfile.read())
        logfile.close()
        os.remove(DIR_VERSION_TEMP_FILE)
        return dirVersionList
      else:
        self.logit("The director version list temp file was not found.")
    except (Exception), e:
      self.logit("Failed to import director version list from temp file due to error: %s" % e ,verbose=True)

#### Progress indicator for the checks ##############
#### User of the progress bar should use startProgressBar and stopProgressBar method only.
  def startProgressBar(self, msg):
      try:
          self.progress_bar_stop = False
          self.progress_bar_obj = threading.Timer(2.0, self.loopProgressBar, [msg])
          self.progress_bar_obj.start()
      except (Exception), e:
          if self.progress_bar_obj:
            self.progress_bar_obj.cancel()
          self.progress_bar_stop = True
          return
  
  def stopProgressBar(self, error = False):
      try:
        self.progress_bar_event_handle.wait()  
        self.progress_bar_stop = True
        self.progress_bar_obj.cancel()
        if error:
            sys.stdout.write("Error" + os.linesep)
        else:
            sys.stdout.write("Done" + os.linesep)
      except (Exception), e:
          self.progress_bar_stop = True
          return

  def loopProgressBar(self, msg):
      try:
          sys.stdout.write(msg + " :")
          self.progress_bar_event_handle.set()
          while not self.progress_bar_stop:
              sys.stdout.write('.')
              sys.stdout.flush()
              sys.stdout.flush()
              time.sleep(2)
          print '\r\n'
      except:
          self.progress_bar_stop = True
          return
#### Progress indicator for the checks Ends ##############

#End of File

