#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
#
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.
#

__author__ = "Dell Inc."
__date__ = "2021"
__version__ = "2.8"

__all__ = ['VPlexUpgradeChecker']

from datetime import date, timedelta
import datetime
import re
import getpass
import sys, glob, os
from optparse import Option, OptionParser, OptionValueError
import time
import telnetlib
import commands
from threading import Thread
import UpgradeCheckerTool_common
import subprocess
from FLAT import flatBaseModule as flatBaseModule
from multiprocessing.pool import ThreadPool

TOOL_VERSION = 'v2.8'
LOCK_FILE = '/home/service/lock_upgrade'
LOCK_FILE_LINUX = '/home/service/lock_upgrade_linux'
LOG_DIR = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker'
logStartTime = time.strftime("%Y-%m-%d_%H:%M:%S_%Z", time.localtime())
LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_Upgrade_Checker_.log'
C_LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/Cn.log'
CONF = '/var/log/VPlex/cli/VPlexconfig.xml'
MANIFEST = '/etc/vplex/managed-systems/local-system/manufacturing_manifest.properties'
#variables for detremining last 3 days of firmware log events:
FW = '/var/log/VPlex/cli/firmware.log_*'
RECSEP = '==========================================================\n'
vs6GenuIps = [ '35','36','37','38','39','40','41','42','67','68','69','70','71','72','73','74']

# CST lockbox header
CST_ADMIN='/opt/emc/VPlex/cst/cstadmin'
CST_DIR='/etc/opt/emc/VPlex/cst/'
CST_PASSPHRASE='Mi@Dim7T'
CST_LOCKBOX='/etc/opt/emc/VPlex/cst/csp.clb'
CST_DEFAULT_PAM_CONFIG_NAME='VPLEX-PAM-Authority'

OBJTOOLRESOURCES = UpgradeCheckerTool_common.ToolResources()

class MyOption(Option):
    def check_value(option, opt, value):
        for val in value.split(','):
            if val not in option.choices:
                choices = ", ".join(map(repr, option.choices))
                raise OptionValueError("option %s: invalid choice: %r (choose from %s)" % (opt, val, choices))
        return value

class genu_check(object):
  """
  Class genu_check
  """
  def __init__(self, HW, SubCmd, version, director_version, cca_review=None):
    """
    Class genu_check constuctor
    : param HW: vs2 or vs6 hw plat ( Argonaut | VPL )
    : type HW: str
    : param version:
    : type verstion: str
    """
    self.cca_review = cca_review == "SolVeCCAReview"
    self.lockCheck()
    self.onlyGenuchkLock(action='lock')
    self.logname = LOG_FILE
    self.clog = C_LOG_FILE
    self.mkDir = self.makeLogDir(LOG_DIR)
    self.hw = HW
    self.subcmd = SubCmd
    self.directorup = self.listOfDirectors()
    self.vers = version
    self.director_vers = director_version
    self.objToolResources = OBJTOOLRESOURCES

  def onlyGenuchkLock(self,action=False):
    if os.path.isfile(LOCK_FILE_LINUX) and action == 'lock':
      return 'Locked'
    elif not os.path.isfile(LOCK_FILE) and action == 'lock':
      lockinfo = "Locked at " + commands.getoutput('date')
      file(LOCK_FILE_LINUX, 'w').write(str(lockinfo))
    elif action == 'unlock':
      try:
        self.logit('Removing linux-level lockfile',verbose=True)
        os.remove(LOCK_FILE_LINUX)
      except:
        self.logit("Lock file %s, removal failed, delete if no other instances are running" % LOCK_FILE_LINUX,verbose=True)

  def lockCheck(self):
    if os.path.isfile(LOCK_FILE) or os.path.isfile(LOCK_FILE_LINUX):
      # Use print here rather than logit. We don't want to create a log file at this point
      print("WARNING: /home/service/lock_upgrade or /home/service/lock_upgrade_linux lock file exists.")
      print("Ensure other instances of the script are not running")
      print("by checking size of logs in /var/log/VPlex/cli/VPLEX_Upgrade_Checker")
      print("If logs have not grown in the last 15 minues remove the lock file and re-run.")
      sys.exit(1)

  def makeLogDir(self,mkLogDir):
    # Use print here as logdir may not exist yet
    print("Checking if the VPLEX_Upgrade_Checker directory already exists, and cleaning up any old logs found...")
    if not os.path.exists(mkLogDir):
      try:
        os.makedirs(mkLogDir)
      except(Exception), e:
        print("Error: %s Verifying logging directory." % e)
    if os.path.isfile(LOG_FILE):
      try:
        os.remove(LOG_FILE)
      except(Exception), e:
        print("Error: %s removing %s." % (e,LOG_FILE))
    finalDebugLog = glob.glob('/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_Upgrade_Checker_*.log')
    for file in finalDebugLog:
      if os.path.isfile(file):
        try:
          os.remove(file)
        except(Exception), e:
          print("Error: %s removing %s." % (e,file))
    if os.path.isfile(C_LOG_FILE):
      try:
        os.remove(C_LOG_FILE)
      except(Exception), e:
        print("Error: %s removing %s." % (e,C_LOG_FILE))
    autoCleanupLog = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_UCT_AutoCleanup_Log.txt'
    if os.path.isfile(autoCleanupLog):
      try:
        os.remove(autoCleanupLog)
      except(Exception), e:
        print("Error: %s removing %s." % (e,autoCleanupLog))
    tarFiles = glob.glob('/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEXUpgradeCheckerAnalysis_*.tar.gz')
    for file in tarFiles:
      if os.path.isfile(file):
        try:
          os.remove(file)
        except(Exception), e:
          print("Error: %s removing %s." % (e,file))
    self.logit("VPlexUpgradeChecker Tool %s starting." % TOOL_VERSION)

  def logit(self, msg, verbose=False, verbosenotime=False, altFile=False):
    now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
    with open(self.logname, 'a') as fout:
      if verbosenotime:
        fout.write('%s\n' % (msg))
      else:
        fout.write('[%s] %s\n' % (now, msg))
      #if verbose and not verbosenotime:
      #  print '[%s] %s' % (now, msg)
      #elif verbosenotime:
      #  print '%s' % (msg)
    if altFile:
      now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
      with open(self.clog, 'a') as Cout:
        if verbosenotime:
          Cout.write('%s\n' % (msg))
        else:
          Cout.write('[%s] %s\n' % (now, msg))

  def existsTest(self, filename):
    # Make sure file exits and is non-zero bytes
    status = os.path.exists(filename) and os.path.getsize(filename) > 0
    return status

  def findIt(self, fullpath, pat):
    file_ok = self.existsTest(fullpath)
    if not file_ok:
      self.logit("File: " + fullpath + " Either does not exist or is empty.")
      return
    match = re.findall(pat, open(fullpath,'r').read())
    return match

  def productType(self):
    pat = re.compile(r'<producttype>(\w+)', re.M)
    return self.findIt(CONF,pat)

  def directorCount(self):
    pat = re.compile(r'director_count=\d')
    return self.findIt(MANIFEST,pat)

  def getDirectorIPs(self):
    IPs = []
    getProductType = self.productType()
    if not getProductType:
      return None
    getDirectorCount = self.directorCount()
    try:
      if 'Local' in getProductType and 'director_count=2' in getDirectorCount:
        IPs = [ '35','36' ]
        return IPs
      elif 'Local' in getProductType and 'director_count=4' in getDirectorCount:
        IPs = [ '35','36','37','38' ]
        return IPs
      elif 'Local' in getProductType and'director_count=8' in getDirectorCount:
        IPs = [ '35','36','37','38','39','40','41','42' ]
        return IPs
      elif 'Metro' in getProductType and 'director_count=2' in getDirectorCount:
        IPs = [ '35','36','67','68' ]
        return IPs
      elif 'Metro' in getProductType and 'director_count=4' in getDirectorCount:
        IPs = [ '35','36','37','38','67','68','69','70' ]
        return IPs
      elif 'Metro' in getProductType and 'director_count=8' in getDirectorCount:
        IPs = [ '35','36','37','38','39','40','41','42','67','68','69','70','71','72','73','74']
        return IPs
    except (Exception), e:
      self.logit("Error: %s getting the director IPs" % e)

  def getUnconfSysIps(self):
    updirlst = []
    downdirlist = []
    self.logit("Un-configured system detected. Pinging all possible directors to determine which are available...",verbose=True)
    try:
      for director in vs6GenuIps:
        status = os.system('ping -c 3 128.221.252.%s >/dev/null 2>&1 ' % director)
        if not status:
          updirlst.append('128.221.252.'+director)
      if not updirlst:
        self.logit("Could not ping any directors.  Please fix the connectivity and re-run the tool.",verbose=True)
        print("Could not ping any directors.  Please fix the connectivity and re-run the tool.")
        self.onlyGenuchkLock(action='unlock')
        sys.exit()
      else:
        self.logit("Proceeding as connectivity verified to the following directors:",verbose=True)
        for director in updirlst:
          self.logit(director,verbose=True)
        self.logit("If any directors are missing, please fix the connectivity and re-run the tool.",verbose=True)
        return updirlst
    except (Exception), e:
      self.logit("Error: %s Running ping test on VS6 directors for GENU option." % e)

  def runPing252(self,*args):
    IP = (str(args[0]) + str(args[1]))
    pingTest = os.system('ping -c 3 128.221.252.%s >/dev/null 2>&1 ' % IP)
    return '128.221.252.' + IP + '=result=' + str(pingTest)

  def runPing253(self,*args):
    IP = (str(args[0]) + str(args[1]))
    pingTest = os.system('ping -c 3 128.221.253.%s >/dev/null 2>&1 ' % IP)
    return '128.221.253.' + IP + '=result=' + str(pingTest)

  def listOfDirectors(self):
    print("Running initial check of director connectivity...")
    self.logit("Checking connectivity to the directors, this ping test could take time to complete...",verbose=True)
    upOn252List = []
    downOn252List = []
    upOn253List = []
    downOn253List = []
    UnconfSysIpsList = []
    results252lst = []
    results253lst = []
    pool252 = ThreadPool(16)
    pool253 = ThreadPool(16)
    ips = self.getDirectorIPs()
    if not ips:
      UnconfSysIpsList = self.getUnconfSysIps()
      return UnconfSysIpsList
    else:
      try:
        #need to use only one subnet as we don't want to use a list comprised of both subnets for a director in other functions
        self.logit("Checking the .252 subnet...")
        for director in ips:
          results252lst.append(pool252.apply_async(self.runPing252,(director)))
        pool252.close()
        pool252.join()
        results252lst = [r.get() for r in results252lst ]
        for r in results252lst:
          r_status = r.split("=")
          if r_status[2] == '0':
            upOn252List.append(r_status[0])
          else:
            downOn252List.append(r_status[0])
            self.logit("Director: %s did not respond to ping on the 252 subnet." % r_status[0])
        if not downOn252List:
          self.logit("Able to connect to all directors on the .252 subnet, proceeding...")
          return upOn252List
        self.logit("Checking the .253 subnet...")
        for director in ips:
          results253lst.append(pool253.apply_async(self.runPing253,(director)))
        pool253.close()
        pool253.join()
        results253lst = [r.get() for r in results253lst ]
        for r in results253lst:
          r_status = r.split("=")
          if r_status[2] == '0':
            upOn253List.append(r_status[0])
          else:
            downOn253List.append(r_status[0])
            self.logit("Director: %s did not respond to ping on the 253 subnet." % r_status[0])
        if not downOn253List:
          self.logit("Able to connect to all directors on the .253 subnet, proceeding...")
          return upOn253List
        else:
          self.logit("The following directors did not respond to a ping on either subnet:")
          print("The following directors did not respond to a ping on either subnet:")
          for director in downOn252List:
            self.logit(director)
            print(director)
          for director in downOn253List:
            self.logit(director)
            print(director)
          msg = "The required pre-checks can not be performed on these directors, so it's\n" \
                "recommended to fix the connectivity and then re-run the tool.\n" \
                "The tool will now proceed to check for a supported GeoSynchrony version.\n" \
                "If the reachable directors' version is supported the tool will continue by\n" \
                "using the directors reachable on the .252 subnet...\n"
          print(msg)
          self.logit(msg)
          print("Reachable directors are as follows")
          for director in upOn252List:
            print("Director: %s is reachable." % director )
          return upOn252List
      except (Exception), e:
          self.logit("Info: %s Status running ping tests." % e)


  def getLatestFirmwareFile(self, path):
     try:
         list_files = glob.glob(path)
         latestfirmwarefile = []
         format_string = "ls -t " + FW
         status = commands.getstatusoutput(format_string)
         if status[0] == 0:
             latestfirmwarefile = status[1].split('\n')
             self.logit("The firmware.logs that will be passed to FLAT are:", verbose=True,altFile=True)
             for file in latestfirmwarefile:
               self.logit(file, verbose=True,altFile=True)
     except Exception, e:
         self.logit("Error: %s on getting a latest firmware log file." % e, verbose=True)
     return latestfirmwarefile

  def checkFilePerms(self,pat,rmtfile):
    checkName = "VS2 portLayout File Permissions Check"
    self.logit("GeoSynchrony version on the directors is 6.0.1 Patch 4...",verbose=True,altFile=True)
    self.logit("Checking for the portLayout file, and its permissions, on each director...\n",verbose=True,altFile=True)
    permsNotOkList = []
    SummaryReport = []
    for ip in self.directorup:
      try:
        # See if the file exits
        rmtexists = os.system('ssh root@{0} test -f {1}'.format(ip,rmtfile))
        if rmtexists:
          self.logit("Director %s: portLayout file not present, no action needed.\n" % (ip),verbose=True,altFile=True)
        if not rmtexists:
          self.logit("Director %s: portLayout file exists, checking its permissions..." % (ip),verbose=True,altFile=True)
          out = os.system('ssh root@{0} ls -l {1} | grep -q ^{2}'.format(ip,rmtfile,pat))
          if not out:
            self.logit("Director %s: Permissions are correct, no action needed. \n" % (ip),verbose=True,altFile=True)
          else:
            self.logit("Director %s: The permissions need to be fixed. \n" % (ip),verbose=True,altFile=True)
            permsNotOkList.append(ip)
      except (Exception), e:
        self.logit("Error: %s on checking file perms for %s on %s." % (e,rmtfile,ip),verbose=True,altFile=True)
    if not permsNotOkList:
      self.logit("The portLayout permissions are ok on all reachable directors, no action needed.\n",verbose=True,altFile=True)
    else:
      msg = "The following directors need their portLayout file permissions fixed to grant\n" \
            "read access to service. A Customer Engineer (CE) should implement the workaround.\n" \
            "For each of these directors, SSH to the director and run this cmd to fix the permissions:\n" \
            "chmod a+r /var/opt/zephyr/flashDir/portLayout\n" \
            "Directors:"
      self.logit(msg,verbose=True,altFile=True)
      SummaryReport.append(msg)
      for ip in permsNotOkList:
        self.logit(ip,verbose=True,altFile=True)
        SummaryReport.append(ip)
      self.logit(RECSEP,verbose=True,altFile=True)
      self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)

  def tailFile(self):
    readit = True
    readcont = open(LOG_FILE,'r')
    readcont.seek(0)
    while readit:
      line = readcont.readline()
      if 'Removing VPlexcli-level lockfile' in line:
        readit = False
        print line,
        break
      if 'Removing linux-level lockfile' in line:
        readit = False
        print line,
        break
      if not line:
        time.sleep(1)
        continue
      print line,
    readcont.close()

  # Firmware log analysis tool (FLAT) module check.
  def flatCheck(self):
      self.logit("Calling FLAT to analyze the firmware events from the last 3 days...",verbose=True,altFile=True)
      self.logit("FLAT analysis will log to /var/log/VPlex/cli/VPLEX_Upgrade_Checker/FirmwareReportSummary.txt.",verbose=True,altFile=True)
      try:
          # Check if FLAT folder is present in LibExt
        #/opt/emc/VPlex/jython*/LibExt/FLAT
        flatfolderpath = os.path.split(os.path.realpath(__file__))[0] + '/FLAT'
        status = self.existsTest(flatfolderpath)
        if not status:
          self.logit("The %s directory is missing." % (flatfolderpath), altFile=True)
          self.logit("This directory is required for firmware log analysis. Copy the FLAT dir and its content in place and retry.", altFile=True)
          self.onlyGenuchkLock(action='unlock')
          exit(1)
          # Check if FLAT config XML file is present in LibExt
        #/opt/emc/VPlex/jython*/LibExt/FLAT/config/FLAT.xml
        flatfolderpath += '/config/FLAT.xml'
        status = self.existsTest(flatfolderpath)
        if not status:
          self.logit("The %s file is missing." % (flatfolderpath), altFile=True)
          self.logit("This FLAT.xml is required for firmware log analysis. Copy the FLAT dir and its content in place and retry.", altFile=True)
          self.onlyGenuchkLock(action='unlock')
          exit(1)
        fwlist = self.getLatestFirmwareFile(FW)
        director_ip_list = self.directorup
        product_version = None
        if self.vers and "mismatch" not in self.vers and "NDU" not in self.vers:
          product_version = self.vers
        time.sleep(30)
        objflatBaseModule = flatBaseModule.flatBaseModule(fwlist, False, LOG_DIR, director_ip_list,product_version)
        objflatBaseModule.start()
      except Exception, e:
          self.logit("Error calling FLAT: %s" % e, verbose=True)
          return


  def nduOsCommands(self):
    self.logit(RECSEP, altFile=True)
    self.logit('Collecting the OS-level commands required for the NDU CCA Process:',  altFile=True)
    self.logit(RECSEP, altFile=True)
    self.logit('Running: exec cat /opt/emc/VPlex/EmaAdaptorConfig.properties | grep com.emc.zephyr.emaAdaptor.TLA', altFile=True)
    self.logit(commands.getoutput('exec cat /opt/emc/VPlex/EmaAdaptorConfig.properties | grep com.emc.zephyr.emaAdaptor.TLA'),altFile=True)
    self.logit(RECSEP, altFile=True)
    self.logit('Running: exec cat /opt/emc/VPlex/EmaAdaptorConfig.properties | grep com.emc.zephyr.emaAdaptor.CSISiteID', altFile=True)
    self.logit(commands.getoutput('exec cat /opt/emc/VPlex/EmaAdaptorConfig.properties | grep com.emc.zephyr.emaAdaptor.CSISiteID'),altFile=True)
    self.logit(RECSEP, altFile=True)

  def oldRoutesWarning(self):
    SummaryReport = []
    checkName = "VPN Connectivity Issue After GenU IP Swap"
    self.logit(RECSEP, altFile=True)
    msg = 'WARNING: If the VS2 & VS6 are on different subnets and the IP swap option is used as part of the\n' \
               'Upgrade Manager it results in VPN connectivity loss following the IP swap. If on different\n' \
               'subnets, its recommended to skip the automated IP swap option to prevent this issue.\n' \
               'Following the completion of the GenU the SolVe Desktop procedure "Change IP addresses-\n' \
               'VPLEX Metro (VS6)" can be used to manually swap the IP addresses after ensuring the VS2\n' \
               'system(s) is no longer on the network. Also, prior to the GenU the\n' \
               '/etc/sysconfig/network/ifroute-eth3 file on the VS2 can be checked for any old routes that\n' \
               'could potentially cause a conflict. Professional Services should discuss this potential issue\n' \
               'with the customer prior to the GenU to determine the best course of action to take.\n'
    self.logit(msg, altFile=True)
    self.logit(RECSEP, altFile=True)
    SummaryReport.append(msg)
    self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)

  def vplexPlatHC(self):
    checkName = "VPlexPlatformHealthCheck Analysis"
    SummaryReport = []
    self.logit(RECSEP, altFile=True)
    self.logit('Running the "VPlexPlatformHealthCheck -v" command.', altFile=True)
    self.logit('This will take time to complete, and will print to the console once finished...')
    self.objToolResources.startProgressBar(checkName)
    try:
      command = 'VPlexPlatformHealthCheck -v'
      p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=None, shell=True)
      printOutput = p.stdout.read()
      self.logit(printOutput, altFile=True)
      if not printOutput:
        msg = "Could not get the output of the VPlexPlatformHealthCheck -v command.\nA Customer Engineer(CE) should run the command " \
              "manually on the VS6\nmanagement-server, and check for any issues reported."
        self.objToolResources.stopProgressBar(error=True)
        self.logit(msg, altFile=True)
        SummaryReport.append(msg)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      elif "Platform Health Check Passed" in printOutput:
        self.objToolResources.stopProgressBar(error=False)
        self.logit("No errors were found in the 'VPlexPlatformHealthCheck -v' command output.",altFile=True)
      else:
        msg = "Errors were found in the output of the VPlexPlatformHealthCheck -v command.\nA Customer Engineer (CE) should " \
              "review the output (included in the\nCx_<last4TLA>.log), and resolve the issues reported before the GenU.\n\n" \
              "NOTE: Errors logged for the A4-FC and B4-FC SLICs are expected during the GenU\nand can be ignored.\n\n" \
             "NOTE: If the VPlexPlatformHealthCheck reports that the ECOM process is not\nrunning on one or more " \
             "VS6 directors then Professional Services should follow\nKBA 519163 to work around this issue. " \
             "Warning: not following the recommended\nsteps can potentially lead to continuous firmware crashes on the " \
             "VS6 directors.\n"
        self.objToolResources.stopProgressBar(error=False)
        self.logit(msg, altFile=True)
        SummaryReport.append(msg)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
    except (Exception), e:
      self.objToolResources.stopProgressBar(error=True)
      self.logit("Error: %s analyzing VPlexPlatformHealthCheck" % (e),altFile=True)

  # Check for verifying CST lockbox on mangement server, saves report to summary and print message to console.
  def checkAndLogCSTLockBox(self):
    checkName = 'Check CST lockbox'
    SummaryReport = []
    self.logit(RECSEP, altFile=True)
    self.logit("Checking the CST lockbox on the management-server...",altFile=True)
    nret = self.verifyCSTLockBoxCorruption()
    if nret < 0:
      if nret == -1:
          msg = "CST lockbox service is not running. A Customer Engineer (CE) should follow\n" \
               "KBA 457268 to correct the CST lockbox file(s) on the management-server.\n"
          self.logit(msg, altFile=True)
          SummaryReport.append(msg)
          self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      elif nret == -2:
          msg =   "CST lockbox permissions are incorrect. A Customer Engineer (CE) should follow\n" \
                 "KBA 457268 to correct the CST lockbox permissions.\n"
          self.logit(msg, altFile=True)
          SummaryReport.append(msg)
          self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      elif nret == -3:
          msg = "The CST VPLEX-PAM-Authority is invalid, indicating the CST lockbox is corrupt.\n" \
          "A Customer Engineer (CE) should implement the workaround (engage a FSS or VPLEX Remote\n" \
          "Support if needed) in KBA 457268 to correct the CST lockbox on the management-server.\n"
          self.logit(msg, altFile=True)
          SummaryReport.append(msg)
          self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
    else:
      msg = "CST lockbox verified, no issues found. No action needed."
      self.logit(msg, altFile=True)


    # Verify CST lock box is not corrupt
    # Checks:
        ## sudo /etc/init.d/cst status is running
        ## permission for lockbox files are correct
            # user permission should not be root, but the owner should be service user
            #service@ManagementServer:~> ls -l /etc/opt/emc/VPlex/cst/*.clb*
            #-rw-r--r-- 1 service users 22990 Mar  6 07:01 /etc/opt/emc/VPlex/cst/csp.clb
            #-rw-r--r-- 1 service users  2842 Aug 30  2017 /etc/opt/emc/VPlex/cst/csp.clb.bak
            #-rw-r--r-- 1 service users     4 Aug 30  2017 /etc/opt/emc/VPlex/cst/csp.clb.bak.FCD
            #-rw-r--r-- 1 service users   256 Mar  6 07:01 /etc/opt/emc/VPlex/cst/csp.clb.FCD
        ## Check if the VPLEX-PAM-Authority is valid in Lockbox
            # /opt/emc/VPlex/cst/cstadmin validate-config PAM VPLEX-PAM-Authority -cstdir=/etc/opt/emc/VPlex/cst/ -passphrase=Mi@Dim7T

  def verifyCSTLockBoxCorruption(self):
    try:
        '''
        Check CST service is running or not
        '''
        checkName = 'Check CST lockbox'
        self.objToolResources.startProgressBar(checkName)
        stringCommandFormat = 'sudo /etc/init.d/cst status'
        result = commands.getstatusoutput(stringCommandFormat)
        if result and result[0] == 0:
            if result[1].find('running') > 0:
                pass
            else:
                self.objToolResources.stopProgressBar(error=True)
                self.logit("CST lockbox service not running.", altFile=True)
                return -1
        else:
            self.objToolResources.stopProgressBar(error=True)
            self.logit("CST lockbox service not running.", altFile=True)
            return -1

        '''
        Check file permissions, all files in /etc/opt/emc/VPlex/cst/ should have permissions of 644 and service users
        '''
        stringPath = '/etc/opt/emc/VPlex/cst/'
        for strFilenames in os.listdir(stringPath):
            path_name = os.path.join(stringPath, strFilenames)
            file_UID = os.stat(path_name).st_uid
            if file_UID != 1000:
                self.objToolResources.stopProgressBar(error=True)
                self.logit("Unable to validate, CST file (%s) requires service user permissions" % path_name, altFile=True)
                return -2

        '''
        Check lock box validity
        '''
        stringCommandFormat = CST_ADMIN + " validate-config PAM " + CST_DEFAULT_PAM_CONFIG_NAME + " -cstdir=" + CST_DIR + " -passphrase=" + CST_PASSPHRASE
        result = commands.getstatusoutput(stringCommandFormat)
        if result and result[0] == 0:
            if result[1].find('PAM VPLEX-PAM-Authority validated.') > 0:
                pass
            else:
                self.objToolResources.stopProgressBar(error=True)
                self.logit("Unable to validate, VPLEX-PAM-Authority in lock box.", altFile=True)
                return -3
        else:
            self.objToolResources.stopProgressBar(error=True)
            self.logit("Unable to validate, VPLEX-PAM-Authority in lock box.", altFile=True)
            return -3

        self.objToolResources.stopProgressBar(error=False)
        return 0
    except Exception,e:
        self.objToolResources.stopProgressBar(error=True)
        self.logit("Error: %s validating CST lock box" % (e),altFile=True)
        return -4

  def checkPortRoles(self):
    checkName = "Check VS6 for portRoles with all ports set to off"
    dirsWithPortRolesIssue = []
    SummaryReport = []
    self.logit("Analyzing the portRoles file to look for all ports set to off...", altFile=True)
    try:
      for dir in self.directorup:
        # Check if front ports are "off", if so assume all ports are off
        match = os.system('ssh root@{0} grep -w front /var/opt/zephyr/flashDir/portRoles >> /dev/null 2>&1'.format(dir))
        if not match:
          self.logit("The portRoles file on director %s is ok." % dir, altFile=True)
        else:
            dirsWithPortRolesIssue.append(dir)

      if dirsWithPortRolesIssue:
        msg = "A workaround is needed to correct the portRoles file prior to GENU. " \
              "Please\nengage Professional Services to implement the workaround documented " \
              "in KBA\n515617 to correct the portRoles file on the following director(s):"
        self.logit(msg, altFile=True)
        SummaryReport.append(msg)
        for dir in dirsWithPortRolesIssue:
          self.logit(dir, altFile=True)
          SummaryReport.append(dir)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
        self.logit(RECSEP,altFile=True)
      else:
        self.logit("The portRoles files on all directors are ok. No action needed.", altFile=True)
        self.logit(RECSEP,altFile=True)

    except (Exception), e:
        self.logit("Error: %s on analyzing the portRoles files on the directors." % (e),altFile=True)


  def checkVS6ForSLIC4(self):
    checkName = "VS6 SLIC-4 Check"
    dirsMissingSLIC4 = []
    SummaryReport = []
    self.logit(RECSEP,altFile=True)
    self.logit("Checking the portRoles file on the VS6 directors for the temporary IO Module (SLIC-4)...",altFile=True)
    try:
      for dir in self.directorup:
        # Check for A4/B4 entries in portRoles file
        match = os.system('ssh root@{0} grep 4-FC /var/opt/zephyr/flashDir/portRoles >> /dev/null 2>&1'.format(dir))
        if not match:
          self.logit("The SLIC-4 entry was found on director %s." % dir, altFile=True)
        else:
          dirsMissingSLIC4.append(dir)
      if dirsMissingSLIC4:
        msg = "SLIC-4 entries were not detected in the portRoles file of one or more directors\n" \
              "on this VS6, indicating the temporary COM IO module has not been inserted yet, or\n" \
              "was inserted after the director was powered up (and therefore not detected).\n" \
              "A Customer Engineer (CE) should be engaged to verify the SLIC-4 is installed on\n" \
              "the following directors, and install it if missing.  Then follow KBA 524376 for\n" \
              "instructions to follow to ensure the director(s) detect SLIC-4 prior to the\n" \
              "GenU.\n" \
              "The SLIC-4 entries were missing on the following director(s):\n"
        self.logit(msg, altFile=True)
        SummaryReport.append(msg)
        for dir in dirsMissingSLIC4:
          self.logit(dir, altFile=True)
          SummaryReport.append(dir)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
        self.logit(RECSEP,altFile=True)
      else:
        self.logit("SLIC-4 entries found in portRoles for all directors. No action needed.", altFile=True)
        self.logit(RECSEP,altFile=True)
    except (Exception), e:
      self.logit("Error: %s on analyzing the portRoles files for SLIC-4 entries." % (e),altFile=True)


  def callgenuUpgradeCheck(self,subcmd):
    self.logit("Calling the VPlexcli script with the %s option. This will take a couple minutes to complete..." % subcmd)
    status = self.existsTest('/opt/emc/VPlex/jython2.7b4/LibExt/VPlexUpgradeCheckerCli.py')
    if not status:
      self.logit("The /opt/emc/VPlex/jython2.7b4/LibExt/VPlexUpgradeCheckerCli.py is missing.")
      self.logit("Copy the file in place and retry.")
      self.onlyGenuchkLock(action='unlock')
      exit(1)

    checkName = "Importing/executing VPlexcli-level script"
    self.objToolResources.startProgressBar(checkName)
    go = 0
    Host = "localhost"
    timeout = 120
    connection = telnetlib.Telnet(Host, 49500,)
#    while not go:
#      resultin = connection.expect(["VPlexcli:/>","Enter User Name:"], timeout)
#      time.sleep(2)
#      if "VPlexcli:/>" in resultin[2]:
#        break
#      user = raw_input("Enter the VPlexcli username. Type 'Exit' to exit: ")
#      time.sleep(10)
#      if user == 'Exit':
#        connection.close()
#        sys.exit()
#        self.objToolResources.stopProgressBar(error=False)
#      connection.write(user + "\n")
#      time.sleep(3)
#      result = connection.read_until("Password: ",timeout)
#      password = getpass.getpass()
#      connection.write(password +"\n")
#    connection.write('script -u VPlexUpgradeCheckerCli\n')
#    time.sleep(8)
#    connection.write('script -i VPlexUpgradeCheckerCli\n')
    #The import can take a long time to complete on slow running systems
    time.sleep(30)
    print "Writing"
    connection.write('upgradeCheck %s\n' % self.subcmd)
    print "Done"
    time.sleep(8)
    connection.write('exit\n')
    go = 1
    self.objToolResources.stopProgressBar(error=False)

  def executevs6(self):
    self.logit('Starting the GENU analysis for a VS6...')
    # Plug in vs6 fx calls here
    try:
      tailThrd = Thread(target=self.tailFile)
      tailThrd.start()
    except KeyboardInterrupt:
      sys.exit()
    self.checkVS6ForSLIC4()
    #portRoles 'OFF' issue only needs to be checked for 6.0.1.x releases
    if '6.0.1.04.00.09' in self.vers or '6.0.1.05.00.09' in self.vers or '6.0.1.06.00.03' in self.vers or '6.0.1.07.00.04' in self.vers:
        self.logit(RECSEP,verbose=True,altFile=True)
        self.logit('GeoSynchrony %s detected.  Analyzing portRoles for ports in OFF state...' % (self.vers),verbose=True,altFile=True)
        self.checkPortRoles()
    elif '7.5.125.21.0' in self.director_vers or '7.5.127.52.0' in self.director_vers or '7.5.128.5.0' in self.director_vers or '7.5.129.4.0' in self.director_vers:
        self.logit('Director version %s detected. Analyzing portRoles for ports in OFF state...' %(self.director_vers),verbose=True,altFile=True)
        self.checkPortRoles()
    else:
        self.logit('Version is not 6.0.x, skipping the portRoles analysis for ports in OFF state.',verbose=True,altFile=True)
        self.logit(RECSEP,altFile=True)
    self.vplexPlatHC()
    self.objToolResources.storeVplexUpgradeCheckerDict()
    self.callgenuUpgradeCheck(self.subcmd)

  def execute(self):
    self.logit("Collecting the GENU pre-check analysis & output of the commands required for the GENU CCA process:",verbose=True,altFile=True)
    try:
      tailThrd = Thread(target=self.tailFile)
      # at this point we can't run this as a daemon. If we do the tail dies when the linux script is done
      tailThrd.start()
    except KeyboardInterrupt:
      sys.exit()
    self.logit(RECSEP,verbose=True,altFile=True)
    # 6.0.1.04.00.09 is the only version that needs the perms check
    if '6.0.1.04.00.09' in self.vers or '7.5.125.21.0' in self.director_vers:
      perms = '-rw-r--r--'
      status = self.checkFilePerms(perms,'/var/opt/zephyr/flashDir/portLayout')
    else:
      self.logit(RECSEP,verbose=True,altFile=True)
      self.logit('Version is not 6.0.1 P4, skipping the portLayout permissions check.',verbose=True,altFile=True)
    self.logit(RECSEP,verbose=True,altFile=True)
    #adding sleep before calling FLAT to ensure all prior logging to console complete before FLAT runs
    time.sleep(10)
    self.flatCheck() # Firmware log analysis done by FLAT modules.
    self.oldRoutesWarning()
    self.checkAndLogCSTLockBox() # Check and log CST lockbox status on this management server
    self.objToolResources.storeVplexUpgradeCheckerDict()
    self.callgenuUpgradeCheck(self.subcmd)

  def executendu(self,subcmd):
    try:
      tailThrd = Thread(target=self.tailFile)
      # At this point we can't run in daemon mode or tailFile will die when the linux script finishes
      tailThrd.start()
    except KeyboardInterrupt:
      sys.exit()
    self.nduOsCommands()
    time.sleep(10)
    if self.cca_review:
      self.flatCheck() # Firmware log analysis done by FLAT module
    self.callgenuUpgradeCheck(self.subcmd)

  def runndu(self,subcmd):
    self.executendu(subcmd)

  def run(self):
    self.execute()

  def runvs6(self):
    self.executevs6()

def main():
  # Before continueing we need two critical pieces of data as the checks will vary based on these
  # data points. We need the platform and the version number
  """
  Get the options and args for processing
  """
  desc = "Usage: ./%prog -u ndu/genu\n"
  parser = OptionParser(option_class=MyOption, usage = desc)
  parser.add_option('-u','--upgradetype', action='store', type='choice', choices=['ndu','genu'], default=False,dest='upgradetype',
                    help='Valid args are: \'ndu\' or \'genu\' or \'ndu --checks <check to run> --cca <DayOfUpgrade or SolVeCCAReview>\'.'
                         '\n\t\t\t                                                                                         ndu:'                                         '                                                                 Performs a number of checks, and collects the output'
                         ' from a set of commands required for a NDU on any VS2 or VS6 system running GeoSynchrony 6.0 P1 and higher. Also the user has the flexibility to select whether the tool performs ndu checks (checks option), collects VPlexcli command output for CCA review (cca option) or both.  If the user doesn\'t add these options then the tool will only run all ndu checks by default.  Reference the README for the SolVe CCA review command list options and examples.\n\t\t\t'

                         '\n\t\t\tgenu:                                                Performs the checks required for a GenU on a VS2 or VS6 '
                         '(GeoSynchrony 6.0.1 P4 and higher), including the collection of the output of a set of commands required from a'
                         ' VS2 for a GenU. To run the GenU option = ./VPlexUpgradeChecker.py -u genu')

  parser.add_option('-c','--checks', type='choice', choices=['checkForMemoryFragmentation', 'checkCertificateStatus','checkSfpVoltage'], default=False, help='Examples:\n\t\t\t\t\t\t\nNDU Checks = ./VPlexUpgradeChecker.py -u ndu --checks "checkSfpVoltage,checkForMemoryFragmentation"\n'
    '\n\t\t\t These are the available checks that can be run with the ndu command:'
    '\n\t\t\t checkForMemoryFragmentation - To validate whether the system has or about to hit memory fragmentation.'
    '\n\t\t\t checkCertificateStatus      - To check the validity of the VPN, WEB and CA certificate(s).'
    '\n\t\t\t checkSfpVoltage             - To check the RX/TX power levels of the port(s).')

  parser.add_option('-a','--cca', type='choice', choices=['DayOfUpgrade','SolVeCCAReview'], help='Examples:\n\t\t\t\t\t\t\nNDU Checks = ./VPlexUpgradeChecker.py -u ndu --cca "DayOfUpgrade"\n\t\t\t\t\t\t\t\t\t'
    '\n\t\t\t These are the available cca options that can be run with the ndu command:'
    '\n\t\t\t DayOfUpgrade  - Collects a smaller set of VPlexcli command output needed on the day of the upgrade, reference the README for the list of commands.'
    '\n\t\t\t SolVeCCAReview  - Collects the full set of VPlexcli command output needed for a NDU SolVe CCA review, reference the README for the list of commands.')

  (options, args) = parser.parse_args()
  if  options.upgradetype == False:
    parser.error("Arguments not set properly, please use the --help option and then retry.")
  if options.upgradetype == "genu" and options.cca:
    parser.error("Arguments not set properly. You can not use cca option under genu.")
  if options.upgradetype == "genu" and options.checks:
    parser.error("Arguments not set properly. You can not use checks option under genu.")
  HW=None
  version = ""
  director_version = ""
  if options.upgradetype == 'genu':
    SubCmd = 'rungenu'
  else:
    if options.checks and options.cca:
      SubCmd = 'runnducheck -c %s -a %s' %(options.checks, options.cca)
    elif options.checks:
      SubCmd = 'runnducheck -c %s' %(options.checks)
    elif options.cca:
      SubCmd = 'runnducheck -a %s' %(options.cca)
    else:
      SubCmd = 'runnducheck'
  if SubCmd.startswith('runnducheck'):
    live_obj = genu_check(HW,SubCmd,version,director_version, options.cca)
    live_obj.runndu(SubCmd)
  elif SubCmd:
    print("Collecting the GeoSynchrony version and VPLEX platform.  This will take a couple minutes to complete...")
    plat = False
    Host = "localhost"
    timeout = 120
    # We have to run some pre-processing items as the user may not know the plat/vers
    connection = telnetlib.Telnet(Host, 49500,)
    while not plat:
      resultin = connection.expect(["VPlexcli:/>","Enter User Name:"], timeout)
      time.sleep(2)
      if "VPlexcli:/>" in resultin[2]:
        break
      user = raw_input("Enter the VPlexcli username. Type 'Exit' to exit: ")
      time.sleep(10)
      if user == 'Exit':
        connection.close()
        sys.exit()
      connection.write(user + "\n")
      time.sleep(3)
      result = connection.read_until("Password: ",timeout)
      password = getpass.getpass()
      connection.write(password +"\n")
      time.sleep(3)
    connection.write('ls -t /engines/engine-*-1/::engine-family\n')
    time.sleep(9)
    connection.write('version -a\n')
    #Changed the sleep time after executing version command from 9 to 90 sec, as the execution of this command takes more time when VPN is down.
    time.sleep(90)
    connection.write('exit\n')
    time.sleep(9)
    result = connection.read_all()
    pat=re.compile(r'engine-family\s+(VPL|Argonaut)\s+', re.M)
    plat = re.search(pat,result)
    if plat:
      HW = plat.group(1)
      print("The VPLEX platform found is: %s" % HW)
    product_version_info=re.findall(r'Product Version.*\s', result)
    try:
      if product_version_info:
        for s in product_version_info:
          vers = re.split(r'\s{2,}', s)
        version = vers[1]
        print("The 'Product version' detected in the 'version -a' output is: %s" % version)
    except (Exception), e:
      print("Error: %s while fetching the product version." % e)
    #Commenting out the code to fetch branch from version as director version is used instead. Below code can be used somewhere else if needed.
    #mgmt_server_version_pat=re.compile(r'Mgmt Server Software.*?([1-9].*?)\s', re.M)
    #mgmt_server_version = re.search(mgmt_server_version_pat,result)
    #if mgmt_server_version:
      #branch = mgmt_server_version.group(1)
    director_version = []
    director_info = re.findall(r'director-.*\s', result, re.M)
    try:
      if director_info:
        for ver in director_info:
          dir_version = ver.split()[1]
          director_version.append(dir_version)
    except (Exception), e:
      print("Error: %s while fetching the director version." % e)
    #if we are performing a GenU then we are concerned about the version
  if SubCmd == 'rungenu':
      try:
        live_obj = genu_check(HW, SubCmd, version, director_version)
        if '6.0.1.04.00.09' in version or '6.0.1.05.00.09' in version or '6.0.1.06.00.03' in version or '6.0.1.07.00.04' in version or '6.1.0.00.00.17' in version or '6.1.0.00.00.23' in version or '6.1.0.01.00.13' in version or '6.1.0.02.00.04' in version or '6.2.0.00.00.32' in version or '6.2.0.02.00.04' in version or '6.2.0.03.00.02' in version or '6.2.0.04.00.07' in version:
            #live_obj = genu_check(HW, SubCmd, version)
            live_obj.logit('The GeoSynchrony version is %s' % version,verbose=True)
            live_obj.logit('This version is supported, proceeding...',verbose=True)
            if HW == 'Argonaut':
              live_obj.objToolResources.addDirectorVersionToTempFile(director_version)
              live_obj.run()
            elif HW == 'VPL':
              #dir version currently only used in Cli script for VS6 GenU check, if that changes this will need to move
              live_obj.objToolResources.addDirectorVersionToTempFile(director_version)
              live_obj.runvs6()

        elif 'mixed' in version:
          if 'n/a' in director_version:
            warning_msg = "ERROR: Mixed product version detected. Either one or more directors is/are\n" \
                        "down or the VPN tunnel to the remote management-server is down.\n" \
                        "To fix the VPN connectivity, a Customer Engineer (CE) should work\n" \
                        "with the customer to follow KBA 448400 'VPLEX: After an IP network\n" \
                        "outage VPN is down' to troubleshoot the VPN connectivity issue.\n" \
                        "If the director or VPN connectivity can't be fixed before the tool needs to be run\n" \
                        "then the user must log into the VPlexcli and use the 'disconnect' command\n" \
                        "to disconnect the unreachable directors from VPlexcli. Then re-run the tool.  Exiting...\n"
            print(warning_msg)
            live_obj.logit(RECSEP)
            live_obj.logit(warning_msg)
            live_obj.onlyGenuchkLock(action='unlock')
            sys.exit()
          else:
            warning_msg = "ERROR: Mixed product version detected, but all directors appear to be connected.\n" \
                        "This indicates the connected directors are running different GeoSynchrony versions.\n" \
                        "All directors must be running the same supported GeoSynchrony version (reference the\n" \
                        "README for supported versions) in order for the tool to proceed.  Please upgrade the\n" \
                        "directors as needed to achieve this, and then re-run the tool.  Exiting...\n"
            print(warning_msg)
            live_obj.logit(RECSEP)
            live_obj.logit(warning_msg)
            live_obj.onlyGenuchkLock(action='unlock')
            sys.exit()
        #elif '50.10.0.24' in branch or '50.40.0.42' in branch or '50.40.1.1' in branch or '50.40.2.4' in branch:
        elif 'Version mismatch' in version:
          if 'n/a' in director_version:
            warning_msg = "ERROR: Version mismatch detected with director(s) shown with 'n/a' version\n" \
                        "Either one or more directors is/are down or the VPN tunnel to the\n" \
                        "remote management-server is down.\n" \
                        "To fix the VPN connectivity, a Customer Engineer (CE) should work\n" \
                        "with the customer to follow KBA 448400 'VPLEX: After an IP network\n" \
                        "outage VPN is down' to troubleshoot the VPN connectivity issue.\n" \
                        "If the director or VPN connectivity can't be fixed before the tool needs to be run\n" \
                        "then the user must log into the VPlexcli and use the 'disconnect' command\n" \
                        "to disconnect the unreachable directors from VPlexcli. Then re-run the tool.  Exiting...\n"
            print(warning_msg)
            live_obj.logit(RECSEP)
            live_obj.logit(warning_msg)
            live_obj.onlyGenuchkLock(action='unlock')
            sys.exit()
          elif '7.5.125.21.0' in director_version or '7.5.127.52.0' in director_version or '7.5.128.5.0' in director_version or '7.5.129.4.0' in director_version or '7.7.448.0.0' in director_version or '7.7.486.0.0' in director_version or '161.1.0.78.0' in director_version or '161.2.0.24.0' in director_version or '162.0.0.422.0' in director_version or '162.2.0.23.0' in director_version or '162.3.0.6.0' in director_version or '162.4.0.13.0' in director_version:
            msg = "WARNING: 'Version mismatch' product version detected, but the discovered\n" \
                "directors are running a supported GeoSynchrony version. The tool will proceed\n" \
                "based on the GeoSynchrony version running on the directors...\n"
            print(msg)
            #live_obj = genu_check(HW, SubCmd, director_version)
            live_obj.logit(RECSEP,verbose=True)
            live_obj.logit(msg,verbose=True)
            live_obj.logit(RECSEP,verbose=True)
            if HW == 'Argonaut':
              live_obj.objToolResources.addDirectorVersionToTempFile(director_version)
              live_obj.run()
            elif HW == 'VPL':
              #dir version currently only used in Cli script for VS6 GenU check, if that changes this will need to move
              live_obj.objToolResources.addDirectorVersionToTempFile(director_version)
              live_obj.runvs6()
          else:
            msg = "ERROR: 'Version mismatch' product version detected, and none of the discovered\n" \
                "directors are running a supported GeoSynchrony version (reference the README for\n" \
                "supported versions). Upgrade to a supported version and then re-run the tool. Exiting...\n"
            print(msg)
            live_obj.logit(RECSEP)
            live_obj.logit(msg)
            live_obj.onlyGenuchkLock(action='unlock')
            sys.exit()
        else:
          live_obj.logit(RECSEP)
          version_msg = ("\nThe GeoSynchrony version is: %s\n" % version)
          print(version_msg)
          live_obj.logit(version_msg)
          msg = "The GenU option is not supported with this version of GeoSynchrony.\n" \
              "Either upgrade to 6.0.1 P4 or higher, or engage a FSS to manually check\n" \
              "if the system is ready for a GENU. Exiting...\n"
          print(msg)
          live_obj.logit(msg)
          live_obj.onlyGenuchkLock(action='unlock')
          sys.exit()

      except (Exception), e:
        print("Error analyzing the GeoSynchrony version.  Error: %s" %e)


if __name__ == "__main__":
  main()
