#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright © 2018-2021 Dell Inc. or its subsidiaries.  All Rights Reserved.
#
# This software contains the intellectual property of Dell Inc. or is
# licensed to Dell Inc. from third parties. Use of this software and the
# intellectual property contained therein is expressly limited to the terms and
# conditions of the License Agreement under which it is provided by or on behalf
# of Dell Inc. or one of its subsidiaries.


__author__ = "Dell Inc."
__date__ = "2021"
__version__ = "2.8"

__all__ = ['UpgradeCheckerVPlexcli']

import os, sys, re, tarfile
import shutil
import glob as osglob
import string
import yyapi
from com.emc.vplex.cli.ICommand import RegistrationInfo
from yyapi import glob
import time
import dapi
from yyapi import BusinessLayer
from com.emc.vplex.smsv2.firmware import CommandResult
from com.emc.vplex.smsv2.cli import MainContextFactory
from VPN.API.src.VPNCallHomeManager import VPNState
import telnetlib
import UpgradeCheckerTool_common
# json import fails on GEO 5.x but json is not needed to run the 5.x NDU date collection
try:
  import json
except:
  pass
try:
    from cStringIO import StringIO
except ImportError:
    # python >= 3
    from io import StringIO
from java.time import LocalDate
from java.time.temporal import ChronoUnit
from java.time.format import DateTimeFormatter
from VPN.API.src.VPlexCertRenewalMediator import VPlexCertRenewalMediator
from VPN.API.src.VPlexCertRenewalData import VPlexCertRenewalData

logStartTime = time.strftime("%Y-%m-%d-%H.%M.%S.%Z", time.localtime())
LOG_FILE_FINAL = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_Upgrade_Checker_' + str(logStartTime) + '.log'
LOG_DIR='/var/log/VPlex/cli/VPLEX_Upgrade_Checker'
LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_Upgrade_Checker_.log'
C_LOG_FILE = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/Cn.log'
NDU_SESSION_FILE = '/var/log/VPlex/cli/capture/ndu-pre-check-session.txt'
LOCK_FILE = '/home/service/lock_upgrade'
LOCK_FILE_LINUX = '/home/service/lock_upgrade_linux'
LOCAL = 'Local'
METRO = 'Metro'
OBJTOOLRESOURCES = UpgradeCheckerTool_common.ToolResources()
CERTIFICATE_EXP_DAYS = 30
HOST_CERT_WARNING = "WARNING: The Host certificate /etc/ipsec.d/certs/hostCert.pem on the "\
    "management-server will expire within a month. Refer to the 'Renew Security Certificates' "\
    "procedure in SolVe Desktop and renew the Host certificate.\n"
HOST_CERT_ERROR = "ERROR: The Host certificate /etc/ipsec.d/certs/hostCert.pem on the "\
    "management-server has expired. Refer to the 'Renew Security Certificates' procedure in SolVe "\
    "Desktop and renew the Host certificate.\n"
WEB_CERT_WARNING = "WARNING: The WEB certificate /etc/ipsec.d/certs/webServerHostCertFile.pem on"\
    " the management-server will expire within a month. Refer to the 'Renew Security Certificates' "\
    "procedure in SolVe Desktop and renew the WEB certificate.\n"
WEB_CERT_ERROR = "ERROR: The WEB certificate /etc/ipsec.d/certs/webServerHostCertFile.pem on the"\
    " management-server has expired. Refer to the 'Renew Security Certificates' procedure in SolVe "\
    "Desktop and renew the WEB certificate.\n"
CA_CERT_WARNING = "WARNING: The CA certificate /etc/ipsec.d/cacerts/strongswanCert.pem on the "\
    "management-server will expire within a month. Refer to the 'Renew Security Certificates' "\
    "procedure in SolVe Desktop and renew the CA certificate.\n"
CA_CERT_ERROR = "ERROR: The CA certificate /etc/ipsec.d/certs/webServerHostCertFile.pem on the "\
    "management-server has expired. Refer to the 'Renew Security Certificates' procedure in SolVe "\
    "Desktop and renew the CA certificate.\n"


class upgradeCheck(yyapi.PyCmd):

  def __init__(self):
    yyapi.PyCmd.__init__(self)
    self.logname = LOG_FILE
    self.clog = C_LOG_FILE
    self.C_LOG_FILE_FINAL = self._createCLog()
    self.reportbrk = '==========================================================\n'
    self.objToolResources = OBJTOOLRESOURCES

  def getCommandRegistrationInfo(self):
    from com.emc.vplex.cli import ICommand
    return [ICommand.RegistrationInfo(True, '/', 'upgradeCheck')]

  def logit(self, msg, verbose=False, verbosenotime=False, altFile=False):
    now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
    logfile = open(self.logname,'a')
    if verbosenotime:
      logfile.write('%s\n' % (msg))
      logfile.close()
    else:
      logfile.write('[%s] %s\n' % (now, msg))
      logfile.close()
    if altFile:
      now = time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime())
      logfilealt = open(self.clog,'a')
      if verbosenotime:
        logfilealt.write('%s\n' % (msg))
        logfilealt.close()
      else:
        logfilealt.write('[%s] %s\n' % (now, msg))
        logfilealt.close()

  def _getClusterOneEngAttributes(self):
    cmd = '**/engine-1-1/'
    return yyapi.glob(cmd)

  def _getClusterTwoEngAttributes(self):
    cmd = '**/engine-2-1/'
    return yyapi.glob(cmd)

  def _createCLog(self):
    #create log file with cluster ID and last 4 of TLA for SolVe Activity requirement
    cluster = " "
    try:
        engine = self._isItCluster1()
        if engine:
            cluster = 'cluster-1'
            TLA = self._getTLA(cluster)
            lastFour = TLA[-4:]
            existingFile = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/C1_' + str(lastFour) + '.log'
            try:
              os.remove(existingFile)
            except OSError:
              pass
            cLogFile = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/C1_' + str(lastFour) + '.log'
            return cLogFile

        else:
            cluster = 'cluster-2'
            TLA = self._getTLA(cluster)
            lastFour = TLA[-4:]
            existingFile = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/C2_' + str(lastFour) + '.log'
            try:
              os.remove(existingFile)
            except OSError:
              pass
            cLogFile = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/C2_' + str(lastFour) + '.log'
            return cLogFile
    except (Exception), e:
        self.logit("Error creating the analysis/command collection log file. %s" %e ,verbose=True)

  def _isItCluster1(self):
    try:
        out = yyapi.glob('/management-server')
        ipseed = str(out[0].getAttributes()[0].getValue())
        if '1' in ipseed:
            # seed is 1
            return True
        else:
            # seed is not 1
            return False
    except (Exception), e:
        self.logit("Error getting cluster-ip-seed. %s" %e ,verbose=True)

  def _getProductType(self):
    try:
      result = yyapi.cmd("configuration get-product-type").getResult()
      if LOCAL in result[0]:
        return "LOCAL"
      else:
        return "METRO"
    except(Exception), e:
      self.logit("Unable to retrieve Product Type ( Metro|Local)", verbose=True)
      return

  def _getTLA(self, cluster):
    try:
        if cluster == 'cluster-1':
            out = self._getClusterOneEngAttributes()
            TLA = str(out[0].getBackingObject().getTopLevelAssembly())
            return TLA
        else:
            out = self._getClusterTwoEngAttributes()
            TLA = str(out[0].getBackingObject().getTopLevelAssembly())
            return TLA
    except (Exception), e:
        self.logit("Error getting the TLA. %s" %e ,verbose=True)

  def _getVersion(self):
    try:
      version = "N-A"
      result = yyapi.cmd("version -a").getResult()
      result = result.versionData
      for v in result:
        if (str(v.shortName).strip() == "Product Version") and v.shortVersion:
          version = v.shortVersion
          break
    except (Exception, JavaException), e:
      version = "N-A"
      self.logit(version)
    self.logit("Product version is GeoSynchrony %s" % version,verbose=True )
    self.logit(self.reportbrk,verbose=True)
    return version

  def _getVSx(self):
    self.logit("Determining the VPLEX platform (hardware) type...",verbose=True)
    try:
      out = yyapi.glob('**/engine-*-1/')
      allResults = str(out[0].getBackingObject().getEngineFamily())
      if 'Argonaut' in allResults:
        self.logit("Found Argonaut = VS2 hardware",verbose=True)
        self.logit(self.reportbrk,verbose=True)
        return "VS2"
      elif 'VPL' in allResults:
        self.logit("Found VPL = VS6 hardware",verbose=True)
        self.logit(self.reportbrk,verbose=True)
        return "VS6"
      else:
        self.logit("Platform search either returned VS1, or didn't complete successfully. This may lead the tool to exit,",verbose=True)
        self.logit("or to return an inaccurate platform type in the logs...",verbose=True)
        return "VS1"
    except(Exception), e:
      self.logit("Error getting hw platform. %s" %e ,verbose=True)

  def _dumpCons(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    capture = StringIO()
    save_stdout = sys.stdout
    sys.stdout = capture
    try:
      yyapi.cmd(cmd)
    finally:
      sys.stdout = save_stdout
    self.logit("%s" % capture.getvalue())

  def _dumpEntire(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    self.logit(self.reportbrk,verbose=True)
    try:
      inits=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmd).result)
      self.logit(inits,verbosenotime=True,altFile=True)
    except(Exception), e:
      self.logit("Running: %s failed" % e,verbose=True )


  def _reNamereMove(self, staticlog, newlog):
    if os.path.exists(staticlog):
      try:
        os.rename(staticlog,newlog)
      except (Exception), e:
        self.logit("Error: %s rename of static log file failed." % e,verbose=True )

  def _lock(self,action=False):
    if os.path.isfile(LOCK_FILE) and action == 'lock':
      return 'Locked'
    elif not os.path.isfile(LOCK_FILE) and action == 'lock':
      lockinfo = {'instance': self.instance,'time':time.time()}
      file(LOCK_FILE, 'w').write(str(lockinfo))
    elif action == 'unlock':
      try:
        if os.path.isfile(LOCK_FILE_LINUX):
          self.logit('Removing Linux-level lockfile.')
          os.remove(LOCK_FILE_LINUX)
        else:
          pass
        self.logit('Removing VPlexcli-level lockfile.')
        os.remove(LOCK_FILE)
      except:
        self.logit("Lock file %s, removal failed, delete if no other instances are running" % LOCK_FILE,verbose=True)

  def _checkForVS2NonStandardWwns(self):
    checkName = "Check for Non-Standard WWNs on a VS2"
    SummaryReport = []
    nonStdWwnDirs = []
    directors = yyapi.glob("/engines/*/directors/*")
    for director in directors:
      cmd_result = yyapi.cmd('emc-internal has-nonstandard-port-wwns '
                             '--director-ip %s' %
                             director.getAttributeValue('hostname'))
      json_result = json.loads(cmd_result.getResult())
      has_nonstandard = json_result.get('result')
      if has_nonstandard is None:
        raise Exception('Unable to determine %s WWN compatibility, please use KBA 487310.  Error returned: %s' %
                        (director.name, json_result.get('error')))
      elif has_nonstandard:
        nonStdWwnDirs.append(director)
      else:
        continue
    if nonStdWwnDirs:
      msg = 'Non-standard WWPNs have been found on the VS2 system, and the GenU can not be\n' \
            'performed in this state. A Customer Engineer (CE) should follow the\nworkaround ' \
            'in KBA 487310 to convert the non-standard WWPNs to\nstandard WWPNs prior to the GenU ' \
            '(engage a FSS if needed).\nThe directors with non-standard WWNs are:'
      self.logit(msg,verbose=True,altFile=True)
      SummaryReport.append(msg)
      for director in nonStdWwnDirs:
        self.logit(director,verbose=True,altFile=True)
        SummaryReport.append(director)
      self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
    else:
      self.logit("This VS2 has standard port WWNs and is eligible for GENU to VS6.",verbose=True,altFile=True)


  def _getClusterName(self, engine):
    self.logit("Getting the name of the local cluster for the export storage-view find command...",verbose=True)
    try:
      out = yyapi.glob('/clusters/*')
      if engine:
        c1Name = out[0].getAttributes()[15].getValue()
        self.logit("The cluster-1 name is: %s" % c1Name,verbose=True)
        return c1Name
      else:
        c2Name = out[1].getAttributes()[15].getValue()
        self.logit("The cluster-2 name is: %s" % c2Name,verbose=True)
        return c2Name
    except(Exception), e:
      self.logit("Error getting cluster name: %s" %e ,verbose=True)

  def _onlyRunNduCmd(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    self.logit("NOTE: The output of this cmd will print to the console after it finishes. This will take time to complete...",verbose=True)
    try:
      inits=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmd).result)
    except(Exception), e:
      self.logit("Running: %s failed" % e,verbose=True )

  def _appendNduPreCheck(self):
    #This is needed to ensure all output from the 'ndu pre-check --verbose' is included in our log files.
    nduPreCheckFile = open(NDU_SESSION_FILE,'r')
    nduPreCheckFile.seek(0)
    try:
      for line in nduPreCheckFile:
        self.logit(line, verbosenotime=True, altFile=True)
    except (Exception), e:
      self.logit("Failed to append the NDU pre-check session file to the log.  Error: %s" % (cmd,e) ,verbose=True)
    nduPreCheckFile.close()

  def _towerComponentExists(self, towerCommandList, directors=[]):
    """
      Check whether a tower component exists
        True:  if all the tower commands given in 'towerCommandList' execute with no errors
        False: otherwise
    """
    if towerCommandList is None:
        return False
    if not isinstance(towerCommandList, list):
        towerCommandList = [towerCommandList]
    if not directors:
        directors = yyapi.glob('/**:::director')
    if not isinstance(directors, list):
        directors = [directors]
    for cmd in towerCommandList:
        try:
            result = yyapi.nsfwcmd(directors[0], cmd)
            if isinstance(result, yyapi.CommandResult):
                rc = result.getReturnCode()
            elif isinstance(result, yyapi.attrdict) and 'STATUS' in result:
                rc = result['STATUS']
            else:
                return False
            if rc == CommandResult.NSFW_RETURN_CODE_NO_SUCH_COMMAND or \
               rc == CommandResult.NSFW_RETURN_CODE_NO_SUCH_COMPONENT:
                return False
        except(Exception), e:
            self.logit("Firmware Command exception", verbose=True, altFile=True)
            return False
    return True

  def _checkForMemoryFragmentation(self):
    platform = self._getVSx()
    checkName = "Check for Memory Fragmentation on a %s" % platform
    if self._towerComponentExists('mem jems'):
       self.logit("Firmware is already using jemalloc, Skipping Memory " \
                  "Fragmentation check ", verbose=True, altFile=True)
       return
    SummaryReport = []
    dirs_with_mem_frag = []
    directors = yyapi.glob("/engines/*/directors/*")
    for d in directors:
        misc_stats = [h for h in yyapi.nsfwcmd(d, 'mem fcs')['heaps']
                      if h['name'] == 'misc'][0]
        chunk_count = 0
        for t in misc_stats['allocTypes']:
            if not isinstance(t, dict):
                continue
            for bin_size, details in t.iteritems():
                if bin_size.isdigit() and int(bin_size) >= 4096:
                    chunk_count += int(details['count'])
        if chunk_count >= 20000:
            dirs_with_mem_frag.append(d)
    if dirs_with_mem_frag:
        msg = "Directors "
        SummaryReport.append(msg)
        self.logit(msg, verbose=True, altFile=True)
        for director in dirs_with_mem_frag:
            SummaryReport.append(director)
            self.logit(director,verbose=True,altFile=True)
        msg = "were detected to have memory fragmentation.  Engage the VPLEX Remote " \
              "Reactive team to analyze this.  If the VPLEX is a VS6 Metro running " \
              "GeoSynchrony 6.1.x the memory fragmentation issue is documented in " \
              "KBA 538517.  Engage the VPLEX Remote Reactive team to assess how to " \
              "handle this prior to the NDU, the directors will likely need to be " \
              "rebooted.  Note that the memory fragmentation is triggered by workload " \
              "changes, and can appear/disappear within hours as the workload fluctuates, " \
              "meaning the memory fragmentation could still reoccur just prior to the " \
              "director firmware upgrade.  It's recommended to perform NDU during off-peak " \
              "hours, particularly when read-intensive workloads are minimal (if possible) " \
              "to avoid disruption to the NDU."
        SummaryReport.append(msg)
        self.logit(msg, verbose=True, altFile=True)
    if SummaryReport:
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
    else:
        self.logit("No Memory Fragmentation issues were found on the directors.",verbose=True,altFile=True)

  def _checkSfpVoltage(self):
    checkName = "Check Enabled Ports' RX/TX Power"
    SummaryReport = []
    dironlylst = []
    prtuptlst = []
    getprt = re.compile(r'(^[AB].*)\s\d.*\s\w.*\s(up)',re.M)
    # This will be used to parse the active director names
    cmd = 'director uptime'
    try:
      result = yyapi.cmd(cmd).getResult()
      uptlst = result.split("\n")
    except (Exception), e:
      self.logit("Failed to run director uptime  Error: %s" % (cmd,e) ,verbose=True)
    getdir = re.compile(r'(^D.*)\s(direct.*)\:.*',re.M)
    for line in uptlst:
      dirs = re.search(getdir,line)
      if dirs:
        # This gives us the list of directors to work with in STRING form.
        dironlylst.append(dirs.group(2))
    for line in dironlylst:
      sfpstocheck = []
      # List out the ports. If the port is up append group(1) to a list
      cmd = 'll /eng*/eng*/directors/{0}/hardware/ports/'.format(line)
      try:
        result = MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmd).result)
        resultmsg = result.encode('ascii','ignore')
        if not resultmsg:
          self.logit("Failed ll for listing of %s is not getting populated result is %s" % (line,result) ,verbose=True)
        else:
          prtuptlst = resultmsg.split("\n")
      except (Exception), e:
        self.logit("Failed to run the ll to get the ports CMD %s Error: %s " % (cmd,e) ,verbose=True)
      for prtline in prtuptlst:
        sfpsup = re.search(getprt,prtline)
        if sfpsup:
          sfpstocheck.append(sfpsup.group(1))
      for sfp in sfpstocheck:
         # Get the power values
        cmdrx = "ls -t /engines/eng*/directors/{0}/hardware/sfps/{1}::current-rx-power".format(line,sfp.rstrip())
        cmdtx = "ls -t /engines/eng*/directors/{0}/hardware/sfps/{1}::current-tx-power".format(line,sfp.rstrip())
        rxvoltage=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmdrx).result)
        txvoltage=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmdtx).result)
        rxpat = re.compile(r'current\-rx\-power\s+(\d+)',re.M)
        txpat = re.compile(r'current\-tx\-power\s+(\d+)',re.M)
        rxpwr=re.search(rxpat,rxvoltage)
        txpwr=re.search(txpat,txvoltage)
        if rxpwr and txpwr:
          # Check the rx/tx power values
          if int(rxpwr.group(1)) <= 200 or int(txpwr.group(1)) <= 200:
            msg = "WARNING: %s port %s was found to have low RX and/or TX power (<=200uW):\nRX power= %s\n" \
                  "TX power= %s\nA Customer Engineer(CE) should reference KBA 336564 for steps to troubleshoot this issue.\n"                                                                                                  % ( line,sfp,rxpwr.group(1),txpwr.group(1))
            SummaryReport.append(msg)
            self.logit(msg, verbose=True, altFile=True)
    if SummaryReport:
      self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
    else:
      self.logit("No issues were found with the VPLEX ports' RX/TX power levels.",verbose=True,altFile=True)

  def _tarLogs(self):
    # At this point the logs have been renamed so no logit
    # GS requested to include the TLA & time stamp in the .tar name
    # tarfile.open does not work with variables in this script
    # Printing to end of debug log in case we need to see what tar did.
    fp = open(LOG_FILE_FINAL, 'a')
    print >> fp, ("*****************************************************************************************")
    print >> fp, ("Calling the function to tar the log files in /var/log/VPlex/cli/VPLEX_Upgrade_Checker/...")
    try:
      engine = self._isItCluster1()
      if engine:
        cluster = 'cluster-1'
        TLA = self._getTLA(cluster)
      else:
        cluster = 'cluster-2'
        TLA = self._getTLA(cluster)
      tarFileName = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEXUpgradeCheckerAnalysis_' + TLA + '_' + str(logStartTime) + '.tar.gz'
      print >> fp, ("Creating the .tar file...")
      tar = tarfile.open(tarFileName, "w:gz")
      for filename in os.listdir(LOG_DIR):
        if not 'VPLEXUpgradeCheckerAnalysis' in filename:
          print >> fp, ("Adding %s to tar file" % filename)
          tar.add( LOG_DIR + "/" + filename, arcname=filename)
      tar.close()
    except(Exception), e:
      print >> fp, ("Error creating tarfile: %s." % e )
    fp.close()
    return

  def _getPorts(self):
    try:
      ports = yyapi.glob('/engines/engine-*/directors/*/hardware/ports/*')
      return ports
    except (Exception), e:
      self.logit("Error getting the VS6 ports.  Error: %s" %e,verbose=True)


  def _checkDisabledVs6WanPorts(self):
    checkName = "Check for Disabled VS6 WAN Ports"
    disabledWanPorts = []
    SummaryReport = []
    try:
      ports = self._getPorts()
      for p in ports:
        port = p.getBackingObject()
        if 'wan-com' in str(port.getRole()):
          if port.getEnabled():
            continue
          else:
            disabledWanPorts.append(port)
      if disabledWanPorts:
        msg = "The following WAN COM ports were found to be disabled. A Customer Engineer (CE)\nshould manually enable " \
             "them prior to the GenU, and ensure they have 'no-link'\nstatus (no cabling). Reference the Master Document for more details.\n"
        self.logit(msg, verbose=True, altFile=True)
        SummaryReport.append(msg)
        for p in disabledWanPorts:
          name = p.getDirector().toString() + ' ' + p.getName()
          msg = ("%s\n" % name)
          SummaryReport.append(msg)
          self.logit(msg, verbose=True, altFile=True)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      else:
        self.logit("There's either no WAN COM SLIC on this VS6, or all WAN COM ports ",verbose=True,altFile=True)
        self.logit("are enabled. No action needed.",verbose=True,altFile=True)
    except (Exception), e:
      self.logit("Error checking if VS6 WAN COM ports are disabled.  Error: %s" %e,verbose=True)


  def _checkUpVs6WanPorts(self):
    checkName = "Check for 'Up' VS6 WAN Ports"
    upWanPorts = []
    SummaryReport = []
    try:
      ports = self._getPorts()
      for p in ports:
        port = p.getBackingObject()
        if 'wan-com' in str(port.getRole()):
          if port.getEnabled():
            if 'up' in str(port.getPortStatus()):
              upWanPorts.append(port)
      if upWanPorts:
        msg = "The following WAN COM ports were found to be enabled, with 'up' port status,\n" \
              "indicating there is cabling/connectivity. All WAN COM ports should have no\n" \
              "cabling/connectivity prior to the GenU. A Customer Engineer (CE) should remove all cabling\n" \
             "and then verify the WAN ports all have status 'no-link' in VPlexcli.\n"
        self.logit(msg, verbose=True, altFile=True)
        SummaryReport.append(msg)
        for p in upWanPorts:
          name = p.getDirector().toString() + ' ' + p.getName()
          msg = ("%s\n" % name)
          self.logit(msg, verbose=True, altFile=True)
          SummaryReport.append(msg)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      else:
        self.logit("There's either no WAN COM SLIC on this VS6, or no WAN COM ports were found",verbose=True,altFile=True)
        self.logit("in 'up' state). No action needed.",verbose=True,altFile=True)
    except (Exception), e:
      self.logit("Error checking if VS6 WAN COM ports are 'Up'.  Error: %s" %e,verbose=True)


  def _getValidDays(self, certObj):
    current = LocalDate.now()
    formatter = DateTimeFormatter.ofPattern("MMM d H:m:s yyyy z")
    start = LocalDate.parse(' '.join(certObj.startDate.split()), formatter)
    end = LocalDate.parse(' '.join(certObj.expirationDate.split()), formatter)
    elapsed_period = ChronoUnit.DAYS.between(start, current)
    if elapsed_period < 0:
      return elapsed_period
    else:
      return ChronoUnit.DAYS.between(current, end)


  def _checkCertificateStatus(self):
    checkName = "Check certificate status"
    SummaryReport = []
    #self.objToolResources.startProgressBar(checkName)
    try:
      certManager = VPlexCertRenewalMediator().getInstance()
      isCWS = VPlexCertRenewalData().isCWSConfigured()
      certificates = certManager.getAllCertificatesInfo(not isCWS)
      certInfo = dict((k, {'valid_days':self._getValidDays(v)}) for k, v in certificates.items())
      for cert in certInfo:
        for key in certInfo[cert]:
          daysRemaining = certInfo[cert][key]
          if cert == 'VPN':
            if daysRemaining < 0:
              SummaryReport.append(HOST_CERT_ERROR)
            elif daysRemaining < CERTIFICATE_EXP_DAYS:
              SummaryReport.append(HOST_CERT_WARNING)
            else:
              self.logit("VPN certificate on the management-server is valid.",verbose=True,altFile=True)
          if cert == 'WEB':
            if daysRemaining < 0:
              SummaryReport.append(WEB_CERT_ERROR)
            elif daysRemaining < CERTIFICATE_EXP_DAYS:
              SummaryReport.append(WEB_CERT_WARNING)
            else:
              self.logit("WEB certificate on the management-server is valid.",verbose=True,altFile=True)
          if cert == 'CA':
            if daysRemaining < 0:
              SummaryReport.append(CA_CERT_ERROR)
            elif daysRemaining < CERTIFICATE_EXP_DAYS:
              SummaryReport.append(CA_CERT_WARNING)
            else:
              self.logit("CA certificate on the management-server is valid.",verbose=True,altFile=True)
      if SummaryReport:
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      #self.objToolResources.stopProgressBar(error=False)
    except (Exception), e:
      #self.objToolResources.stopProgressBar(error=True)
      self.logit("Error occurred while checking the certificate status on the management-server."\
                 " Error: %s" %e,verbose=True,altFile=True)


  '''
        To check for below cases and if found report it.
        VS2
                6.0.1 P4 - 6.0.1 P7
                WAN COM using IPv6
        VS6
                6.0.1 P4 - 6.0.1 P7
                WAN COM using IPv6
  '''
  def _checkForIPV6UsageOnWAN(self):
    try:
      checkName = "Check for IPv6 on the WAN COM ports"
      SummaryReport = []
      b_print_header_once = True
      b_using_ipv6 = False
      result = yyapi.glob("/clusters/**/wan-com")
      for each_entry in result:
        com_groups = each_entry.getBackingObject()
        for each_port_grps in com_groups.getPortGroups():
          for port in each_port_grps.getMemberPortsFull(each_port_grps.MemberPortQuery.ALL):
            if ":" in str(port.getAddress()):
              b_using_ipv6 = True
              if b_print_header_once:
                  msg = "WARNING: The use of IPv6 has been detected. GeoSynchrony 6.0.x releases have\n" \
                          "not been qualified for VS2 to VS6 hardware upgrade (GenU) if IPv6 is used for\n" \
                          "management connectivity (on Local/Metro) or WAN COM connectivity.\n" \
                          "IPv6 address detected on the WAN COM port(s):\n\n"
                  self.logit(msg, verbose=True, altFile=True)
                  SummaryReport.append(msg)
                  b_print_header_once = False
                  self.logit('IPV6'.ljust(64) + 'WAN Port'.ljust(16), verbose=True, altFile=True)
                  SummaryReport.append('IPV6'.ljust(64) + 'WAN Port'.ljust(16))
                  self.logit('-' * 80, verbose=True, altFile=True)
                  SummaryReport.append('-' * 80)
              self.logit(str(port.getAddress()).ljust(64) + str(port.getName()).ljust(16), verbose=True, altFile=True)
              SummaryReport.append(str(port.getAddress()).ljust(64) + str(port.getName()).ljust(16))

      if b_using_ipv6:
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      else:
        msg = "IPv6 not detected on the WAN COM ports of this VPLEX."
        self.logit(msg,verbose=True, altFile=True)
    except (Exception), e:
      self.logit("Error checking if IPV6 is used for WAN COM for version 6.0.X.  Error: %s" %e,verbose=True)


  def _checkForIPv6OnSms(self):
    checkName = "Check for IPv6 on the management-server eth3 port"
    SummaryReport = []
    try:
      smsEth3Listing = yyapi.glob('/management-server/ports/eth3')
      ipv6Gateway = str(smsEth3Listing[0].getAttributes()[5].getValue())
      if ':' in ipv6Gateway:
        ipv6Address = str(smsEth3Listing[0].getAttributes()[4].getValue())
        msg = "WARNING: The use of IPv6 has been detected. GeoSynchrony 6.0.x releases have\n" \
        "not been qualified for VS2 to VS6 hardware upgrade (GenU) if IPv6 is used for\n" \
        "management connectivity (on Local/Metro) or WAN COM connectivity.\n" \
        "IPv6 address detected on the management-server eth3 port:\n %s\n" % (ipv6Address[27:78])
        self.logit(msg, verbose=True, altFile=True)
        SummaryReport.append(msg)
        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      else:
        self.logit("IPv6 not detected on the eth3 port of the management-server.",verbose=True, altFile=True)
    except (Exception), e:
      self.logit("Error checking the eth3 port of the sms for IPv6.  Error: %s" %e,verbose=True)


  def _checkForDownVs2FEPorts(self):
    checkName = "Check for 'down' VS2 FE Ports"
    downFEPortsList = []
    directorsWithDownFEPorts = []
    SummaryReport = []
    tPortBadSxportStateList = []
    badSxportState = 'Enabled: -'
    try:
      ports = self._getPorts()
      for p in ports:
        port = p.getBackingObject()
        if 'front-end' in str(port.getRole()):
          if 'down' in str(port.getPortStatus()):
            directorAndPort = port.getDirector().toString() + ' ' + port.getName()
            downFEPortsList.append(directorAndPort)
            directorWithDownFEPort = port.getDirector().toString()
            if directorWithDownFEPort not in directorsWithDownFEPorts:
              directorsWithDownFEPorts.append(directorWithDownFEPort)

      if downFEPortsList:
        msg = "The VS2 has the following FE ports in 'down' (disabled) state:\n"
        self.logit(msg, verbose=True, altFile=True)
        SummaryReport.append(msg)
        for p in downFEPortsList:
          msg = ("%s\n" % p)
          SummaryReport.append(msg)
          self.logit(msg, verbose=True, altFile=True)
        msg = "These need to be enabled prior to the GenU.  A CE should enable them in the\n" \
            "VPlexcli /engines/*/directors/*/hardware/ports context by using the 'set enabled true'\n" \
            "VPlexcli command.  Reference the Master Document for more details.\n" \
            "If the ports in question remain in down state thereafter engage the VPLEX\n" \
            "Remote Support team and ask for the CS Recovery Team to follow KBA 529368.\n\n"
        self.logit(msg, verbose=True, altFile=True)
        SummaryReport.append(msg)

        for director in directorsWithDownFEPorts:
          sxportShowdeviceOutput = yyapi.nsfwcmd(director, 'sxport showdevice', interactive = True)
          for line in iter(sxportShowdeviceOutput.splitlines()):
            if badSxportState in line:
              tPortLine = re.findall('(.*)(?:.*\n){2}.*Enabled: -', sxportShowdeviceOutput)
              if tPortLine not in tPortBadSxportStateList:
                tPortBadSxportStateList.append(tPortLine)
              break

        if tPortBadSxportStateList:
          msg = "The following ports were found to have a disabled sxport state:\n"
          self.logit(msg, verbose=True, altFile=True)
          SummaryReport.append(msg)
          for tPort in tPortBadSxportStateList:
            msg = ''.join(map(str, tPort))
            SummaryReport.append(msg)
            self.logit(msg, verbose=True, altFile=True)
          msg = "\nThese FE ports should be the same as noted above. Reference the Master Document\n" \
              "for more details on how to verify. Due to the disabled sxport state these ports\n" \
              "likely won't be able to be enabled via VPlexcli. A CE should attempt this using\n" \
              "the steps above, and if unable to then engage the VPLEX Remote Support team\n" \
              "and ask for the CS Recovery Team to follow KBA 529368.\n"
          self.logit(msg, verbose=True, altFile=True)
          SummaryReport.append(msg)

        self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)
      else:
        self.logit("All FE ports on the VS2 are enabled. No action needed.",verbose=True,altFile=True)
    except (Exception), e:
      self.logit("Error checking for 'down' FE ports on the VS2.  Error: %s" %e,verbose=True)



  def _cleanUp(self):
    cleanupFile = '/var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_UCT_AutoCleanup_Log.txt'
    if os.path.exists(cleanupFile):
      try:
        os.remove(cleanupFile)
      except OSError:
        pass
    cleanupLog = open(cleanupFile,'a')
    print >> cleanupLog, ("Starting the auto-cleanup function to clean up the tool package, script & individual logs...")
    libExtDir = osglob.glob('/opt/emc/VPlex/jython*/LibExt')
    scriptsToRemove = osglob.glob(libExtDir[0]+"/"+'*UpgradeChecker*')
    scriptDirsToRemove = [ 'FLAT','Documentation']
    logFilesToRemove = osglob.glob(LOG_DIR+"/"+'*')

    print >> cleanupLog, ("\nLibExt directory is: ", libExtDir)
    print >> cleanupLog, ("\nScripts being removed from the LibExt directory are: ", scriptsToRemove)
    try:
      [ os.remove(x) for x in ( scriptsToRemove ) if ( os.path.isfile(x)) ]
    except (Exception), e:
      print >> cleanupLog, ("Error removing the scripts in the LibExt directory: %s" % e)

    print >> cleanupLog, ("\nScript directories being removed from the LibExt directory are: ", scriptDirsToRemove)
    try:
      [ shutil.rmtree(libExtDir[0]+"/"+x) for x in ( scriptDirsToRemove ) if ( os.path.isdir(libExtDir[0]+"/"+x)) ]
    except (Exception), e:
      print >> cleanupLog, ("Error removing the script directories from the LibExt directory: %s" % e)

    print >> cleanupLog, ("\nRemoving log files in the VPLEX_Upgrade_Checker directory except the .tar file...")
    try:
     [ os.remove(x) for x in logFilesToRemove if os.path.isfile(x) and 'VPLEXUpgradeCheckerAnalysis' not in x and 'VPLEX_UCT_AutoCleanup_Log' not in x ]
    except (Exception), e:
      print >> cleanupLog, ("Error removing the individual log files in /var/log/VPlex/cli/VPLEX_Upgrade_Checker: %s" % e)
    print >> cleanupLog, ("\nAuto-cleanup function is complete.  If any errors have logged please manually remove")
    print >> cleanupLog, ("\nthe associated files/directories.")
    cleanupLog.close()




class redirect_stream(object):
  def __init__(self, new_target, stream='stdout'):
    self._new_target = new_target
    self._stream = stream
    self._old_targets = []

  def __enter__(self):
    self._old_targets.append(getattr(sys, self._stream))
    setattr(sys, self._stream, self._new_target)
    return self._new_target

  def __exit__(self, exctype, excinst, exctb):
    setattr(sys, self._stream, self._old_targets.pop())

class genuUpgradeData(upgradeCheck):
  def __init__(self):
    upgradeCheck.__init__(self)

  def getName(self):
    return "rungenu"

  def dumpConsGenu(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    capture = StringIO()
    save_stdout = sys.stdout
    sys.stdout = capture
    try:
      yyapi.cmd(cmd)
    finally:
      sys.stdout = save_stdout
    self.logit("%s" % capture.getvalue(),verbose=True,altFile=True)

  def dumpEntireGenu(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    try:
      inits=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmd).result)
      self.logit(inits,verbose=True,altFile=True)
    except(Exception), e:
      self.logit("Running: %s failed" % e,verbose=True )

  def _vpnStatusCheck(self):
    checkName = "Check VPN status"
    SummaryReport = []
    try:
      clusterType = self._getProductType()

      if clusterType == "LOCAL":
        msg = "This is a Vplex local system, skipping VPN status check.."
        self.logit(msg, verbose=True, altFile=True)

      elif clusterType == "METRO":
        #self.objToolResources.startProgressBar(checkName)
        vpnState = VPNState()

        if not vpnState.isVPNUpState():
          err_msg = "The VPN tunnel to the remote management-server is down. A Customer Engineer (CE)\n" \
                    "should work with the customer to follow KBA 448400 'VPLEX: After an IP network\n" \
                    "outage VPN is down' to troubleshoot the VPN connectivity issue.\n" \
                    "WARNING: In this case the required checks were not performed on the remote\n" \
                    "directors.  Please execute the tool separately on the remote management-server\n" \
                    "to ensure the required checks are performed on all directors.\n"

          #self.objToolResources.stopProgressBar(error=True)
          self.logit(err_msg, verbose=True, altFile=True)
          SummaryReport.append(err_msg)
        else:
          self.logit("VPN status is UP between the management servers", verbose=True, altFile=True)
          #self.objToolResources.stopProgressBar(error=False)
    except(Exception), e:
      #self.objToolResources.stopProgressBar(error=True)
      self.logit("Error while checking VPN status: %s" %e, verbose=True)
    if SummaryReport:
      self.objToolResources.addSummaryIssuesToReport(checkName, SummaryReport)

  def _execute_(self, shell, context, args):
    lockstat = self._lock(action='lock')
    if lockstat == 'Locked':
      self.logit("Another instance of the script is running. Exiting...",verbose=True)
      return
    self.objToolResources.importVplexUpgradeCheckerDict()
    self.logit("Starting VPlexcli checks for VS2 to VS6 GenU...",verbose=True,altFile=True)
    self.logit(self.reportbrk,verbose=True,altFile=True)
    time.sleep(2)
    try:
      vers = self._getVersion()
      plat = self._getVSx()
      if plat == "VS2":
        self.logit("Checking for non-standard WWNs on the VS2...",verbose=True,altFile=True)
        self._checkForVS2NonStandardWwns()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Checking the RX/TX power levels of the VPLEX ports...",verbose=True,altFile=True)
        self._checkSfpVoltage()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Checking the VPN, WEB and CA certificate validity...", verbose=True, altFile=True)
        self._checkCertificateStatus()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Checking the VPN status...", verbose=True, altFile=True)
        self._vpnStatusCheck()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Checking for 'down'(disabled) FE ports on the VS2...",verbose=True,altFile=True)
        self._checkForDownVs2FEPorts()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        director_version = self.objToolResources.getDirectorVersion()
        if '7.5.125.21.0' in director_version or '7.5.127.52.0' in director_version or '7.5.128.5.0' in director_version or '7.5.129.4.0' in director_version:
          self.logit("Checking for IPv6 on the eth3 port of the management-server...",verbose=True,altFile=True)
          self._checkForIPv6OnSms()
          self.logit("Checking for IPv6 on the WAN COM ports of this VPLEX...",verbose=True,altFile=True)
          self._checkForIPV6UsageOnWAN()
          self.logit(self.reportbrk,verbose=True,altFile=True)
        else:
          self.logit("Version is not a GeoSynchrony 6.0.x release, skipping the check for IPv6...",verbose=True,altFile=True)
          self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Collecting the output of the commands required for the GenU CCA Process for a VS2.",verbose=True,altFile=True)
        self.logit("NOTE: Some commands may fail to run, such as 'vpn status' on a VPLEX Local.",verbose=True,altFile=True)
        self.logit(self.reportbrk,verbose=True,altFile=True)
        vs2List = [ 'ls -t /management-server/ports/eth3::address', 'ls -t /management-server::cluster-ip-seed',
                    'ls -t /engines/*::top-level-assembly ','cluster summary', 'version -a','health-check',
                    'll /clusters/*/exports/initiator-ports','director uptime','date -u','health-check --full --verbose',
                    'll /engines/**/ports','ll /engines/**/sfps','ll /recoverpoint/rpa-clusters/*',
                    'll /clusters/**/consistency-groups/','opm initiator-port show-mode -i *' ]
        vs2NoShowList = ['vpn status','rp validate-configuration','security list-certificates','script -i hardwareUpgrade',
                          'hardware-upgrade hw-precheck' ]
        for cmd in vs2List:
          try:
            self.dumpEntireGenu(cmd)
          except (Exception), e:
            self.logit("Command %s failed to execute all cmds for the GENU cmd collection. Error: %s." % (cmd,e) ,verbose=True)
        for cmd in vs2NoShowList:
          try:
            self.dumpConsGenu(cmd)
          except (Exception), e:
            self.logit("Command %s failed to execute all cmds for the GENU cmd collection. Error: %s." % (cmd,e) ,verbose=True)
        nduPreCheck = ('ndu pre-check --verbose')
        try:
          self._onlyRunNduCmd(nduPreCheck)
        except (Exception), e:
          self.logit("Command %s failed to execute. Error: %s." % (cmd,e) ,verbose=True)
        time.sleep(2)
        self._appendNduPreCheck()
        self.logit("GenU command output collection is complete.",verbose=True,altFile=True)
      elif plat == "VS6":
        self.logit("Checking for enabled WAN COM ports in 'up' state on the VS6...",verbose=True,altFile=True)
        #can't check product type on unconf VS6 so will check all VS6 systems
        self._checkUpVs6WanPorts()
        self.logit(self.reportbrk,verbose=True,altFile=True)
        director_version = self.objToolResources.getDirectorVersion()
        if '7.5.127.52.0' in director_version or '7.5.128.5.0' in director_version or '7.5.129.4.0' in director_version:
          self.logit("Checking for disabled WAN COM ports on the VS6...",verbose=True,altFile=True)
          #can't check product type on unconf VS6 so will check all VS6 systems
          self._checkDisabledVs6WanPorts()
          self.logit(self.reportbrk,verbose=True,altFile=True)
        else:
          self.logit("Director version is not 6.0.1 P5 - 6.0.1 P7, skipping check for disabled VS6 WAN COM ports.",verbose=True,altFile=True)
          self.logit(self.reportbrk,verbose=True,altFile=True)
        #Only need to check for IPv6 on GeoSynchrony 6.0.x releases
        if '7.5.125.21.0' in director_version or '7.5.127.52.0' in director_version or '7.5.128.5.0' in director_version or '7.5.129.4.0' in director_version:
          self.logit("Checking for IPv6 on the eth3 port of the management-server...",verbose=True,altFile=True)
          self._checkForIPv6OnSms()
          self.logit(self.reportbrk,verbose=True,altFile=True)
        else:
          self.logit("Version is not a GeoSynchrony 6.0.x release, skipping the check for IPv6...",verbose=True,altFile=True)
          self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Collecting the VPlexcli command output required for the GenU CCA process for a VS6...",verbose=True,altFile=True)
        self.logit(self.reportbrk,verbose=True,altFile=True)
        vs6List = ['version -a']
        for cmd in vs6List:
          try:
            self.dumpEntireGenu(cmd)
          except (Exception), e:
            self.logit("Command %s failed to execute the GenU cmd collection. Error: %s." % (cmd,e) ,verbose=True)
        self.logit("WARNING: The VS2 and VS6 need to be running the same GeoSynchrony version prior to the GenU",verbose=True,altFile=True)
        self.logit("WARNING: Please ensure all of the directors are connected to VPlexcli prior to the GenU",verbose=True,altFile=True)
        self.logit("GenU command output collection is complete.",verbose=True,altFile=True)
      else:
        self.logit("Not performing GENU command collection for VS1.  Exiting...",verbose=True)
        pass
      self.objToolResources.printIssuesSummary(plat,vers)
      self.logit("Debug logging is in file: %s" % LOG_FILE_FINAL, verbose=True)
      self.logit("GenU pre-check analysis, Issues Summary report, and GenU CCA Process command output collection is in file: %s" % self.C_LOG_FILE_FINAL, verbose=True)
      self.logit("For a VS2 the FLAT analysis is in file /var/log/VPlex/cli/VPLEX_Upgrade_Checker/FirmwareReportSummary.txt",verbose=True)
      self.logit("Logs are in tar file /var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEXUpgradeCheckerAnalysis_<TLA>_<time>.tar.gz.",verbose=True)
      self.logit("Please copy this .tar file off the management-server, and then delete it.",verbose=True)
      self.logit(self.reportbrk,verbose=True,altFile=True)
      self.logit("The tool will now automatically remove the scripts & individual log files (except the .tar file).",verbose=True)
      self.logit("Please check /var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_UCT_AutoCleanup_Log.txt to verify it completes.",verbose=True)
      self.logit(self.reportbrk,verbose=True,altFile=True)
    except (Exception), e:
      self.logit("Fail in _execute_ %s." % e ,verbose=True)
      self._lock(action='unlock')
      self._reNamereMove(C_LOG_FILE,self.C_LOG_FILE_FINAL)
      self._reNamereMove(LOG_FILE,LOG_FILE_FINAL)
      time.sleep(2)
      self._tarLogs()
      return
    self._lock(action='unlock')
    time.sleep(2)
    self._reNamereMove(C_LOG_FILE,self.C_LOG_FILE_FINAL)
    self._reNamereMove(LOG_FILE,LOG_FILE_FINAL)
    time.sleep(2)
    self._tarLogs()
    self._cleanUp()


class nduUpgradeData(upgradeCheck):
  cmd_options = [
    yyapi.Option("checks", "checks to run", short="c", argname='string', list=True, required=False),
    yyapi.Option("cca", "commands to collect for CCA", short="a", argname='string', required=False),
  ]

  def __init__(self):
    upgradeCheck.__init__(self)

  def getName(self):
    return "runnducheck"

  def dumpConsNdu(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    capture = StringIO()
    save_stdout = sys.stdout
    sys.stdout = capture
    try:
      yyapi.cmd(cmd)
      self.logit("%s" % capture.getvalue(),verbose=True,altFile=True)
    finally:
      sys.stdout = save_stdout

  def dumpEntireNduCmds(self,cmd):
    self.logit("Running: %s" % cmd,verbose=True,altFile=True)
    try:
      inits=MainContextFactory.FORMATTER_RENDERER.format(yyapi.cmd(cmd).result)
      self.logit(inits,verbose=True,altFile=True)
    except(Exception), e:
      self.logit("Running: %s failed" % e,verbose=True )

  def execute_memory_fragmentation(self):
    self.logit(self.reportbrk,verbose=True,altFile=True)
    self.logit("Checking for Memory Fragmentation on directors...",verbose=True,altFile=True)
    self._checkForMemoryFragmentation()

  def execute_sfp_voltage(self):
    self.logit(self.reportbrk,verbose=True,altFile=True)
    self.logit("Checking the RX/TX power levels of the VPLEX ports...",verbose=True,altFile=True)
    self._checkSfpVoltage()

  def execute_certificate_status(self):
    self.logit(self.reportbrk,verbose=True,altFile=True)
    self.logit("Checking the VPN, WEB and CA certificate validity...", verbose=True, altFile=True)
    self._checkCertificateStatus()

  def execute_checks(self):
    self.logit("Starting the VPlexcli checks required for a VPLEX NDU...",verbose=True,altFile=True)
    if self.checks:
        for checks in self.checks:
            if checks == "checkForMemoryFragmentation":
                self.execute_memory_fragmentation()
            elif checks == "checkSfpVoltage":
                self.execute_sfp_voltage()
            elif checks == "checkCertificateStatus":
                self.execute_certificate_status()
    else:
        self.execute_memory_fragmentation()
        self.execute_sfp_voltage()
        self.execute_certificate_status()

  def set_nduList(self, nduList):
    if self.cca == "DayOfUpgrade":
      nduList = [ 'ls -t /engines/*::top-level-assembly', 'll /management-server', 'version -a ',
                'll /cluster-witness/', 'll /recoverpoint/rpa-clusters', 'health-check --full --verbose' ]

    else:
      nduList = [ 'ls -t /management-server/ports/eth3::address','ls -t /management-server::cluster-ip-seed',
                'ls -t /engines/*::top-level-assembly','cluster summary', 'version -a','health-check',
                'll /clusters/*/exports/initiator-ports','director uptime','date -u', 'health-check --full --verbose',
                'll /engines/**/ports','ll /engines/**/sfps', 'll /recoverpoint/rpa-clusters/*',
                'll /clusters/**/consistency-groups/','opm initiator-port show-mode -i *', 'll /clusters' ]
    return nduList

  def set_nduNoShowList(self, nduNoShowList):
      nduNoShowList = [ 'vpn status','rp validate-configuration' ]
      return nduNoShowList

  def run_svcommands(self):
      if self.cca == "DayOfUpgrade": return
      engine = self._isItCluster1()
      clusterName = self._getClusterName(engine)
      sViewFindCmd = ('export storage-view find --free-lun -c %s' % clusterName)
      try:
          self.dumpConsNdu(sViewFindCmd)
      except (Exception), e:
          self.logit("Command %s failed to execute. Error: %s." % (cmd,e) ,verbose=True)

  def run_collectCommands(self, nduList, nduNoShowList):
      for cmd in nduList:
          try:
              self.dumpEntireNduCmds(cmd)
          except(Exception), e:
              self.logit("Command %s failed. Error: %s." % (cmd,e) ,verbose=True)

      for cmd in nduNoShowList:
          try:
              self.dumpConsNdu(cmd)
          except (Exception), e:
              self.logit("Command %s failed. Error: %s." % (cmd,e) ,verbose=True)

  def run_nduPreCheck(self):
      nduPreCheck = ('ndu pre-check --verbose')
      if self.cca == "DayOfUpgrade":
          nduPreCheck = ('ndu pre-check')
      try:
          self._onlyRunNduCmd(nduPreCheck)
      except (Exception), e:
          self.logit("Command %s failed to execute. Error: %s." % (cmd,e) ,verbose=True)
      time.sleep(2)
      self._appendNduPreCheck()


  def _execute_(self, shell, context, args):
    lockstat = self._lock(action='lock')
    if lockstat == 'Locked':
      self.logit("Another instance of the script is running. Exiting...",verbose=True)
      return
    # To run check(s) based on the user input.
    self.execute_checks()

    if self.cca:
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Starting NDU VPlexcli command collection...",verbose=True,altFile=True)
        self.logit(self.reportbrk,verbose=True,altFile=True)
        self.logit("Collecting the output of the requested VPlexcli commands...",verbose=True,altFile=True)
        self.logit("NOTE: Some commands may fail to run, such as 'vpn status' on a VPLEX Local.",verbose=True,altFile=True)
        self.logit(self.reportbrk,verbose=True,altFile=True)
        time.sleep(2)

        # To deside on the the vplexcli commands to be collected based on the user input.
        nduList = self.set_nduList(list())
        nduNoShowList = self.set_nduNoShowList(list())

        # To collect the vplexcli commands.
        self.run_collectCommands(nduList, nduNoShowList)
        self.run_svcommands()
        self.run_nduPreCheck()

        self.logit("NDU CCA VPlexcli command output collection is complete.",verbose=True,altFile=True)
    try:
      vers = self._getVersion()
      plat = self._getVSx()
    except (Exception), e:
      self.logit("Failed getting version or platform due to error: %s." % e ,verbose=True)
    self.objToolResources.printIssuesSummary(plat,vers)
    self.logit("Debug logging is in file: %s" % LOG_FILE_FINAL, verbose=True)
    self.logit("NDU pre-check analysis, Issues Summary report, and NDU CCA command output collection is in file: %s" % self.C_LOG_FILE_FINAL, verbose=True)
    if self.cca != "DayOfUpgrade":
      self.logit("For SolVeCCAReview FLAT analysis is in file /var/log/VPlex/cli/VPLEX_Upgrade_Checker/FirmwareReportSummary.txt",verbose=True)
    self.logit("Logs are in tar file /var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEXUpgradeCheckerAnalysis_<TLA>_<time>.tar.gz.",verbose=True)
    self.logit("Please copy this .tar file off the management-server, and then delete it.",verbose=True)
    self.logit(self.reportbrk,verbose=True,altFile=True)
    self.logit("The tool will now automatically remove the scripts & individual log files (except the .tar file).",verbose=True)
    self.logit("Please check /var/log/VPlex/cli/VPLEX_Upgrade_Checker/VPLEX_UCT_AutoCleanup_Log.txt to verify it completes.",verbose=True)
    self.logit(self.reportbrk,verbose=True,altFile=True)
    self._lock(action='unlock')
    time.sleep(2)
    self._reNamereMove(C_LOG_FILE,self.C_LOG_FILE_FINAL)
    self._reNamereMove(LOG_FILE,LOG_FILE_FINAL)
    time.sleep(2)
    self._tarLogs()
    self._cleanUp()
    return



nduUpgradeData.instance = nduUpgradeData()
genuUpgradeData.instance = genuUpgradeData()
